/************************************************************************************/
/*																				  	*/
/*	NAME:		adjusted_crossvalidation.mac.sas									*/
/*  TITLE: 		Adjustierte Crossvalidierung										*/
/*  KEYS: 		crossvalidation, adjusted crossvalidation, validation, 				*/
/*			 	logistic regression, roc analysis									*/
/*	VERSION:	SAS 8.2																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		LOGISTIC, MEANS														*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\adjusted_crossvalidation.mac.sas	*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*	Programmierer:					Christoph Ziegler								*/
/*	Datum der Ersterstellung:		8/2003											*/
/*  Aktuelle Makro- Version:		12/2003											*/
/*																					*/
/*	Kurzbeschreibung:				Allgemeines Makro zur Validierung von 			*/
/*							    	logistischen Regressionsmodellen mittels 		*/
/*									adjustierter Crossvalidierung					*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_CROSSVALIDATION.MAC.SAS						*/
/*																					*/
/*  Genauere Beschreibung siehe:	PM_CROSSVALIDATION.MAC.SAS						*/
/*									"Bootstrap methods and their application", 		*/
/*									Davison/ Hinkley, S. 294/295					*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*									pm_logreg.mac.sas								*/
/*									pm_roc.mac.sas									*/
/*																					*/
/*	Bentigte Dateien:				Datensatz (Parameter 'data=')					*/
/*																					*/
/*																					*/
/*  Erzeugte Dateien:				-----											*/
/*																					*/
/*  Programmnderungen				Sandra Mller, 04.05.2016, Makrovariable		*/
/*  und -ergnzungen 				binwidth hinzugefgt							*/ 
/*  (Name, Datum, Beschreibung):											 		*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%MACRO ADJUSTED_CROSSVALIDATION		 (	data				= ,
										resp_var			= , 
										event				= 1,
										xvar				= ,
										cvar				= ,
										ref					= ,
										miss 				= 0,
										macro_path			= ,
										gruppen				= 10,
										anzahl				= 50,
										binwidth			= 0
	       						   );

/*Externe Makros, die innerhalb des adjustierten Crossvalidierungs- Makros verwendet werden*/
/*(Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('pm_logreg.mac.sas');
%INCLUDE y ('pm_roc.mac.sas');

%GLOBAL nnnn1;
%LET datensatzxxx=&data;

%LET  datensatz=&data;
/*Output unterdrcken:*/

ODS SELECT NONE;

/*Logistische Regression ber den Ausgangsdatensatz*/

%PM_LOGREG ( 	data			=&data,
				resp_var		=&resp_var,
				event			=&event,
				xvar			=&xvar,
				cvar			=&cvar,
				ref				=&ref,
				fxvar			=,
				fcvar			=,
				miss			=&miss,
				sel				=NONE,
				sel_details		= 1,
				slstay			=0.05,
				slentry			=0.05,
				clodds			=0,
				alpha			=0.05,
				rsquare			=1,
				ctable			=0,
				ctable_pprob	=,
				out_pred		=work.gof_ori,
				pred			=1,
				predprobs_art	=INDIVIDUAL,
				roc				=1,
				out_roc			=work.roc_ori,
				est				=0,
				out_est			=,
				inest_flag		=0,
				inest_data		=,
				boot_data		=,
				c_wert			=1,
				c_data			=work.association_ori,
				test			=,
				mi				=0,
				out_mi			=work.out_mi_log,
				macro_path		=&macro_path,
				binwidth		=&binwidth
		);

/*ROC-Analyse (Originaldatensatz) mit Makro %ROC_KURVE*/
	
%PM_ROC			(
				 data				=&data,
				 resp_var			=&resp_var,
				 event				=&event,
				 data_roc			=work.roc_ori,
				 data_pred			=work.gof_ori,
				 data_asso			=work.association_ori,
				 validation			=CROSSVALIDATION,
				 praevalenz			= ,
				 all_cp				=0,
			     cp					=,
				 cibinom			=0,
				 rocdat				=0,
				 auroc				=0,
				 printit			=0,
				 alpha_cl			=0.95,
				 ext				=tiffp,
				 winkel				=1,
				 point				=1,
				 path				='c:\temp',
				 mi					=0,
				 out_mi				=work.out_mi_roc,
				 macro_path			=&macro_path,
				 binwidth			=&binwidth
			    );

/*Variablen umbenennen:*/
/*ROC- / Sonstige Kenngren des Originaldatensatzes zusammenfassen*/

DATA result_roc_ori1;
	SET result;
	RENAME maxyouden 	= maxyouden_ori;
	RENAME _PROB_ 		= prob_ori;
	RENAME _SENSIT_		= sens_ori;
	RENAME spez 		= spez_ori;
	RENAME ppv 			= ppv_ori;
	RENAME npv 			= npv_ori;
	RENAME psep 		= psep_ori;
	dummy				= 1;
RUN;

DATA result_roc_ori2;
	SET index_ori;
	dummy				= 1;
	emax_ori			= 0;
	slope_ori			= 1;
RUN;

/*Cutpoint des Ausgangsdatensatzes zur spteren Verwendung in einer Makrovariable */
/*zwischenspeichern*/

DATA result_roc_ori1;
	SET result_roc_ori1;
	call symput('prob_original',prob_ori);
RUN;

/*Anzahl der stetigen Variablen ermitteln:*/
%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The xvar contains &count_x variables.;

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count_c var;
%LET count_c=1;
%LET var=%QSCAN(&cvar,&count_c,%STR( ));
%DO %WHILE(&var ne);
     %LET count_c=%EVAL(&count_c+1);
     %LET var=%QSCAN(&cvar,&count_c,%STR( ));
%END;
%LET count_c=%EVAL(&count_c-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The cvar contains &count_c variables.;

/*Elimination aller missing values falls miss=0 angegeben wurde*/

%IF &miss=0 %THEN
	%DO;

		DATA work.nomissing;
		SET  &data;
		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count_c;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

		RUN;
%LET data=nomissing; 
%END;

/*Anzahl der Beobachtungen im Datensatz bestimmen*/

%NUMBEROBS(datensatz=&data);

%LET anz_beob=&nnnn1;

/*Davon die Anzahl der Beobachtungen bestimmen, die zur Validierung herangezogen werden sollen*/
/*(in Abhngigkeit der angegebenen Gruppenzahl)*/

%LET anz_val=%SYSEVALF(&anz_beob/&gruppen);
%PUT &anz_val;

/*runden*/

%LET anz_val=%SYSEVALF(&anz_val, floor);
%PUT &anz_val;

/*Gruppenstrke der letzen Gruppe bestimmen (Restgruppe)*/

%LET rest = %EVAL(&anz_beob-(&gruppen*&anz_val));
%PUT &rest;

/*Ergebnis im Log-Fenster ausgeben*/

%PUT Ergebnis: Gruppenzahl = &gruppen . Gruppenstrke = &anz_val;
%PUT Rest (letzte Gruppe) = &rest;

/*Makro- Gruppen- Variablen anlegen und Ausgangsgruppenstrke zuweisen*/

%DO l=1 %TO &gruppen;
	%LET gruppe&l= %EVAL(&anz_val);
%END;

/*Falls Rest existiert, Beobachtungen auf existierende Gruppen aufteilen*/

%LET resttest=&rest;

/*Falls Rest existiert:*/

%IF &rest ne 0 %THEN %DO;

	/*Schleife von 1 bis Anzahl Rest*/

	%DO k=1 %TO &rest;
		
		/*Innere Schleife von 1 bis Anzahl der Gruppen*/
		%DO j=1 %TO &gruppen;
			
			/*Falls noch Rest zu vergeben ist:*/
			/*Gruppenstrke um 1 erhhen*/
			%IF &resttest > 0 %THEN %DO;
				%LET gruppe&j= %EVAL(&&gruppe&j + 1);
				%PUT &&gruppe&j;
			%END;

			/*Falls kein Rest mehr zu vergeben ist:*/
			/*Gruppenstrke bleibt gleich*/
			%ELSE %DO;
				%LET gruppe&j= %EVAL(&&gruppe&j);
				%PUT &&gruppe&j;
			%END;

			/*Rest um eine Beobachtung reduzieren (wurde einer Gruppe zugeteilt)*/
			%LET resttest=%EVAL(&resttest - 1);
		%END;

		/*Falls alle Restbeobachtungen aufgeteilt wurden: Ende der Schleife*/
		%IF &resttest <= 0 %THEN %DO;
		
		%GOTO ende;

		%END;
	%END;
		
%END;

%ende:

/*Anteile jeder Gruppe als Makro- Variablen speichern*/

%LET onepercent=%SYSEVALF(&anz_beob/100);

%DO hg=1 %TO &gruppen;


%LET anteil&hg=%SYSEVALF((&&gruppe&hg/&onepercent)/100);
%PUT &&anteil&hg;

%END;

/*Ergebnisdatei fr Kenngren null-initialisieren*/

DATA k_kenn3; SET _NULL_;
DATA k_kenn4; SET _NULL_;

DATA k_kenn3a; SET _NULL_;
DATA k_kenn4a; SET _NULL_;

/*Schleife (Anzahl der Durchlufe)*/

%DO q=1 %TO &anzahl;

%LET maxyouden_sum&q=0;
%LET prob_sum&q=0;
%LET sens_sum&q=0;
%LET spez_sum&q=0;
%LET ppv_sum&q=0;
%LET npv_sum&q=0;
%LET psep_sum&q=0;

%LET brier_sum&q=0;
%LET slope_sum&q=0;
%LET emax_sum&q=0;
%LET max_resca_sum&q=0;
%LET r_square_sum&q=0;
%LET auc_sum&q=0;
%LET som_sum&q=0;

/*Datensatz in Gruppen aufteilen (In Abhngigkeit der ermittelten Gruppenstrken)*/

/*neuer Zwischendatensatz anlegen, neue Beobachtungsnummer einfhren*/

DATA groups;
	SET &data;
	obs_vwxyz=_N_;
RUN;

/*Ergebnisdatei null-initialisieren*/

DATA ergebnis; SET _NULL_;

/*Schleife ber jede Gruppe (Gruppenanzahl)*/

%DO h=1 %TO &gruppen;

	/*Funktion fr Ziehen ohne Zurcklegen*/

	DATA group&h (drop=i j count);
		count=0;
		array sel_obs(&&gruppe&h) _TEMPORARY_;
    	DO i=1 TO &&gruppe&h;	    
    		redo: select=CEIL(RANUNI(0)*total);
			DO j=1 TO count;
				IF sel_obs(j)=select THEN GOTO redo;
			END;
			SET groups POINT=select NOBS=total;
			count=count+1;
			sel_obs(count)=select;
			OUTPUT;
		END;
		STOP;
	RUN;

	/*Jede Gruppendatei nach der neue Beobachtungsnummer sortieren*/

	PROC SORT DATA=group&h;
		BY obs_vwxyz;
	RUN;

	/*Speichern, welche Beobachtungen ausgewhlt wurden*/

	DATA group&h;
		SET group&h;
		CALL SYMPUT('names'||left(_n_), obs_vwxyz);
	RUN;

	/*Diese lschen, damit sie in der nchsten Gruppe nicht mehr enthalten sein knnen*/

	DATA groups;
		SET groups group&h;
	
		%DO g=1 %TO &&gruppe&h;

		IF obs_vwxyz = &&names&g THEN DELETE;

		%END;
	RUN;

	/*Neue Spalte einfhren, die die Gruppennummer enthlt*/

	DATA group&h;
		SET group&h;
		groupnumber=&h;
	RUN;

	/*An Ergebnisdatei anhngen*/

	DATA ergebnis;
		SET ergebnis group&h;
	RUN;

	/*Ergebnisdatei wieder sortieren*/

	PROC SORT DATA=ergebnis;
		BY obs_vwxyz;
	RUN;
	
	/*einzelne Gruppen-Zwischendateien lschen*/

	PROC DATASETS NOLIST;
		DELETE group&h;
	RUN;
	QUIT;

	/*Log- Fenster lschen (Schutz vor berlauf)*/

	DM "clear log;";

%END;

/*K-Fold- Crossvalidierung durchfhren:*/

/*Schleife (ber jede Gruppe)*/

%DO a=1 %TO &gruppen;

	/*Output unterdrcken:*/

	ODS SELECT NONE;

	/*jeweils eine Gruppe rausnehmen (wird zur Validierung herangezogen)*/
	/*mit dem Rest wird das Modell gebildet*/

	DATA auswert;
		SET ergebnis;
		IF groupnumber=&a THEN DELETE;
	RUN;

	DATA validate;
		SET ergebnis;
		IF groupnumber ne &a THEN DELETE;
	RUN;

	/*logistische Regression ---> outest*/

	%PM_LOGREG( data			=auswert,
				resp_var		=&resp_var,
				event			=&event,
				xvar			=&xvar,
				cvar			=&cvar,
				ref				=&ref,
				fxvar			=,
				fcvar			=,
				miss			=1,
				sel				=NONE,
				sel_details		= 1,
				slstay			=0.05,
				slentry			=0.05,
				clodds			=0,
				alpha			=0.05,
				rsquare			=1,
				ctable			=0,
				ctable_pprob	=,
				out_pred		=,
				pred			=0,
				predprobs_art	=INDIVIDUAL,
				roc				=0,
				out_roc			=,
				est				=1,
				out_est			=work.out_est,
				inest_flag		=0,
				inest_data		=,
				boot_data		=,
				c_wert			=0,
				c_data			=,
				test			=,
				mi				=0,
				out_mi			=work.out_mi_log,
				macro_path		=&macro_path,
				binwidth		=&binwidth
		);

		/*logistische Regression ---> inest*/

	%PM_LOGREG( data			=validate,
				resp_var		=&resp_var,
				event			=&event,
				xvar			=&xvar,
				cvar			=&cvar,
				ref				=&ref,
				fxvar			=,
				fcvar			=,
				miss			=1,
				sel				=NONE,
				sel_details		= 1,
				slstay			=0.05,
				slentry			=0.05,
				clodds			=0,
				alpha			=0.05,
				rsquare			=1,
				ctable			=0,
				ctable_pprob	=,
				out_pred		=work.gof,
				pred			=1,
				predprobs_art	=INDIVIDUAL,
				roc				=1,
				out_roc			=work.roc,
				est				=0,
				out_est			=,
				inest_flag		=2,
				inest_data		=work.out_est,
				boot_data		=,
				c_wert			=1,
				c_data			=work.association,
				test			=,
				mi				=0,
				out_mi			=work.out_mi_log,
				macro_path		=&macro_path,
				binwidth		=&binwidth
		);

		/*Kenngren bestimmen*/

%PM_ROC			(
				 data				=validate,
				 resp_var			=&resp_var,
				 event				=&event,
				 data_roc			=work.roc,
				 data_pred			=work.gof,
				 data_asso			=work.association,
				 validation			=CROSSVALIDATION,
				 praevalenz			= ,
				 all_cp				=0,
			     cp					=,
				 cibinom			=0,
				 rocdat				=0,
				 auroc				=0,
				 printit			=0,
				 alpha_cl			=0.95,
				 ext				=tiffp,
				 winkel				=1,
				 point				=1,
				 path				='c:\temp',
				 mi					=0,
				 out_mi				=work.out_mi_roc,
				 macro_path			=&macro_path,
				 binwidth			=&binwidth
			    );

		  DATA k_kenn1;
			SET result;
				RENAME maxyouden 	= maxyouden_k;
				RENAME _PROB_ 		= prob_k;
				RENAME _SENSIT_		= sens_k;
				RENAME spez 		= spez_k;
				RENAME ppv 			= ppv_k;
				RENAME npv 			= npv_k;
				RENAME psep 		= psep_k;
				dummy				= 1;
			RUN;

		 DATA k_kenn3;
		 	SET k_kenn3 k_kenn1;
		 RUN;

		  DATA k_kenn2;
			SET index_ori;
				RENAME brier_ori		= brier_k;
				RENAME slope_ori		= slope_k;
				RENAME emax_ori			= emax_k;
				RENAME max_resca_ori	= max_resca_k;
				RENAME r_square_ori		= r_square_k;
				RENAME auc_ori			= auc_k;
				RENAME som_ori			= som_k;
				dummy					= 1;
			RUN;

		 DATA k_kenn4;
		 	SET k_kenn4 k_kenn2;
		 RUN;


		/*darberhinaus: Datensatz ohne Gruppe k auf den Ausgangsdatensatz anwenden*/

		/*logistische Regression ---> inest*/

	%PM_LOGREG( data			=&data,
				resp_var		=&resp_var,
				event			=&event,
				xvar			=&xvar,
				cvar			=&cvar,
				ref				=&ref,
				fxvar			=,
				fcvar			=,
				miss			=1,
				sel				=NONE,
				sel_details		= 1,
				slstay			=0.05,
				slentry			=0.05,
				clodds			=0,
				alpha			=0.05,
				rsquare			=1,
				ctable			=0,
				ctable_pprob	=,
				out_pred		=work.gof,
				pred			=1,
				predprobs_art	=INDIVIDUAL,
				roc				=1,
				out_roc			=work.roc,
				est				=0,
				out_est			=,
				inest_flag		=2,
				inest_data		=work.out_est,
				boot_data		=,
				c_wert			=1,
				c_data			=work.association,
				test			=,
				mi				=0,
				out_mi			=work.out_mi_log,
				macro_path		=&macro_path,
				binwidth		=&binwidth
		);

		/*Kenngren bestimmen*/

%PM_ROC			(
				 data				=&data,
				 resp_var			=&resp_var,
				 event				=&event,
				 data_roc			=work.roc,
				 data_pred			=work.gof,
				 data_asso			=work.association,
				 validation			=CROSSVALIDATION,
				 praevalenz			= ,
				 all_cp				=0,
			     cp					=,
				 cibinom			=0,
				 rocdat				=0,
				 auroc				=0,
				 printit			=0,
				 alpha_cl			=0.95,
				 ext				=tiffp,
				 winkel				=1,
				 point				=1,
				 path				='c:\temp',
				 mi					=0,
				 out_mi				=work.out_mi_roc,
				 macro_path			=&macro_path,
				 binwidth			=&binwidth
			    );

		  DATA k_kenn1a;
			SET result;
				RENAME maxyouden 	= maxyouden_ka;
				RENAME _PROB_ 		= prob_ka;
				RENAME _SENSIT_		= sens_ka;
				RENAME spez 		= spez_ka;
				RENAME ppv 			= ppv_ka;
				RENAME npv 			= npv_ka;
				RENAME psep 		= psep_ka;
				dummy				= 1;
				runde				= &q;
			RUN;

		DATA k_kenn1;
			SET k_kenn1a;
				CALL symput('maxyouden',maxyouden_ka);
				CALL symput('prob',prob_ka);
				CALL symput('sens',sens_ka);
				CALL symput('spez',spez_ka);
				CALL symput('ppv',ppv_ka);
				CALL symput('npv',npv_ka);
				CALL symput('psep',psep_ka);
		RUN;

		 DATA k_kenn3a;
		 	SET k_kenn3a k_kenn1a;
		 RUN;

		  DATA k_kenn2a;
			SET index_ori;
				RENAME brier_ori		= brier_ka;
				RENAME slope_ori		= slope_ka;
				RENAME emax_ori			= emax_ka;
				RENAME max_resca_ori	= max_resca_ka;
				RENAME r_square_ori		= r_square_ka;
				RENAME auc_ori			= auc_ka;
				RENAME som_ori			= som_ka;
				dummy					= 1;
				runde					= &q;
			RUN;

		DATA k_kenn2a;
			SET k_kenn2a;
				CALL symput('brier',brier_ka);
				CALL symput('slope',slope_ka);
				CALL symput('emax',emax_ka);
				CALL symput('max_resca',max_resca_ka);
				CALL symput('r_square',r_square_ka);
				CALL symput('auc',auc_ka);
				CALL symput('som',som_ka);
		RUN;

		 DATA k_kenn4a;
		 	SET k_kenn4a k_kenn2a;
		 RUN;



		/*Output wieder aktivieren*/

		ODS SELECT ALL;

		DM "clear log;";

		%LET maxyouden_sum		=%SYSEVALF(&maxyouden*&&anteil&a);
		%LET prob_sum			=%SYSEVALF(&prob*&&anteil&a);
		%LET sens_sum			=%SYSEVALF(&sens*&&anteil&a);
		%LET spez_sum			=%SYSEVALF(&spez*&&anteil&a);
		%LET ppv_sum			=%SYSEVALF(&ppv*&&anteil&a);	
		%LET npv_sum			=%SYSEVALF(&npv*&&anteil&a);
		%LET psep_sum			=%SYSEVALF(&psep*&&anteil&a);

		%LET brier_sum			=%SYSEVALF(&brier*&&anteil&a);
		%LET slope_sum			=%SYSEVALF(&slope*&&anteil&a);
		%LET emax_sum			=%SYSEVALF(&emax*&&anteil&a);
		%LET max_resca_sum		=%SYSEVALF(&max_resca*&&anteil&a);
		%LET r_square_sum		=%SYSEVALF(&r_square*&&anteil&a);
		%LET auc_sum			=%SYSEVALF(&auc*&&anteil&a);
		%LET som_sum			=%SYSEVALF(&som*&&anteil&a);
		%PUT &som_sum;

		%LET maxyouden_sum&q		=%SYSEVALF(&&maxyouden_sum&q+&maxyouden_sum);
		%LET prob_sum&q				=%SYSEVALF(&&prob_sum&q+&prob_sum);
		%LET sens_sum&q				=%SYSEVALF(&&sens_sum&q+&sens_sum);
		%LET spez_sum&q				=%SYSEVALF(&&spez_sum&q+&spez_sum);
		%LET ppv_sum&q				=%SYSEVALF(&&ppv_sum&q+&ppv_sum);
		%LET npv_sum&q				=%SYSEVALF(&&npv_sum&q+&npv_sum);
		%LET psep_sum&q				=%SYSEVALF(&&psep_sum&q+&psep_sum);

		%LET brier_sum&q			=%SYSEVALF(&&brier_sum&q+&brier_sum);
		%LET slope_sum&q			=%SYSEVALF(&&slope_sum&q+&slope_sum);
		%LET emax_sum&q				=%SYSEVALF(&&emax_sum&q+&emax_sum);
		%LET max_resca_sum&q		=%SYSEVALF(&&max_resca_sum&q+&max_resca_sum);
		%LET r_square_sum&q			=%SYSEVALF(&&r_square_sum&q+&r_square_sum);
		%LET auc_sum&q				=%SYSEVALF(&&auc_sum&q+&auc_sum);
		%LET som_sum&q				=%SYSEVALF(&&som_sum&q+&som_sum);

%END;


%END;

/*Adjustierte Schtzer berechnen*/

DATA adjusted1;
	MERGE k_kenn4 k_kenn4a result_roc_ori2;
	BY dummy;

	%DO fg=1 %TO &anzahl;

	IF runde=&fg THEN	brier_a		= ((brier_k + brier_ori)-&&brier_sum&fg);
	IF runde=&fg THEN	slope_a		= ((slope_k + slope_ori)-&&slope_sum&fg);
	IF runde=&fg THEN	emax_a		= ((emax_k + emax_ori)-&&emax_sum&fg);
	IF runde=&fg THEN	max_resca_a	= ((max_resca_k + max_resca_ori)-&&max_resca_sum&fg);
	IF runde=&fg THEN	r_square_a	= ((r_square_k + r_square_ori)-&&r_square_sum&fg);
	IF runde=&fg THEN	auc_a		= ((auc_k + auc_ori)-&&auc_sum&fg);
	IF runde=&fg THEN	som_a		= ((som_k + som_ori)-&&som_sum&fg);

	%END;

RUN;

DATA adjusted2;
	MERGE k_kenn3 k_kenn3a result_roc_ori1;
	BY dummy;

	%DO fg=1 %TO &anzahl;

	IF runde=&fg THEN	maxyouden_a		= ((maxyouden_k + maxyouden_ori)-&&maxyouden_sum&fg);
	IF runde=&fg THEN	prob_a			= ((prob_k + prob_ori)-&&prob_sum&fg);
	IF runde=&fg THEN	sens_a			= ((sens_k + sens_ori)-&&sens_sum&fg);
	IF runde=&fg THEN	spez_a			= ((spez_k + spez_ori)-&&spez_sum&fg);
	IF runde=&fg THEN	ppv_a			= ((ppv_k + ppv_ori)-&&ppv_sum&fg);
	IF runde=&fg THEN	npv_a			= ((npv_k + npv_ori)-&&npv_sum&fg);
	IF runde=&fg THEN	psep_a			= ((psep_k + psep_ori)-&&psep_sum&fg);

	%END;

RUN;

DATA adjusted1 (KEEP=brier_a slope_a emax_a max_resca_a r_square_a auc_a som_a);
	SET adjusted1;
RUN;

DATA adjusted2 (KEEP=maxyouden_a prob_a sens_a spez_a ppv_a npv_a psep_a);
	SET adjusted2;
RUN;

DATA k_kenn4;
	SET adjusted1;
				RENAME brier_a		= brier_k;
				RENAME slope_a		= slope_k;
				RENAME emax_a		= emax_k;
				RENAME max_resca_a	= max_resca_k;
				RENAME r_square_a	= r_square_k;
				RENAME auc_a		= auc_k;
				RENAME som_a		= som_k;
RUN;

DATA k_kenn3;
	SET adjusted2;
				RENAME maxyouden_a 	= maxyouden_k;
				RENAME prob_a 		= prob_k;
				RENAME sens_a		= sens_k;
				RENAME spez_a 		= spez_k;
				RENAME ppv_a 		= ppv_k;
				RENAME npv_a 		= npv_k;
				RENAME psep_a 		= psep_k;
RUN;



/*Mittelwert bilden (Datei: k_kenn3)*/

PROC MEANS NOPRINT DATA=k_kenn3 FW=8 MAXDEC=2 ; 
	OUTPUT OUT=k_kenn3Mean;
RUN;

/*Ausgabedatei auf die jeweiligen Mittelwerte beschrnken*/

DATA k_kenn3Mean1;
	SET k_kenn3Mean;
	IF _STAT_='N' THEN DELETE;
	IF _STAT_='MIN' THEN DELETE;
	IF _STAT_='MAX' THEN DELETE;
	IF _STAT_='STD' THEN DELETE;
	dummy=1;
RUN;

/*Mittelwert bilden (Datei: k_kenn4)*/

PROC MEANS NOPRINT DATA=k_kenn4 FW=8 MAXDEC=2 ; 
	OUTPUT OUT=k_kenn4Mean;
RUN;

/*Ausgabedatei auf die jeweiligen Mittelwerte beschrnken*/

DATA k_kenn4Mean1;
	SET k_kenn4Mean;
	IF _STAT_='N' THEN DELETE;
	IF _STAT_='MIN' THEN DELETE;
	IF _STAT_='MAX' THEN DELETE;
	IF _STAT_='STD' THEN DELETE;
	dummy=1;
RUN;

/*berflssige Spaletn lschen*/

DATA k_kenn4Mean1 (DROP= _TYPE_ _FREQ_ _STAT_);
	SET k_kenn4Mean1;
RUN;

DATA k_kenn3Mean1 (DROP= _TYPE_ _FREQ_ _STAT_);
	SET k_kenn3Mean1;
RUN;

/*Ergebnisdateien zusammenfhren:*/
/*1. Cutpoint-unabhngige Prognosegtemae:*/

DATA gesamt1;
	MERGE k_kenn4Mean1 result_roc_ori2;
	BY dummy;
RUN;

/*2. Prognosegtemae beim Cutpoint des Ausgangsmodells:*/

DATA gesamt3;
	MERGE k_kenn3Mean1 result_roc_ori1;
	BY dummy;
RUN;

/*Ergebnisdatei gesamt1 fr schneren Output aufsplitten*/

/*AUC*/

DATA auc (KEEP=auc_ori auc_k auc_diff auc_percent);
	SET gesamt1;
	auc_diff=auc_ori-auc_k;
	auc_percent=(auc_ori-auc_k)/auc_ori;
RUN;

/*SomersD*/

DATA som (KEEP=som_ori som_k som_diff som_percent);
	SET gesamt1;
	som_diff=som_ori-som_k;
	som_percent=(som_ori-som_k)/som_ori;
RUN;

/*Brier Score*/

DATA brier (KEEP=brier_ori brier_k brier_diff brier_percent);
	SET gesamt1;
	brier_diff=brier_ori-brier_k;
	brier_percent=(brier_ori-brier_k)/brier_ori;
RUN;

/*Slope*/

DATA slope (KEEP=slope_ori slope_k slope_diff slope_percent);
	SET gesamt1;
	slope_diff=slope_ori-slope_k;
	slope_percent=(slope_ori-slope_k)/slope_ori;
RUN;

/*EMax*/

DATA emax (KEEP=emax_ori emax_k emax_diff emax_percent);
	SET gesamt1;
	emax_diff=emax_ori-emax_k;
	emax_percent=(emax_ori-emax_k)/emax_ori;
RUN;

/*R-Square*/

DATA rsquare (KEEP=r_square_ori r_square_k r_square_diff r_square_percent);
	SET gesamt1;
	r_square_diff=r_square_ori-r_square_k;
	r_square_percent=(r_square_ori-r_square_k)/r_square_ori;
RUN;

/*MaxRescaled R-Square*/

DATA maxresca (KEEP=max_resca_ori max_resca_k max_resca_diff max_resca_percent);
	SET gesamt1;
	max_resca_diff=max_resca_ori-max_resca_k;
	max_resca_percent=(max_resca_ori-max_resca_k)/max_resca_ori;
RUN;

/*++++++++++++++++++++*/

/*Standardoutput ausgeben*/
/*Fallzahl bestimmen:*/

%IF &miss=1 %THEN %DO;

%NUMBEROBS(datensatz=&datensatzxxx);

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/
%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/


		DATA work.fallzahl;
		SET  &datensatzxxx;
		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

		RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatzxxx    ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE 	standardoutput 

	%IF &miss=0 %THEN %DO;

			fallzahl

	%END;

;
RUN;
QUIT;

/*++++++++++++++++++++*/

/*Ergebnisse (gesamt1)ausgeben*/

PROC PRINT DATA= auc NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR auc_ori auc_k auc_diff auc_percent; 
    TITLE1 "Adjustierte Crossvalidierung Zusammenfassung";
	TITLE2 "Anzahl Gruppen: &gruppen , Anzahl Durchlufe: &anzahl";
	TITLE3 'O=Original / AC=Adjustiert Crossvalidiert / D=Differenz (O-AC)';
	TITLE4 'Korrektur der cutpoint-unabhngigen Prognosegtemae';
	LABEL 	auc_ori			='AUC O' 
			auc_k 			="AUC AC&gruppen"
			auc_diff 		='AUC D'
			auc_percent		='%';
RUN;

PROC PRINT DATA= som NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR som_ori som_k som_diff som_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	som_ori 		='SomersD O'
			som_k 			="SomersD AC&gruppen"
			som_diff 		='SomersD D'
			som_percent		='%';
RUN;

PROC PRINT DATA= brier NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  brier_ori brier_k brier_diff brier_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	brier_ori 		='Brier Score O'
			brier_k 		="Brier Score AC&gruppen"
			brier_diff 		='Brier Score D'	
			brier_percent	='%';
RUN;

PROC PRINT DATA= slope NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  slope_ori slope_k slope_diff slope_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	slope_ori 		='Slope O'
			slope_k 		="Slope AC&gruppen"
			slope_diff 		='Slope D'	
			slope_percent	='%';
RUN;

PROC PRINT DATA= emax NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR emax_ori emax_k emax_diff emax_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	emax_ori 		='Emax O'
			emax_k	 		="Emax AC&gruppen"
			emax_diff 		='Emax D'
			emax_percent	='%';
RUN;

PROC PRINT DATA= rsquare NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR r_square_ori r_square_k r_square_diff r_square_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	r_square_ori 		='R-Square O'
			r_square_k	 		="R-Square AC&gruppen"
			r_square_diff 		='R-Square D'
			r_square_percent	='%';
RUN;

PROC PRINT DATA= maxresca NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR max_resca_ori max_resca_k max_resca_diff max_resca_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	max_resca_ori 		='Max-Rescaled R-Square O'
			max_resca_k 		="Max-Rescaled R-Square AC&gruppen"
			max_resca_diff 		='Max-Rescaled R-Square D'
			max_resca_percent	='%';
RUN;

/*Ergebnisdatei gesamt3 fr schneren Output aufsplitten*/

/*Sensitivitt*/

DATA sens (KEEP=sens_ori sens_k sens_diff sens_percent);
	SET gesamt3;
	sens_diff=sens_ori-sens_k;
	sens_percent=(sens_ori-sens_k)/sens_ori;
RUN;

/*Spezifitt*/

DATA spez (KEEP=spez_ori spez_k spez_diff spez_percent);
	SET gesamt3;
	spez_diff=spez_ori-spez_k;
	spez_percent=(spez_ori-spez_k)/spez_ori;
RUN;

/*YoudenIndex*/

DATA youden (KEEP=maxyouden_ori maxyouden_k maxyouden_diff maxyouden_percent);
	SET gesamt3;
	maxyouden_diff=maxyouden_ori-maxyouden_k;
	maxyouden_percent=(maxyouden_ori-maxyouden_k)/maxyouden_ori;
RUN;

/*PPV*/

DATA ppv (KEEP=ppv_ori ppv_k ppv_diff ppv_percent);
	SET gesamt3;
	ppv_diff=ppv_ori-ppv_k;
	ppv_percent=(ppv_ori-ppv_k)/ppv_ori;
RUN;

/*NPV*/

DATA npv (KEEP=npv_ori npv_k npv_diff npv_percent);
	SET gesamt3;
	npv_diff=npv_ori-npv_k;
	npv_percent=(npv_ori-npv_k)/npv_ori;
RUN;

/*PSEP*/

DATA psep (KEEP=psep_ori psep_k psep_diff psep_percent);
	SET gesamt3;
	psep_diff=psep_ori-psep_k;
	psep_percent=(psep_ori-psep_k)/psep_ori;
RUN;

/*Ergebnisse (gesamt3) ausgeben*/

PROC PRINT DATA= sens NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR sens_ori sens_k sens_diff sens_percent; 
	TITLE1;
	TITLE2;
	TITLE3 'Korrektur der cutpoint-abhngigen Prognosegtemae';
	TITLE4 "(bei Cutpoint des Ausgangsmodells (cp=&prob_original)";
	LABEL 	sens_ori		='Sensitivitt O' 
			sens_k			="Sensitivitt AC&gruppen"
			sens_diff 		='Sensitivitt D'
			sens_percent	='%';
RUN;

PROC PRINT DATA= spez NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR spez_ori spez_k spez_diff spez_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	spez_ori 		='Spezifitt O'
			spez_k		 	="Spezifitt AC&gruppen"
			spez_diff 		='Spezifitt D'
			spez_percent	='%';
RUN;

PROC PRINT DATA= youden NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR maxyouden_ori maxyouden_k maxyouden_diff maxyouden_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	maxyouden_ori 		='Max. YoudenIndex O'
			maxyouden_k			="YoudenIndex AC&gruppen"
			maxyouden_diff 		='YoudenIndex D'
			maxyouden_percent	='%';
RUN;

PROC PRINT DATA= ppv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR ppv_ori ppv_k ppv_diff ppv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	ppv_ori			='Positive Korrektheit O'
			ppv_k 			="Positive Korrektheit AC&gruppen"
			ppv_diff 		='Positive Korrektheit D'
			ppv_percent		='%';
RUN;

PROC PRINT DATA= npv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  npv_ori npv_k npv_diff npv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	npv_ori 		='Negative Korrektheit O'
			npv_k 			="Negative Korrektheit AC&gruppen"
			npv_diff 		='Negative Korrektheit D'
			npv_percent		='%';
RUN;

PROC PRINT DATA= psep NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR psep_ori psep_k psep_diff psep_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	psep_ori		='Separation Index PSEP O'
			psep_k	 		="Separation Index PSEP AC&gruppen"
			psep_diff 		='Separation Index PSEP D'
			psep_percent	='%';
RUN;

/*Zwischendatenstze lschen*/

PROC DATASETS NOLIST;
	DELETE k_kenn1 k_kenn2 association association_ori auswert ergebnis gof gof_ori
		   groups out_est roc roc_ori result2 result validate
		   k_kenn1a k_kenn2a k_kenn3a k_kenn4a 
		   adjusted1 adjusted2
		   k_kenn3mean k_kenn4mean k_kenn4 k_kenn3 index_ori
		   youden spez som sens rsquare result_roc_ori1 result_roc_ori2 psep ppv npv
		   maxresca k_kenn3mean1 k_kenn4mean1 gesamt3 gesamt1 emax brier auc slope 

		   %IF &miss=0 %THEN %DO;

		   nomissing

		   %END;
 
;
RUN;
QUIT;


%MEND ADJUSTED_CROSSVALIDATION;
