/************************************************************************************/
/*																				  	*/
/*	NAME:		auroc_koenig.mac.sas												*/
/*  TITLE: 		Berechnung der Flche unter einer ROC- Kurve (AUC)			 		*/
/*  KEYS: 		roc analysis, roc curves, confidence limit, area under the curve,	*/
/*				AUC, standard error													*/
/*	VERSION:	SAS ???																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		RANK, EXPAND														*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\auroc_koenig.mac.sas				*/
/*  Autoren:						J. Koenig										*/
/*	Programmierer:					Jochen Koenig									*/
/*	Datum der Ersterstellung:		3/1998											*/
/*  Aktuelle Makro- Version:		3/1998											*/
/*																					*/
/*  Kurzbeschreibung:				Makro zur Berechnung der Flche unter einer 	*/
/*									ROC- Kurve (AUC)								*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_ROC.MAC.SAS									*/
/*																					*/
/*  Genauere Beschreibung siehe:	Makrobeschreibung des Autors					*/
/*																					*/
/*  Verwendete externe Untermakros:	-----											*/
/*																					*/
/*	Bentigte Dateien:				Datensatz (Parameter 'data=')					*/
/*																					*/
/*  Erzeugte Dateien:				work.__wilse work.__sm     work.__sesp work.__sc*/
/*		   							work.__dcum  work.__cmp    work.__chi  work.__c */
/*									work.__s 	 work.modwilse work.pred2  work. __m*/
/*		   							work.__s1 	 work.__s0     work.__r1   work.__n */
/*									work.__r 	 work.__nq 	   work.__n1   work.__n0*/
/*									work.__lang										*/
/*																					*/
/*  Programmnderungen 																*/
/*  und -ergnzungen 				C. Ziegler, 9/2003:								*/
/*  (Name, Datum, Beschreibung):    Ausgabe des Makros wurde angepasst, 			*/
/*									Berechnung eins zustzliches Konfidenzintervalls*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%macro auroc;

%macro v3(a,b,c); %* macht Variablenliste aus  allen Kombinationen von wordliste a,b und c
                     wenn c leer ist werden nur Komb aus a und b gebildet;
  %let ___i=1; %let ___w1=%scan(&a,&___i);
 %if (&c=) %then
  %do %while(&___w1^=); %let ___j=1; %let ___w2=%scan(&b,&___j);
      %do %while(&___w2^=); &___w1.&___w2
      %let ___j=%eval(&___j+1);%let ___w2=%scan(&b,&___j);%end;
  %let ___i=%eval(&___i+1); %let ___w1=%scan(&a,&___i);%end;
 %else
  %do %while(&___w1^=); %let ___j=1; %let ___w2=%scan(&b,&___j);
      %do %while(&___w2^=); %let ___k=1; %let ___w3=%scan(&c,&___k);
          %do %while(&___w3^=); &___w1.&___w2.&___w3
          %let ___k=%eval(&___k+1);%let ___w3=%scan(&c,&___k);%end;
      %let ___j=%eval(&___j+1); %let ___w2=%scan(&b,&___j);%end;
  %let ___i=%eval(&___i+1); %let ___w1=%scan(&a,&___i);%end;
%mend v3;


%let w=%scan(&var,1); %let i=1;
%do %while(&w^=);%let var&i=&w; %let i=%eval(&i+1); %let w=%scan(&var,&i); %end;
 %let varzahl=%eval(&i-1);
 %let vn=;%do i=1 %to &varzahl;%let vn=&vn &i;%end;
 %if &vardef=n %then %let vardef=%str(vardef=n); %else %let vardef=; %* for comparsion with proc catmod;

 %let ri=%v3(__ri,&vn); * Innergruppenraenge; %let rg=%v3(__rg,&vn); %* ranks in the total sample;
 %let rl=%v3(__r,&vn); * DeLong-Ranks: v01 and v10 from DeLong;
 %let rl_=%str(__r2--__r&varzahl);

%MACRO CALCUr; proc sort;by __varnam &event cutpoint;
 proc rank f out=__dcum;var cutpoint;ranks cum;by __varnam &event;
 data __dcum;set __dcum;by __varnam &event cum;if last.cum; proc sort;by __varnam cutpoint &event;
 data __sesp;set __dcum; by __varnam cutpoint; retain spez 0 sens 1; if first.__varnam then do;spez=0;sens=1;output;end;
 if &event=0 then spez=cum;else sens=1-cum; if last.cutpoint then output;   drop &event;
 label sens="sensitivity" spez="specificity";
%MEND CALCUr ;
%macro se_wil(meth=hm,tiedenom=); %*------------- plug in standard error from Hanley & McNeil 1982 ----------------;
 %if (&tiedenom=) %then %do; %if &meth=hm %then %let tiedenom=3; %else %let tiedenom=4;%end;       %let cit=;
 data __wilse;set __sesp;by __varnam;        %*--------------------- nach delong ist tiedenom=4 korrekt(er);
  retain q1 q2 w; lg=1-lag(sens);lf=lag(spez); g=1-sens;
  if first.__varnam then do;q1=0;q2=0;w=0;lg=0;lf=0;end;
  else do;df=spez-lf; dg=g-lg; w+(lf+df/2)*dg; q2+(spez*lf+df*df/&tiedenom)*dg; q1+(g*lg+dg*dg/&tiedenom)*df; end;
  if last.__varnam then do;q1=2*w-1+q1; output;end;
 data __wilse;merge  __n0 __n1 __wilse;by __varnam;
   %if &meth=hm %then %do; se_w=sqrt((  w*(1-w) + (__n0-1)*(q2-w*w) + (__n1-1)*(q1-w*w)  )/__n0/__n1);   %end;
                %else %do; se_w=sqrt(  (q2-w*w)/(__n1-1) + (q1-w*w)/(__n0-1)  ); %let cit=%str((DeLong & al. 1988)); %end;
                                                            keep se_w w q1 q2 __n1 __n0 __varnam;
 label __n1="n1" __n0="n0" 

/* w="Wilcoxon statistic" */
 w="AUC"
 se_w="AUC Standard Error &cit";

 call symput("area"||left(put(_n_,3.)),put(w,6.3)); call symput("sew"||left(put(_n_,3.)),put(se_w,6.3));

 DATA __wilse;
 SET __wilse;
 obs=_N_;
 RUN;

 PROC SORT DATA=__wilse;
 BY DESCENDING obs;
 RUN;

 DATA __wilse;
 SET __wilse;
 ki_auc_u=w-(1.96*se_w);
 ki_auc_o=w+(1.96*se_w);
 LABEL ki_auc_u="95% Konfidenzintervall AUC- untere Grenze"
		  ki_auc_o="95% Konfidenzintervall AUC- obere Grenze";
 RUN;


 * original:;
/* proc print label;id __varnam;var __n0 __n1 w se_w ki_auc_u ki_auc_o ; */
******************************************;
* von Ch. Ring verndert, September 2003:;
* Achtung: darauf achten, dass die Variablenbezeichnung so gewhlt wird, dass der erste Variablenname 
*          auch im Alphabet vor dem zweiten kommt!!;
 data modwilse; set __wilse; if obs=1;

%mend se_wil;
%macro s11;  %*----------------------------------------------- Only for bias correction----------------------;
  %do i=0 %to 1; data __d&i;set __c(where=(&event=&i));%end;
  data __d1;set __d1(rename=( %do i=1 %to &varzahl; &&var&i=__x&i %end;));__i=_n_;
  data __d0;set __d0;do __i=1 to &n1;output;end;proc sort;by __i;
  data __d1;merge __d0 __d1;by __i; %* all pairs ;
   array __x(__r) __x1-__x&varzahl;
   array __y(__r) &var;
   array __psi(__r) &rl; keep &rl;
   do over __x;__psi=(__y<__x)+(__y=__x)*.5;end;
  proc corr cov noprint nocorr vardef=n outp=__s11(where=(_name_^=" ") drop=_type_);
  proc datasets nolist;delete __d0 __d1;
  data __s11;set __s11;array v &rl;do over v;v=-v/(&n1-1)/(&n0-1);end;           %* ______ Koeff. -1/(n-1)(m-1);
  data __s11d;retain v1-v&varzahl;set __s11;   keep __s11d __vgl; length __vgl $5; %* --------------- fr Paarvgl.;
   array v(*) v1-v&varzahl;array psi(*) &rl;   v[_n_]=psi[_n_];
       if _n_>1 then do i=1 to _n_-1;__s11d=psi[_n_]+v[i]-2*psi[i];__vgl=trim(left(put(i,3.)))||"-"||left(put(_n_,3.));output;end;
   proc sort;by __vgl;
%mend s11;
*-------------------- overall comparison -------------------------------------------------------;
%macro s__;%if &biascorr=1 %then %do;%s11;%end;
 proc corr data=__r noprint cov nocorr outp=__s &vardef;var &rl;by &event;
 data __s __sc __m;merge __s(rename=(_type_=_type__)) __n __nq(rename=(__n=__nq));by &event;  array __v &rl;  keep _name_ &rl;
     if _type__="MEAN" & &event=0 then output __m;
     if _type__="COV" then do;
            do over __v;__v=__v/__n; end; output __s;
            do over __v;__v=__v*__nq/(__nq-1); end; output __sc;    end;
 proc means nway noprint missing data=__s;class _name_; output out=__s(drop=_type_ _freq_) sum=;var &rl;
                                                      %if &biascorr=1 %then %do;data __sc;set __sc __s11;
 proc means nway noprint missing;class _name_; output out=__sc(drop=_type_ _freq_) sum=;var &rl;  %end;
%macro chi(c);proc iml;df=&varzahl-1; l=j(df,1,-1)||i(df); use __s&c;use __m;out=j(1,3); create __chi&c var {chisq df p};
  do data;  setin __s&c; read next &varzahl var {&rl} into s;  setin __m;read next 1 var _num_  into xm;
   %if &test=1 %then %do; stderr=sqrt(vecdiag(s)); hanley=sqrt((vecdiag(s)*(&n1-1)*(&n0-1) + xm`#(j(&varzahl,1)-xm`))/&n0/&n1);
/*       print stderr hanley;*/
   %end;
  xm=xm*l`;
   ew=eigval(s); if ew[><]<1e-25 then do; ev=eigvec(s); print "not pos.def. covariance matrix" s ew ev;chisq=.;p=.; end;
             else do; chisq=xm*inv(l*s*l`)*xm`;p=1-probchi(chisq,df); end; append;
  end;  data __chi;set __chi; label df="df" p="p-value" chisq="chi squared";
%mend chi; %chi; %if &biascorr=1 %then %do; %chi(c); data __chi;merge __chi __chic(rename=(chisq=chisq_c p=p_c));
                       label chisq_c="chi squared (unbiased)" p_c="p-value (unbiased)"; %end;
/* proc print label noobs; title "Asymptotic chi-square test for H0: all areas equal";run;title;*/
%mend s__;
*-------------------- pairwise comparisons   --------------------------------------------------------;
%macro cmp; data __cmp;set __r;    length __vgl $5;
  %do i=1 %to %eval(&varzahl-1);%do j=%eval(&i+1) %to &varzahl;__vgl="&i-&j";d_auroc=__r&i-__r&j;output;%end;%end;
 proc sort;by __vgl &event;
  %do i=0 %to 1;
 proc means noprint &vardef data=__cmp(where=(&event=&i)); output out=__s&i stderr=__se&i mean=;var d_auroc;by __vgl;%end;
 data __sm;merge __s0 __s1
    %if &biascorr=1 %then %do; __s11d; se_dc=sqrt(__se0**2*&n1/(&n1-1) + __se1**2*&n0/(&n0-1) + __s11d);
         p_asym_c=2*probnorm(-abs(d_auroc)/se_dc);drop __s11d;
         label  p_asym_c="p-value 2-sided (unbiased Z-approx.)" se_dc="standard error (unbiased)";
    %end;
    ; se_d=sqrt(__se0**2+__se1**2);
  p_asym=2*probnorm(-abs(d_auroc)/se_d);drop _type_ _freq_ __se0 __se1;
 label d_auroc="difference of areas" p_asym="p-value 2-sided (Z-approx.)" __vgl="comparison" se_d="standard error";
/* proc print label; id __vgl;*/
%mend cmp;
*-------------------------------- Main Macro ------------------------;
 data __c;set &data(keep=&var &event);if nmiss(of &var &event)=0 ;proc sort;by &event;
 proc means noprint;output out=__n(drop=_type_ _freq_) n=__n;var &var1;by &event;
    data __nq;set __n; call symput("n"||put(&event,1.), put(__n,4.));&event=(&event=0);proc sort;by &event;run;
 proc rank data=__c out=__r1;var &var;ranks &ri;by &event;
 proc rank data=__c out=__r;var &var;ranks &rg;
 data __r;merge __r __r1 (drop=&var) __nq;by &event;   drop &ri &rg __n;
  array __v(__r) &var;
  array __ri(__r) &ri;
  array __rg(__r) &rg;
  array __rl(__r) &rl;  %* v10,v01 from DeLong;
  do over __rl; if &event=1 then __rl=(__rg-__ri)/__n; else __rl=(__n-__rg+__ri)/__n; end;
 data __lang;set __c; keep &event __varnam cutpoint; length __varnam $16; label __varnam="Variable";
     %do i=1 %to &varzahl;  
     __varnam="&&var&i"; 
     cutpoint=&&var&i;if cutpoint>.z then output; %end;
 %CALCUr;
  %do i=0 %to 1; proc means noprint data=__lang(where=(&event=&i));output out=__n&i n=__n&i;var cutpoint; by __varnam; %end;

/* %se_wil(meth=hm); title "Areas under the curve (termed Wilcoxon statistic) and standard error after Hanley & McNeal 1982";run; title;*/

 %se_wil(meth=delong); 
 title "Areas under the curve (AUC) und Standard Error";
/* title "Areas under the curve (termed Wilcoxon statistic)";*/
/* title2 "and standard error after DeLong & al. 1988 (recommended)";*/
 run;
 title;
 %s__; %cmp;
 run;
%MEND auroc;


************************************************************************
*   MACRO  A U R O C    Version 1.1  March 1998
*
*  Author: Jochem Koenig
*  Institut fuer Medizinische Biometrie, Epidemiologie und Med. Informatik
*  66421 Homburg
*  Tel. 06841-162068
*  jk@med-imbei.uni-sb.de
*  www.med-rz.uni-sb.de/med_fak/imbei
*======================================================================
*  Area under the ROC curve including standard errors and inference
*  concerning several variables. See DeLong & DeLong (1988).Bioms pp 837-
*
*  In addition a biascorrected version of the covariance matrix of
*  the vector of estimated areas under the ROC-curve is available, which
*  is unpublished so far, and was presented by the author on the spring
*  meeting of the german region of the international biometric society in
*  Munich 1997.
*
*  The standard errors following Hanley & McNeal 1982 are included for comparison only.
*  The author recommends use of DeLongs version of the standard error.
*=====================================================================
*  MACROPARAMETER:
*  OBLIGATORY:   var         List of predictor variables
*                event       grouping variable with
*                              coding 1 for diseased and
*                                     0 for undiseased
*  optional:     data=_last_ SAS DATASET NAME
*                vardef=n    Denominator N in covariance matrix (default N-1)
*                biascorr=1  Bias correction of variances (default=0)
*                            requires more time and space
*                test=1      additional output for checking purpose (default=0)
*
*
*  NOTES:
*  1. All predictor variables must satisfy the condition that high values
*     indicate disease. If necessary this canbe achieved by multiplying -1.
*  2. Only records with complete data on all predictors and outcome are used.
*  3. All data set names start with "__".
*  4. Datasets containing results:
*     __wilse contains ROC-areas (W) and standard errors (SE_W)(DeLong)
*     __sm    contains differences between ROC-areas and corresponding SE and p-values
*     __chi   contains chisquare test results for H0: all areas equal
*
*  Evaluation of the program:
*  1. It reproduces area and standard error
*     of the example from Hanley & McNeil(1982), Radiology 143.p30.
*  2. If two dichotomos predictors are compared and denominator definition
*     N is used (instead of N-1), the estimators and standard error
*     of this program can be reproduced by PROC CATMOD;
     /* proc catmod; model x1*x2=event;response 0 1 -1 0; */
*      gibt der Koeffizient von event den Wert Sens1+Spez1-Sens2-Sepz2 an.
*  3. The program was not yet fully tested for extreme cases such as
*        - singular within group covariance matrix or
*        - very small sample size (leading to singular covariance matrix)
*
*  Changes since V1.0: optional parameter nomiss removed, since it was not implemented.
*
*
*======================================================================
* The author does not overtake any responsibility for the correctness
* of results. It must not be modified without removing the heading lines containing
  the authors name and version number
***********************************************************************;

