/************************************************************************************/
/*																				  	*/
/*	NAME:		exist_var.mac.sas													*/
/*  TITLE: 		Prfung ob bestimmte Variablen in einem Datensatz vorhanden sind	*/
/*  KEYS: 		variables, dataset													*/
/*	VERSION:	SAS 8.2																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		TRANSPOSE															*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\exist_var.mac.sas					*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*	Programmierer:					Christoph Ziegler								*/
/*	Datum der Ersterstellung:		8/2003											*/
/*  Aktuelle Makro- Version:		12/2003											*/
/*																					*/
/*	Kurzbeschreibung:				Prfung ob bestimmte Variablen in einem 		*/
/*							    	Datensatz vorhanden sind						*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_MISSING.MAC.SAS								*/
/*									PM_MI_ANALYZE.MAC.SAS							*/
/*									PM_MULTICOLLIN.MAC.SAS							*/
/*									PM_INFLUENCE.MAC.SAS							*/
/*									PM_LOGREG.MAC.SAS								*/
/*									PM_ROC.MAC.SAS									*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*																					*/
/*  Bentigte Dateien:				Datensatz 		  (Parameter 'dsn=')			*/
/*																					*/
/*  Erzeugte Dateien:				-----											*/
/*																					*/
/*  Programmnderungen 																*/
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):     												*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%MACRO EXIST_VAR(	dsn= ,
					var=
				);

	%GLOBAL exist;

	/*Anzahl der Variablen ermitteln:*/
	%GLOBAL count_x varx;
	%LET count_x=1;
	%LET varx=%QSCAN(&var,&count_x,%STR( ));
	%DO %WHILE(&varx ne);
     	%LET count_x=%EVAL(&count_x+1);
     	%LET varx=%QSCAN(&var,&count_x,%STR( ));
	%END;
	%LET count_x=%EVAL(&count_x-1);

	/*Datastep, der bewute Fehlermeldung generiert, falls Variablen fehlen*/

	DATA test;
		SET &dsn;
		KEEP &var;
	RUN;

	/*Transponieren, um die Anzahl der Variablen mittels 'numberobs.mac.sas'*/
    /*bestimmen zu knnen*/

	PROC TRANSPOSE DATA=test OUT=test2;
	RUN;

	%NUMBEROBS(datensatz=test2)

	%let num_vars=&nnnn1;

	/*Vergleich*/

	%IF &num_vars = &count_x %THEN %DO;
		%LET exist='YES';
	%END;
	%ELSE %LET exist='NO';

	PROC DATASETS NOLIST;
		DELETE test test2;
	RUN;
	QUIT;

%MEND EXIST_VAR;