/************************************************************************************/
/*																				  	*/
/*	NAME:		fracpoly.mac.sas													*/
/*  TITLE: 		Univariate Betrachtung von stetigen Variablen mit Fractional 		*/
/*				Polynomials sowie univariate logistische Regressionen mit den 		*/
/*				besten Fractional Polynomials vom Grad 1 und 2						*/
/*  KEYS:		fractional polynomials, logistic regression							*/
/*	VERSION:	SAS 8.2																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		LOGISTIC															*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\fracpoly.mac.sas					*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*	Programmierer:					Christoph Ziegler								*/
/*	Datum der Ersterstellung:		8/2003											*/
/*  Aktuelle Makro- Version:		12/2003											*/
/*																					*/
/*	Kurzbeschreibung:				Allgemeines Makro zur univariaten Betrachtung 	*/
/*							    	von stetigen Variablen mit Fractional 			*/
/*									Polynomials sowie fr univariate logistische 	*/
/*									Regressionen mit den besten Fractional 			*/
/*									Polynomials vom Grad 1 und 2					*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_UNI_LOGREG.MAC.SAS							*/
/*																					*/
/*  Genauere Beschreibung siehe:	PM_UNI_LOGREG.MAC.SAS							*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*																					*/
/*	Bentigte Dateien:				Datensatz (Parameter 'data=')					*/
/*																					*/
/*																					*/
/*  Erzeugte Dateien:				-----											*/
/*																					*/
/*  Programmnderungen 																*/
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):     												*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%MACRO FRACPOLY (data		= ,
				 resp_var	= ,
				 event		= ,
				 xvar		= ,
				 macro_path	=
			    );

FILENAME y &macro_path;
%INCLUDE y ('numberobs.mac.sas');

%GLOBAL nnnn1;

/*Anzahl der stetigen Variablen ermitteln:*/
%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Elimination aller missing values innerhalb der xvars & resp_var (standardmig)*/

DATA work.fracpoly (KEEP=&resp_var &xvar);
	SET  &data;

	%DO iu=1 %TO &count_x;
		%LET xvar_liste=%SCAN(&xvar,&iu,%STR( ));
			IF &xvar_liste=. THEN DELETE;
	%END;

			IF &resp_var=. THEN DELETE;
RUN;


/*Fr jede stetige Variable alle Kombinationen bestimmen*/

%DO it=1 %TO &count_x;
	%LET xvar_liste=%SCAN(&xvar,&it,%STR( ));

/*prfen, ob in den Werten 0 und / oder negative Werte vorkommen*/
/*	--> fr diese Variable keine Fractional polynomials berechnen*/
/*	---> Hinweis im Output*/

DATA negativtest;
	SET fracpoly;
	KEEP &xvar_liste;
RUN;

ODS LISTING CLOSE;
ODS OUTPUT OneWayFreqs=negativ;
PROC FREQ DATA=negativtest;
TABLES &xvar_liste;
RUN;
ODS OUTPUT CLOSE;
ODS LISTING;

DATA negativ (KEEP=&xvar_liste);
SET negativ;
RUN;

%NUMBEROBS (datensatz=negativ);

%LET neg1=&nnnn1;

DATA negativ2;
SET negativ;
IF &xvar_liste <= 0 THEN DELETE;
RUN;

%NUMBEROBS (datensatz=negativ2);

%LET neg2=&nnnn1;

%IF &neg1 ne &neg2 %THEN %DO;

DATA note;
	note="";
RUN;

PROC PRINT DATA=note NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung (Fractional Polynomials):                        ";
	TITLE4 ;
	TITLE5 "Fr die Variable &xvar_liste konnten keine               "; 
	TITLE6 "fractional polynomials berechnet werden, da";
	TITLE7 "diese negative Werte oder 0 enthlt!";
	TITLE8 ;
	TITLE9 'Mgliche Abhilfe: Neue, transformierte Variable erzeugen';
	TITLE10 'welche Werte >0 besitzt! Bsp.: X1new= X1 + 1';

	
	LABEL note = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
RUN;

PROC DATASETS NOLIST;
	DELETE negativ negativ2 negativtest note;
RUN;
QUIT;

%GOTO negativ;

%END;

PROC DATASETS NOLIST;
	DELETE negativ negativ2 negativtest;
RUN;
QUIT;

DATA fracpoly;
	SET fracpoly;

	&xvar_liste._m2		=1/(&xvar_liste**2);
	&xvar_liste._m1		=1/&xvar_liste;
	&xvar_liste._m05	=1/(sqrt(&xvar_liste));
	&xvar_liste._0		=log(&xvar_liste);
	&xvar_liste._05		=sqrt(&xvar_liste);
	&xvar_liste._1		=&xvar_liste;
	&xvar_liste._2		=&xvar_liste**2;
	&xvar_liste._3		=&xvar_liste**3; 

	&xvar_liste._m22	=&xvar_liste._m2*&xvar_liste._0;
	&xvar_liste._m11	=&xvar_liste._m1*&xvar_liste._0;
	&xvar_liste._m0505	=&xvar_liste._m05*&xvar_liste._0;
	&xvar_liste._00		=&xvar_liste._0*&xvar_liste._0;
	&xvar_liste._0505	=&xvar_liste._05*&xvar_liste._0;
	&xvar_liste._11		=&xvar_liste._1*&xvar_liste._0;
	&xvar_liste._22		=&xvar_liste._2*&xvar_liste._0;
	&xvar_liste._33		=&xvar_liste._3*&xvar_liste._0;

RUN;

/*ber alle mglichen 44 Kombinationen univariate Proc Logistic*/

%LET kombinationen_count=44;

%LET comb1= &xvar_liste._m2;
%LET comb2= &xvar_liste._m1;
%LET comb3= &xvar_liste._m05;
%LET comb4= &xvar_liste._0;
%LET comb5= &xvar_liste._05;
%LET comb6= &xvar_liste._1;
%LET comb7= &xvar_liste._2;
%LET comb8= &xvar_liste._3;

%LET comb9= &xvar_liste._m2 &xvar_liste._m22;
%LET comb10= &xvar_liste._m2 &xvar_liste._m1;
%LET comb11= &xvar_liste._m2 &xvar_liste._m05;
%LET comb12= &xvar_liste._m2 &xvar_liste._0;
%LET comb13= &xvar_liste._m2 &xvar_liste._05;
%LET comb14= &xvar_liste._m2 &xvar_liste._1;
%LET comb15= &xvar_liste._m2 &xvar_liste._2;
%LET comb16= &xvar_liste._m2 &xvar_liste._3;

%LET comb17= &xvar_liste._m1 &xvar_liste._m11;
%LET comb18= &xvar_liste._m1 &xvar_liste._m05;
%LET comb19= &xvar_liste._m1 &xvar_liste._0;
%LET comb20= &xvar_liste._m1 &xvar_liste._05;
%LET comb21= &xvar_liste._m1 &xvar_liste._1;
%LET comb22= &xvar_liste._m1 &xvar_liste._2;
%LET comb23= &xvar_liste._m1 &xvar_liste._3;

%LET comb24= &xvar_liste._m05 &xvar_liste._m0505;
%LET comb25= &xvar_liste._m05 &xvar_liste._0;
%LET comb26= &xvar_liste._m05 &xvar_liste._05;
%LET comb27= &xvar_liste._m05 &xvar_liste._1;
%LET comb28= &xvar_liste._m05 &xvar_liste._2;
%LET comb29= &xvar_liste._m05 &xvar_liste._3;

%LET comb30= &xvar_liste._0 &xvar_liste._00;
%LET comb31= &xvar_liste._0 &xvar_liste._05;
%LET comb32= &xvar_liste._0 &xvar_liste._1;
%LET comb33= &xvar_liste._0 &xvar_liste._2;
%LET comb34= &xvar_liste._0 &xvar_liste._3;

%LET comb35= &xvar_liste._05 &xvar_liste._0505;
%LET comb36= &xvar_liste._05 &xvar_liste._1;
%LET comb37= &xvar_liste._05 &xvar_liste._2;
%LET comb38= &xvar_liste._05 &xvar_liste._3;

%LET comb39= &xvar_liste._1 &xvar_liste._11;
%LET comb40= &xvar_liste._1 &xvar_liste._2;
%LET comb41= &xvar_liste._1 &xvar_liste._3;

%LET comb42= &xvar_liste._2 &xvar_liste._22;
%LET comb43= &xvar_liste._2 &xvar_liste._3;

%LET comb44= &xvar_liste._3 &xvar_liste._33;


DATA &xvar_liste._ergebnis8; SET _NULL_;

DATA &xvar_liste._ergebnis36; SET _NULL_;

DATA temp;
    variable='                                               ';
	wert    ='                                               ';
RUN;

%DO zu=1 %TO &kombinationen_count;

/*Proc Logistic- Output unterdrcken*/

ODS LISTING CLOSE;

/*mittels ODS gewnschte Tabelle mit Testergebnis rausschreiben*/

ODS OUTPUT GlobalTests=gt&zu;

PROC LOGISTIC DATA=fracpoly NAMELEN=32; 
MODEL &resp_var (event="&event") = &&comb&zu; 
RUN;

/*ODS Output shclieen*/

ODS OUTPUT CLOSE;

/*Ausgabe wieder aktivieren*/

ODS LISTING;

DATA gt&zu ;
	SET gt&zu;
	IF test='Score' THEN DELETE;
	IF test='Wald'  THEN DELETE;
RUN;

DATA gt&zu (DROP=test df probchisq);
	SET gt&zu;
RUN;

/*bentigter Wert als Makro- Variable speichern */

DATA  gt&zu;
		SET  gt&zu;
		CALL SYMPUT('wert',trim(left(put(chisq,BEST6.))));
RUN;

PROC DATASETS NOLIST;
	DELETE gt&zu;
RUN;
QUIT;

DATA temp;
	SET temp;
		variable="&&comb&zu";
		wert    ="&wert";
RUN;

/*Ergebnisse zu einer Tabelle zusammenfassen*/

/*8*/

%IF &zu <=8 %THEN %DO;


DATA &xvar_liste._ergebnis8;
	SET temp &xvar_liste._ergebnis8;
RUN;

%END;

/*36*/

%ELSE %DO;

DATA &xvar_liste._ergebnis36;
	SET temp &xvar_liste._ergebnis36;
RUN;

%END;



%END;

/*Variable wert in eine numerische Variable umwandeln*/

DATA &xvar_liste._ergebnis36;
	SET &xvar_liste._ergebnis36;
	wert2=input(wert,6.);
RUN;

DATA &xvar_liste._ergebnis8;
	SET &xvar_liste._ergebnis8;
	wert2=input(wert,6.);
RUN;

DATA &xvar_liste._ergebnis36 (DROP=wert);
	SET &xvar_liste._ergebnis36;
RUN;

DATA &xvar_liste._ergebnis8 (DROP=wert);
	SET &xvar_liste._ergebnis8;
RUN;

DATA &xvar_liste._ergebnis36;
	SET &xvar_liste._ergebnis36;
	RENAME wert2=wert;
RUN;

DATA &xvar_liste._ergebnis8;
	SET &xvar_liste._ergebnis8;
	RENAME wert2=wert;
RUN;

PROC SORT DATA = &xvar_liste._ergebnis8;
	BY DESCENDING wert;
RUN;

PROC SORT DATA = &xvar_liste._ergebnis36;
	BY DESCENDING wert;
RUN;

/*Vergleich max2 <--> max1*/

/*maximaler Wert vom 36er DS bestimmen*/

DATA max2;
	SET &xvar_liste._ergebnis36;
	IF _N_ ne 1 THEN DELETE;
RUN;

DATA max2;
		SET  max2;
		CALL SYMPUT('max2',trim(left(put(wert,BEST6.))));
		CALL SYMPUT('max2comb',variable);
RUN;

/*maximaler Wert vom 8er DS bestimmen*/

DATA max1;
	SET &xvar_liste._ergebnis8;
	IF _N_ ne 1 THEN DELETE;
RUN;

DATA max1;
		SET  max1;
		CALL SYMPUT('max1',trim(left(put(wert,BEST6.))));
		CALL SYMPUT('max1comb',variable);
RUN;

PROC DATASETS NOLIST;
DELETE max2 max1;
RUN;
QUIT;

/*falls Differenz negativ ist: Differenz auf 0 setzen --> p-Wert=1*/

%LET diff=%SYSEVALF(&max2-&max1);

%LET diff=%SYSFUNC(ROUND(&diff,0.001));

%LET difftest=%SYSEVALF(&diff,INTEGER);

%IF &difftest <0 %THEN %DO;

%LET diff=0;

%END;

/*p- Wert bestimmen*/

DATA &xvar_liste._p_value1;
    diff=&diff;
	p_value=1-PROBCHI(&diff,1);
	call symput('p_value',trim(left(put(p_value,BEST6.))));
RUN;

DATA ergebnis4;
	function='Grad 2';
	comb	="&max2comb";
	max		="&max2";
	diff	="&diff";
	p		="&p_value";
	dummy	=1;
RUN;

/*Vergleich max1 <--> max0*/

/*zuerst max0 bestimmen*/
/*(jeweilige stetige Variable normal ins Modell)*/
/*erhltlich unter ergebnis8 (variable_1)*/

DATA max0;
	SET &xvar_liste._ergebnis8;
    IF variable ne "&xvar_liste._1" THEN DELETE;
RUN;

DATA max0;
	SET max0;
RUN;


/*bentigter Wert als Makro- Variable speichern */

DATA  max0;
		SET  max0;
		CALL SYMPUT('max0',trim(left(put(wert,BEST6.))));
		CALL SYMPUT('max0comb',variable);
RUN;

PROC DATASETS NOLIST;
	DELETE max0;
RUN;
QUIT;

/*falls Differenz negativ ist: Differenz auf 0 setzen --> p-Wert=1*/

%LET diff=%SYSEVALF(&max1-&max0);

%LET diff=%SYSFUNC(ROUND(&diff,0.001));

%LET difftest=%SYSEVALF(&diff,INTEGER);

%IF &difftest <0 %THEN %DO;

%LET diff=0;

%END;

/*p- Wert bestimmen*/

DATA &xvar_liste._p_value2;
    diff=&diff;
	p_value=1-PROBCHI(&diff,1);
	call symput('p_value',trim(left(put(p_value,BEST6.))));
RUN;

DATA ergebnis3;
		function='Grad 1';
		comb	="&max1comb";
		max		="&max1";
		diff	="&diff";
		p		="&p_value";
		dummy	=1;
RUN;



/*Vergleich max0 <--> max -1*/

%LET maxm1=0;

/*falls Differenz negativ ist: Differenz auf 0 setzen --> p-Wert=1*/


%LET diff=%SYSEVALF(&max0-&maxm1);

%LET diff=%SYSFUNC(ROUND(&diff,0.001));

%LET difftest=%SYSEVALF(&diff,INTEGER);

%IF &difftest <0 %THEN %DO;

%LET diff=0;

%END;

/*p- Wert bestimmen*/

DATA &xvar_liste._p_value3;
    diff=&diff;
	p_value=1-PROBCHI(&diff,1);
	call symput('p_value',trim(left(put(p_value,BEST6.))));
RUN;

DATA ergebnis2;
		function='Linear';
		comb	="&max0comb";
		max		="&max0";
		diff	="&diff";
		p		="&p_value";
		dummy	=1;
RUN;

/*Ergebnisse je Variable bersichtlich ausgeben*/

/*Ergebnistabelle36 in die Royston/Altmann- Polynom-Nomenklatur umwandeln */

DATA &xvar_liste._ergebnis36;
	SET &xvar_liste._ergebnis36;
	p1='     ';
	p2='     ';

IF variable= "&xvar_liste._m2 &xvar_liste._m22" THEN p1='-2' ; 
IF variable= "&xvar_liste._m2 &xvar_liste._m1"	THEN p1='-2' ; 
IF variable= "&xvar_liste._m2 &xvar_liste._m05"	THEN p1='-2' ;
IF variable= "&xvar_liste._m2 &xvar_liste._0"	THEN p1='-2' ;
IF variable= "&xvar_liste._m2 &xvar_liste._05"	THEN p1='-2' ;
IF variable= "&xvar_liste._m2 &xvar_liste._1"	THEN p1='-2' ;
IF variable= "&xvar_liste._m2 &xvar_liste._2"	THEN p1='-2' ;
IF variable= "&xvar_liste._m2 &xvar_liste._3"	THEN p1='-2' ;

IF variable= "&xvar_liste._m1 &xvar_liste._m11"	THEN p1='-1' ;
IF variable= "&xvar_liste._m1 &xvar_liste._m05"	THEN p1='-1' ;
IF variable= "&xvar_liste._m1 &xvar_liste._0"	THEN p1='-1' ;
IF variable= "&xvar_liste._m1 &xvar_liste._05"	THEN p1='-1' ;
IF variable= "&xvar_liste._m1 &xvar_liste._1"	THEN p1='-1' ;
IF variable= "&xvar_liste._m1 &xvar_liste._2"	THEN p1='-1' ;
IF variable= "&xvar_liste._m1 &xvar_liste._3"	THEN p1='-1' ;

IF variable= "&xvar_liste._m05 &xvar_liste._m0505"	THEN p1='-0.5' ;
IF variable= "&xvar_liste._m05 &xvar_liste._0"		THEN p1='-0.5' ;
IF variable= "&xvar_liste._m05 &xvar_liste._05"		THEN p1='-0.5' ;
IF variable= "&xvar_liste._m05 &xvar_liste._1"		THEN p1='-0.5' ;
IF variable= "&xvar_liste._m05 &xvar_liste._2"		THEN p1='-0.5' ;
IF variable= "&xvar_liste._m05 &xvar_liste._3"		THEN p1='-0.5' ;

IF variable= "&xvar_liste._0 &xvar_liste._00"	THEN p1='0';
IF variable= "&xvar_liste._0 &xvar_liste._05"	THEN p1='0' ;
IF variable= "&xvar_liste._0 &xvar_liste._1"	THEN p1='0' ;
IF variable= "&xvar_liste._0 &xvar_liste._2"	THEN p1='0' ;
IF variable= "&xvar_liste._0 &xvar_liste._3"	THEN p1='0' ;

IF variable= "&xvar_liste._05 &xvar_liste._0505"	THEN p1='0.5' ;
IF variable= "&xvar_liste._05 &xvar_liste._1"		THEN p1='0.5' ;
IF variable= "&xvar_liste._05 &xvar_liste._2"		THEN p1='0.5' ;
IF variable= "&xvar_liste._05 &xvar_liste._3"		THEN p1='0.5' ;

IF variable= "&xvar_liste._1 &xvar_liste._11"	THEN p1='1' ;
IF variable= "&xvar_liste._1 &xvar_liste._2"	THEN p1='1' ;
IF variable= "&xvar_liste._1 &xvar_liste._3"	THEN p1='1' ;

IF variable= "&xvar_liste._2 &xvar_liste._22"	THEN p1='2' ;
IF variable= "&xvar_liste._2 &xvar_liste._3"	THEN p1='2' ;

IF variable= "&xvar_liste._3 &xvar_liste._33"	THEN p1='3' ;



IF variable= "&xvar_liste._m2 &xvar_liste._m22" THEN p2='-2';
IF variable= "&xvar_liste._m2 &xvar_liste._m1"	THEN p2='-1';
IF variable= "&xvar_liste._m2 &xvar_liste._m05"	THEN p2='-0.5';
IF variable= "&xvar_liste._m2 &xvar_liste._0"	THEN p2='0';
IF variable= "&xvar_liste._m2 &xvar_liste._05"	THEN p2='0.5';
IF variable= "&xvar_liste._m2 &xvar_liste._1"	THEN p2='1';
IF variable= "&xvar_liste._m2 &xvar_liste._2"	THEN p2='2';
IF variable= "&xvar_liste._m2 &xvar_liste._3"	THEN p2='3';

IF variable= "&xvar_liste._m1 &xvar_liste._m11"	THEN p2='-1';
IF variable= "&xvar_liste._m1 &xvar_liste._m05"	THEN p2='-0.5';
IF variable= "&xvar_liste._m1 &xvar_liste._0"	THEN p2='0';
IF variable= "&xvar_liste._m1 &xvar_liste._05"	THEN p2='0.5';
IF variable= "&xvar_liste._m1 &xvar_liste._1"	THEN p2='1';
IF variable= "&xvar_liste._m1 &xvar_liste._2"	THEN p2='2';
IF variable= "&xvar_liste._m1 &xvar_liste._3"	THEN p2='3';

IF variable= "&xvar_liste._m05 &xvar_liste._m0505"	THEN p2='-0.5';
IF variable= "&xvar_liste._m05 &xvar_liste._0"		THEN p2='0';
IF variable= "&xvar_liste._m05 &xvar_liste._05"		THEN p2='0.5';
IF variable= "&xvar_liste._m05 &xvar_liste._1"		THEN p2='1';
IF variable= "&xvar_liste._m05 &xvar_liste._2"		THEN p2='2';
IF variable= "&xvar_liste._m05 &xvar_liste._3"		THEN p2='3';

IF variable= "&xvar_liste._0 &xvar_liste._00"	THEN p2='0';
IF variable= "&xvar_liste._0 &xvar_liste._05"	THEN p2='0.5';
IF variable= "&xvar_liste._0 &xvar_liste._1"	THEN p2='1';
IF variable= "&xvar_liste._0 &xvar_liste._2"	THEN p2='2';
IF variable= "&xvar_liste._0 &xvar_liste._3"	THEN p2='3';

IF variable= "&xvar_liste._05 &xvar_liste._0505"	THEN p2='0.5';
IF variable= "&xvar_liste._05 &xvar_liste._1"		THEN p2='1';
IF variable= "&xvar_liste._05 &xvar_liste._2"		THEN p2='2';
IF variable= "&xvar_liste._05 &xvar_liste._3"		THEN p2='3';

IF variable= "&xvar_liste._1 &xvar_liste._11"	THEN p2='1';
IF variable= "&xvar_liste._1 &xvar_liste._2"	THEN p2='2';
IF variable= "&xvar_liste._1 &xvar_liste._3"	THEN p2='3';

IF variable= "&xvar_liste._2 &xvar_liste._22"	THEN p2='2';
IF variable= "&xvar_liste._2 &xvar_liste._3"	THEN p2='3';

IF variable= "&xvar_liste._3 &xvar_liste._33"	THEN p2='3';

RUN;

/*zuerst Original- Ergebnistabelle36 aufteilen*/

DATA split2;
	SET &xvar_liste._ergebnis36;
	IF _N_ < 19 THEN DELETE;
	RENAME p1		=p12;
	RENAME p2		=p22;
	RENAME variable =variable2;
	RENAME wert 	=wert2;
	dummy			=1;
RUN;

DATA split1;
	SET &xvar_liste._ergebnis36;
	IF _N_ > 18 THEN DELETE;
	dummy=1;
RUN;

DATA gemeinsam;
	MERGE split1 split2;
	BY dummy;
	leer='                  ';
RUN;

/*Ergebnistabelle8 in die Royston/Altmann- Polynom-Nomenklatur umwandeln */

DATA &xvar_liste._ergebnis8;
	SET &xvar_liste._ergebnis8;

	IF variable="&&xvar_liste._m2" THEN variable='-2';
	IF variable="&&xvar_liste._m1" THEN variable='-1';
	IF variable="&&xvar_liste._m05" THEN variable='-0.5';
    IF variable="&&xvar_liste._0" THEN variable='0';
	IF variable="&&xvar_liste._05" THEN variable='0.5';
	IF variable="&&xvar_liste._1" THEN variable='1';
	IF variable="&&xvar_liste._2" THEN variable='2';
	IF variable="&&xvar_liste._3" THEN variable='3';

RUN;

PROC PRINT DATA=&xvar_liste._ergebnis8 NOOBS LABEL;
	TITLE1 "Univariate Betrachtung der stetigen Variable";
	TITLE2 "&xvar_liste mit fractional polynomials";
	TITLE3 ;
	TITLE4 ;
	TITLE5 "Modelle vom Grad 1";
		VAR variable wert;
			LABEL variable='p';
			LABEL wert    ='Deviance';
RUN;

PROC PRINT DATA=gemeinsam NOOBS LABEL;
	TITLE1 ;
	TITLE2 ;
	TITLE3 ;
	TITLE4 ;
	TITLE5 "Modelle vom Grad 2";
		VAR p1 p2 wert leer p12 p22 wert2;
			LABEL p1		='P1';
			LABEL p2		='P2';
			LABEL wert    	='Deviance';
			LABEL leer		='|';
			LABEL p12		='P1';
			LABEL p22		='P2';
			LABEL wert2    	='Deviance';

RUN;

DATA ergebnis1;
		function='Ohne';
		comb	='.';
		max		='0';
		diff	='.';
		p		='.';
		dummy	=1;
RUN;

/*Damit Felder fr die Werte gro genug sind*/

DATA ergebnis5;
	    function='                                   ';
		comb	='                                   ';
		max		='                                   ';
		diff	='                                   ';
		p		='                                   ';
		dummy	=1;
RUN;

/*Zusammenfasen*/

DATA ergebnis;
	SET ergebnis5 ergebnis4 ergebnis3 ergebnis2 ergebnis1;
RUN;

/*erste (leere) Zeile lschen*/

DATA ergebnis;
	SET ergebnis;
	IF _N_=1 THEN DELETE;
RUN;

/*Ausgabe der Zusammenfassung*/

/*zuerst: Umwandlung in Royston/Altman- Polynom-Nomenklatur*/

DATA ps;
	SET gemeinsam;
	IF _N_ ne 1 THEN DELETE;
RUN;

DATA ps;
	SET ps;
	CALL SYMPUT('p1',p1);
	CALL SYMPUT('p2',p2);
RUN;

DATA p;
	SET &xvar_liste._ergebnis8;
	IF _N_ ne 1 THEN DELETE;
RUN;

DATA p;
	SET p;
	CALL SYMPUT('p',variable);
RUN;

DATA ergebnis;
	SET ergebnis;
	IF _N_=1 THEN p1="&p1";
	IF _N_=1 THEN p2="&p2";
	IF _N_=2 THEN p1="&p";
	IF _N_=2 THEN p2='.';
	IF _N_=3 THEN p1='.';
	IF _N_=3 THEN p2='.';
	IF _N_=4 THEN p1='.';
	IF _N_=4 THEN p2='.';
RUN;
/*----------------------------*/

/*PROC LOGISTIC mit TEST- Statement, um p- Wert fr Polynome vom Grad 2 zu erhalten!*/

/*zuerst: bestes Polynom vom Grad 2 bestimmen*/

DATA max_poly2;
	SET &xvar_liste._ergebnis36;
	IF _N_=1;
	CALL SYMPUT ('maxpoly2',variable);
RUN;

ODS LISTING CLOSE;
ODS OUTPUT TestStmts=&xvar_liste._stmts2;

PROC LOGISTIC DATA= fracpoly NAMELEN=32;
MODEL &resp_var (

%IF &event=1 %THEN
	%DO;
		event='1'
%END;

%IF &event=0 %THEN
	%DO;
		event='0'
%END;

				)
= &maxpoly2 / SELECTION=NONE;

%LET finalteststring=;

%DO klo=1 %TO 2;

%LET part=%QSCAN(&maxpoly2,&klo,%STR( ));

%IF &klo=1 %THEN %DO;

%LET finalteststring=&part;

%END;
%ELSE %DO;

%LET finalteststring=&finalteststring = &part;

%END;


%END;

TestGrad2&xvar_liste:TEST &finalteststring=0;
RUN;

ODS OUTPUT CLOSE;
ODS LISTING;

/*PROC LOGISTIC mit TEST- Statement, um p- Wert fr Polynome vom Grad 1 zu erhalten!*/

/*zuerst: bestes Polynom vom Grad 1 bestimmen*/

DATA max_poly1;
	SET ergebnis3;
	IF _N_=1;
	CALL SYMPUT ('maxpoly1',comb);
RUN;

ODS LISTING CLOSE;
ODS OUTPUT TestStmts=&xvar_liste._stmts1;

PROC LOGISTIC DATA= fracpoly NAMELEN=32;
MODEL &resp_var (

%IF &event=1 %THEN
	%DO;
		event='1'
%END;

%IF &event=0 %THEN
	%DO;
		event='0'
%END;

				)
= &maxpoly1 / SELECTION=NONE;

TestGrad1&xvar_liste:TEST &maxpoly1=0;
RUN;

ODS OUTPUT CLOSE;
ODS LISTING;

/*Ergebnistabellen zusammenfhren*/

DATA &xvar_liste._stmts_gesamt;
	SET &xvar_liste._stmts1 &xvar_liste._stmts2;
RUN;

/*----------------------------*/

PROC PRINT DATA=ergebnis NOOBS LABEL;
	TITLE1 "Zusammenfassung: Univariate fractional polynomials ";
	TITLE2 "Variable: &xvar_liste";
	TITLE3 ;
	TITLE4 ;
		VAR function p1 p2 max diff p;
			LABEL function='Modell';
			LABEL p1	  ='P1';
			LABEL p2	  ='P2';
			LABEL max	  ='Deviance';
			LABEL diff	  ='Differenz';
			LABEL p		  ='ChiSq p-Value';
RUN;

PROC PRINT DATA=&xvar_liste._stmts_gesamt NOOBS LABEL;
	TITLE1 'Univariate logistische Regressionen mit den besten';
	TITLE2 'Fractional Polynomials vom Grad 1 und 2';
	TITLE3 ;
	TITLE4 "Test Grad 1: %QCMPRES(&maxpoly1)=0";
	TITLE5 "Test Grad 2: %QCMPRES(&finalteststring)=0";
	
		VAR label waldchisq df probchisq;
		LABEL label			='Label';
		LABEL waldchisq		='Wald Chi-Square';
		LABEL df			='DF';
		LABEL probchisq		='Pr > ChiSq';
RUN;

/*berflssige Dateien lschen*/

PROC DATASETS NOLIST;
	DELETE &xvar_liste._ergebnis8  &xvar_liste._ergebnis36 &xvar_liste._p_value1
           &xvar_liste._p_value2 &xvar_liste._p_value3
		   ergebnis1 ergebnis2 ergebnis3 ergebnis4	ergebnis5 ergebnis
		   gemeinsam split1 split2 p ps max_poly2 max_poly1 &xvar_liste._stmts1 &xvar_liste._stmts2
		   &xvar_liste._stmts_gesamt temp
;
RUN;
QUIT;

%negativ:

%END;


PROC DATASETS NOLIST;
	DELETE fracpoly;
RUN;
QUIT;

%MEND FRACPOLY;