/************************************************************************************/
/*																				  	*/
/*	NAME:		jackknife_crossvalidation.mac.sas									*/
/*  TITLE: 		Jackknife-Crossvalidierung (approximativ/genau)						*/
/*  KEYS: 		crossvalidation, jackknife-crossvalidation, validation, 			*/
/*			 	logistic regression, roc analysis									*/
/*	VERSION:	SAS 8.2																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		LOGISTIC															*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\jackknife_crossvalidation.mac.sas	*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*	Programmierer:					Christoph Ziegler								*/
/*	Datum der Ersterstellung:		3/2003											*/
/*  Aktuelle Makro- Version:		12/2003											*/
/*																					*/
/*	Kurzbeschreibung:				Allgemeines Makro zur Validierung von 			*/
/*							    	logistischen Regressionsmodellen mittels 		*/
/*									Jackknife-Crossvalidierung (genau  				*/
/*									("Leaving One Out")	& approximativ 				*/
/*									("PROC LOGISTIC OPTION PREDPROBS=X"))			*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_CROSSVALIDATION.MAC.SAS						*/
/*																					*/
/*  Genauere Beschreibung siehe:	PM_CROSSVALIDATION.MAC.SAS						*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*									pm_logreg.mac.sas								*/
/*									pm_roc.mac.sas									*/
/*									reclass.mac.sas									*/
/*																					*/
/*	Bentigte Dateien:				Datensatz (Parameter 'data=')					*/
/*																					*/
/*																					*/
/*  Erzeugte Dateien:				-----											*/
/*																					*/
/*  Programmnderungen				Sandra Mller, 04.05.2016, Makrovariable		*/
/*  und -ergnzungen 				binwidth hinzugefgt							*/ 
/*  (Name, Datum, Beschreibung):											 		*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%MACRO JACKKNIFE_CROSSVALIDATION(		data				= ,
										resp_var			= , 
										event				= 1,
										xvar				= ,
										cvar				= ,
										ref					= ,
										miss 				= 0,
										exact				= 0,
										roc_auto			= 1,
										macro_path			= ,
										binwidth			= 0
	            			      );

/*Externe Makros, die innerhalb des Crossvalidierungs- Makros verwendet werden*/
/*(Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('pm_logreg.mac.sas');
%INCLUDE y ('pm_roc.mac.sas');
%INCLUDE y ('reclass.mac.sas');

%GLOBAL nnnn1;
%LET datensatzz=&data;

/*1. Approximative Berechnung (mittels PROC LOGISTIC- Option PREDPROBS=X):*/

/*Output unterdrcken:*/

/*Logistic ber Ausgangsdatensatz*/

ODS SELECT NONE;

%PM_LOGREG ( 	data			=&data,
				resp_var		=&resp_var,
				event			=&event,
				xvar			=&xvar,
				cvar			=&cvar,
				ref				=&ref,
				fxvar			=,
				fcvar			=,
				miss			=&miss,
				sel				=NONE,
				sel_details		= 1,
				slstay			=0.05,
				slentry			=0.05,
				clodds			=0,
				alpha			=0.05,
				rsquare			=1,
				ctable			=0,
				ctable_pprob	=,
				out_pred		=work.gof_ori,
				pred			=1,
				predprobs_art	=INDIVIDUAL,
				roc				=1,
				out_roc			=work.roc_ori,
				est				=0,
				out_est			=,
				inest_flag		=0,
				inest_data		=,
				boot_data		=,
				c_wert			=1,
				c_data			=work.association_ori,
				test			=,
				mi				=0,
				out_mi			=work.out_mi_log,
				macro_path		=&macro_path,
				binwidth		=&binwidth
		);

/*ROC-Analyse (Originaldatensatz) mit Makro %ROC_KURVE*/

%PM_ROC			(
				 data				=&data,
				 resp_var			=&resp_var,
				 event				=&event,
				 data_roc			=work.roc_ori,
				 data_pred			=work.gof_ori,
				 data_asso			=work.association_ori,
				 validation			=CROSSVALIDATION,
				 praevalenz			= ,
				 all_cp				=0,
			     cp					=,
				 cibinom			=0,
				 rocdat				=0,
				 auroc				=0,
				 printit			=0,
				 alpha_cl			=0.95,
				 ext				=tiffp,
				 winkel				=1,
				 point				=1,
				 path				='c:\temp',
				 mi					=0,
				 out_mi				=work.out_mi_roc,
				 macro_path			=&macro_path,
				 binwidth			=&binwidth
			    );

/*Variablen umbenennen:*/
/*ROC- / Sonstige Kenngren des Originaldatensatzes zusammenfassen*/

DATA result_roc_ori1;
	SET result;
	RENAME maxyouden 	= maxyouden_ori;
	RENAME _PROB_ 		= prob_ori;
	RENAME _SENSIT_		= sens_ori;
	RENAME spez 		= spez_ori;
	RENAME ppv 			= ppv_ori;
	RENAME npv 			= npv_ori;
	RENAME psep 		= psep_ori;
	dummy				= 1;
RUN;

DATA result_roc_ori2;
	SET index_ori;
	dummy				= 1;
	emax_ori			= 0;
	slope_ori			= 1;
RUN;

/*Cutpoint des Ausgangsdatensatzes zur spteren Verwendung in einer Makrovariable */
/*zwischenspeichern*/

DATA result_roc_ori1;
	SET result_roc_ori1;
	call symput('prob_original',prob_ori);
RUN;

%IF &exact=0 %THEN %DO;

/*PREDPROBS=X (CROSSVALIDATE) rausschreiben*/

%PM_LOGREG ( 	data			=&data,
				resp_var		=&resp_var,
				event			=&event,
				xvar			=&xvar,
				cvar			=&cvar,
				ref				=&ref,
				fxvar			=,
				fcvar			=,
				miss			=&miss,
				sel				=NONE,
				sel_details		= 1,
				slstay			=0.05,
				slentry			=0.05,
				clodds			=0,
				alpha			=0.05,
				rsquare			=1,
				ctable			=0,
				ctable_pprob	=,
				out_pred		=work.gof_orix,
				pred			=1,
				predprobs_art	=X,
				roc				=1,
				out_roc			=work.roc_orix,
				est				=0,
				out_est			=,
				inest_flag		=0,
				inest_data		=,
				boot_data		=,
				c_wert			=1,
				c_data			=work.association_orix,
				test			=,
				mi				=0,
				out_mi			=work.out_mi_log,
				macro_path		=&macro_path,
				binwidth		=&binwidth
		);


/*Automatische ROC- Tabelle fr ROC- Analyse zuweisen*/

%IF &roc_auto=1 %THEN %DO;

%LET roc_data=roc_orix;

%END;

/*Falls manuelle Generierung der ROC Tabelle erwnscht Aufruf der Macros RECLASS*/
/*sowie Zuweisung der Ergebnisdatei fr ROC- Analyse*/

%IF &roc_auto=0 %THEN %DO;

%RECLASS (	macro_path	=&macro_path,
			pred_data	=gof_orix,
			resp_var	=&resp_var,
			prob		=XP_&event
		 );

%LET roc_data=ergebnis;

%END;

/*ROC- Analyse: */

%PM_ROC			(
				 data				=&data,
				 resp_var			=&resp_var,
				 event				=&event,
				 data_roc			=&roc_data,
				 data_pred			=work.gof_orix,
				 data_asso			=work.association_orix,
				 validation			=CROSSVALIDATION,
				 praevalenz			= ,
				 all_cp				=0,
			     cp					=&prob_original,
				 cibinom			=0,
				 rocdat				=0,
				 auroc				=0,
				 printit			=0,
				 alpha_cl			=0.95,
				 ext				=tiffp,
				 winkel				=1,
				 point				=1,
				 path				='c:\temp',
				 mi					=0,
				 out_mi				=work.out_mi_roc,
				 macro_path			=&macro_path,
			 	 binwidth			=&binwidth
			    );

/*Output wieder aktivieren*/

ODS SELECT ALL;

/*ROC- / Sonstige Kenngren der Crossvalidierung zusammenfassen*/

/*Cutpoint-abhngige Prognosegtemae*/

DATA result_roc_cross1;
	SET result;
	RENAME maxyouden 	= maxyouden_cross;
	RENAME _PROB_ 		= prob_cross;
	RENAME _SENSIT_		= sens_cross;
	RENAME spez 		= spez_cross;
	RENAME ppv 			= ppv_cross;
	RENAME npv 			= npv_cross;
	RENAME psep 		= psep_cross;
	dummy				= 1;
RUN;

/*Neuer optimaler Cutpoint zur spteren Verwendung in einer Makrovariable */
/*zwischenspeichern*/

DATA result_roc_cross1;
	SET result_roc_cross1;
	call symput('prob_cross',prob_cross);
RUN;

/*Cutpoint- unabhngige Prognosegtemae:*/

DATA result_roc_cross2;
	SET index_ori;
	slope_ori=1;
	emax_ori=0;
	RENAME	brier_ori		= brier_cross;
	RENAME	emax_ori		= emax_cross;			
	RENAME  max_resca_ori	= max_resca_cross;
	RENAME  r_square_ori	= rsquare_cross;
	RENAME	auc_ori			= auc_cross;
	RENAME  slope_ori		= slope_cross;
	RENAME  som_ori			= som_cross;
	dummy					= 1;
RUN;

/*Cutpoint-abhngige Prognosegtemae (beim Cutpoint des Ausgangsmodells)*/

DATA result_roc_cross3 (DROP=_POS_ _NEG_ _FALNEG_ _FALPOS_);
	SET result2;
	RENAME	_SENSIT_		= sensit_cross_cp;
	RENAME	spez			= spez_cross_cp;			
	RENAME  youden 			= youden_cross_cp;
	RENAME  ppv				= ppv_cross_cp;
	RENAME	npv				= npv_cross_cp;
	RENAME  psep			= psep_cross_cp;
	dummy					= 1;
RUN;

/*Ergebnisdateien zusammenfhren:*/
/*1. Cutpoint-unabhngige Prognosegtemae:*/

DATA gesamt1;
	MERGE result_roc_cross2 result_roc_ori2;
	BY dummy;
RUN;

/*2. Cutpoint-abhngige Prognosegtemae:*/

DATA gesamt2;
	MERGE result_roc_cross1 result_roc_ori1;
	BY dummy;
RUN;

/*3. Prognosegtemae beim Cutpoint des Ausgangsmodells:*/

DATA gesamt3;
	MERGE result_roc_cross3 result_roc_ori1;
	BY dummy;
RUN;

/*Ergebnisdatei gesamt1 fr schneren Output aufsplitten*/

/*AUC*/

DATA auc (KEEP=auc_ori auc_cross auc_diff auc_percent);
	SET gesamt1;
	auc_diff=auc_ori-auc_cross;
	auc_percent=(auc_ori-auc_cross)/auc_ori;
RUN;

/*SomersD*/

DATA som (KEEP=som_ori som_cross som_diff som_percent);
	SET gesamt1;
	som_diff=som_ori-som_cross;
	som_percent=(som_ori-som_cross)/som_ori;
RUN;

/*Brier Score*/

DATA brier (KEEP=brier_ori brier_cross brier_diff brier_percent);
	SET gesamt1;
	brier_diff=brier_ori-brier_cross;
	brier_percent=(brier_ori-brier_cross)/brier_ori;
RUN;

/*Slope*/

DATA slope (KEEP=slope_ori slope_cross slope_diff slope_percent);
	SET gesamt1;
	slope_diff=slope_ori-slope_cross;
	slope_percent=(slope_ori-slope_cross)/slope_ori;
RUN;

/*EMax*/

DATA emax (KEEP=emax_ori emax_cross emax_diff emax_percent);
	SET gesamt1;
	emax_diff=emax_ori-emax_cross;
	emax_percent=(emax_ori-emax_cross)/emax_ori;
RUN;

/*R-Square*/

DATA rsquare (KEEP=r_square_ori rsquare_cross rsquare_diff rsquare_percent);
	SET gesamt1;
	rsquare_diff=r_square_ori-rsquare_cross;
	rsquare_percent=(r_square_ori-rsquare_cross)/r_square_ori;
RUN;

/*MaxRescaled R-Square*/

DATA maxresca (KEEP=max_resca_ori max_resca_cross max_resca_diff max_resca_percent);
	SET gesamt1;
	max_resca_diff=max_resca_ori-max_resca_cross;
	max_resca_percent=(max_resca_ori-max_resca_cross)/max_resca_ori;
RUN;

/*++++++++++++++++++++*/

/*Standardoutput ausgeben*/

/*Fallzahl bestimmen:*/

%IF &miss=1 %THEN %DO;

%NUMBEROBS(datensatz=&datensatzz);

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/
%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/


		DATA work.fallzahl;
		SET  &datensatzz;
		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

		RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatzz      ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE standardoutput fallzahl;
RUN;
QUIT;

/*++++++++++++++++++++*/


/*Ergebnisse (gesamt1)ausgeben*/

PROC PRINT DATA= auc NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR auc_ori auc_cross auc_diff auc_percent; 
    TITLE1 'Crossvalidierung-Zusammenfassung:';
	TITLE2 "Variante: Approximativ mittels SAS- PROC LOGISTIC- OPTION 'PREDPROBS=X'";
	TITLE3 'O=Original / J=Jackknife-Crossvalidiert / D=Differenz (Original- JackknifeCrossvalidiert)';
	TITLE4 'Cutpoint-unabhngige Prognosegtemae:';
	LABEL 	auc_ori			='AUC O' 
			auc_cross 		='AUC J'
			auc_diff 		='AUC D'
			auc_percent		='%';
RUN;

PROC PRINT DATA= som NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR som_ori som_cross som_diff som_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	som_ori 		='SomersD O'
			som_cross 		='SomersD J'
			som_diff 		='SomersD D'
			som_percent		='%';
RUN;

PROC PRINT DATA= brier NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  brier_ori brier_cross brier_diff brier_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	brier_ori 		='Brier Score O'
			brier_cross 	='Brier Score J'
			brier_diff 		='Brier Score D'
			brier_percent	='%';
RUN;

PROC PRINT DATA= slope NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  slope_ori slope_cross slope_diff slope_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	slope_ori 		='Slope O'
			slope_cross 	='Slope J'
			slope_diff 		='Slope D'
			slope_percent	='%';
RUN;

PROC PRINT DATA= emax NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR emax_ori emax_cross emax_diff emax_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	emax_ori 		='Emax O'
			emax_cross 		='Emax J'
			emax_diff 		='Emax D'
			emax_percent	='%';
RUN;

PROC PRINT DATA= rsquare NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR r_square_ori rsquare_cross rsquare_diff rsquare_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	r_square_ori 		='R-Square O'
			rsquare_cross 		='R-Square J'
			rsquare_diff 		='R-Square D'
			rsquare_percent		='%';
RUN;

PROC PRINT DATA= maxresca NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR max_resca_ori max_resca_cross max_resca_diff max_resca_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	max_resca_ori 		='Max-Rescaled R-Square O'
			max_resca_cross 	='Max-Rescaled R-Square J'
			max_resca_diff 		='Max-Rescaled R-Square D'
			max_resca_percent	='%';
			
RUN;

/*Ergebnisdatei gesamt3 fr schneren Output aufsplitten*/

/*Sensitivitt*/

DATA sens (KEEP=sens_ori sensit_cross_cp sens_diff sens_percent);
	SET gesamt3;
	sens_diff=sens_ori-sensit_cross_cp;
	sens_percent=(sens_ori-sensit_cross_cp)/sens_ori;
RUN;

/*Spezifitt*/

DATA spez (KEEP=spez_ori spez_cross_cp spez_diff spez_percent);
	SET gesamt3;
	spez_diff=spez_ori-spez_cross_cp;
	spez_percent=(spez_ori-spez_cross_cp)/spez_ori;
RUN;

/*YoudenIndex*/

DATA youden (KEEP=maxyouden_ori youden_cross_cp youden_diff youden_percent);
	SET gesamt3;
	youden_diff=maxyouden_ori-youden_cross_cp;
	youden_percent=(maxyouden_ori-youden_cross_cp)/maxyouden_ori;
RUN;

/*PPV*/

DATA ppv (KEEP=ppv_ori ppv_cross_cp ppv_diff ppv_percent);
	SET gesamt3;
	ppv_diff=ppv_ori-ppv_cross_cp;
	ppv_percent=(ppv_ori-ppv_cross_cp)/ppv_ori;
RUN;

/*NPV*/

DATA npv (KEEP=npv_ori npv_cross_cp npv_diff npv_percent);
	SET gesamt3;
	npv_diff=npv_ori-npv_cross_cp;
	npv_percent=(npv_ori-npv_cross_cp)/npv_ori;
RUN;

/*PSEP*/

DATA psep (KEEP=psep_ori psep_cross_cp psep_diff psep_percent);
	SET gesamt3;
	psep_diff=psep_ori-psep_cross_cp;
	psep_percent=(psep_ori-psep_cross_cp)/psep_ori;
RUN;

/*Ergebnisse (gesamt3) ausgeben*/

PROC PRINT DATA= sens NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR sens_ori sensit_cross_cp sens_diff sens_percent; 
	TITLE1;
	TITLE2;
	TITLE3 'Cutpoint-abhngige Prognosegtemae';
	TITLE4 "(bei Cutpoint des Ausgangsmodells (cp=&prob_original):";
	LABEL 	sens_ori		='Sensitivitt O' 
			sensit_cross_cp	='Sensitivitt J'
			sens_diff 		='Sensitivitt D'
			sens_percent	='%';
RUN;

PROC PRINT DATA= spez NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR spez_ori spez_cross_cp spez_diff spez_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	spez_ori 		='Spezifitt O'
			spez_cross_cp 	='Spezifitt J'
			spez_diff 		='Spezifitt D'
			spez_percent	='%';
RUN;

PROC PRINT DATA= youden NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR maxyouden_ori youden_cross_cp youden_diff youden_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	maxyouden_ori 	='Max. YoudenIndex O'
			youden_cross_cp	='YoudenIndex J'
			youden_diff 	='YoudenIndex D'
			youden_percent	='%';
RUN;

PROC PRINT DATA= ppv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR ppv_ori ppv_cross_cp ppv_diff ppv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	ppv_ori			='Positive Korrektheit O'
			ppv_cross_cp 	='Positive Korrektheit J'
			ppv_diff 		='Positive Korrektheit D'
			ppv_percent		='%';
RUN;

PROC PRINT DATA= npv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  npv_ori npv_cross_cp npv_diff npv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	npv_ori 		='Negative Korrektheit O'
			npv_cross_cp 	='Negative Korrektheit J'
			npv_diff 		='Negative Korrektheit D'
			npv_percent		='%';
RUN;

PROC PRINT DATA= psep NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR psep_ori psep_cross_cp psep_diff psep_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	psep_ori		='Separation Index PSEP O'
			psep_cross_cp 	='Separation Index PSEP J'
			psep_diff 		='Separation Index PSEP D'
			psep_percent	='%';
RUN;

/*Ergebnisdatei gesamt2 fr schneren Output aufsplitten*/

/*Sensitivitt*/

DATA sens (KEEP=sens_ori sens_cross sens_diff sens_percent);
	SET gesamt2;
	sens_diff=sens_ori-sens_cross;
	sens_percent=(sens_ori-sens_cross)/sens_ori;
RUN;

/*Spezifitt*/

DATA spez (KEEP=spez_ori spez_cross spez_diff spez_percent);
	SET gesamt2;
	spez_diff=spez_ori-spez_cross;
	spez_percent=(spez_ori-spez_cross)/spez_ori;
RUN;

/*YoudenIndex*/

DATA youden (KEEP=maxyouden_ori maxyouden_cross maxyouden_diff maxyouden_percent);
	SET gesamt2;
	maxyouden_diff=maxyouden_ori-maxyouden_cross;
	maxyouden_percent=(maxyouden_ori-maxyouden_cross)/maxyouden_ori;
RUN;

/*PPV*/

DATA ppv (KEEP=ppv_ori ppv_cross ppv_diff ppv_percent);
	SET gesamt2;
	ppv_diff=ppv_ori-ppv_cross;
	ppv_percent=(ppv_ori-ppv_cross)/ppv_ori;
RUN;

/*NPV*/

DATA npv (KEEP=npv_ori npv_cross npv_diff npv_percent);
	SET gesamt2;
	npv_diff=npv_ori-npv_cross;
	npv_percent=(npv_ori-npv_cross)/npv_ori;
RUN;

/*PSEP*/

DATA psep (KEEP=psep_ori psep_cross psep_diff psep_percent);
	SET gesamt2;
	psep_diff=psep_ori-psep_cross;
	psep_percent=(psep_ori-psep_cross)/psep_ori;
RUN;

/*Ergebnisse (gesamt2) ausgeben*/

PROC PRINT DATA= sens NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR sens_ori sens_cross sens_diff sens_percent; 
	TITLE1;
	TITLE2;
	TITLE3 'Cutpoint-abhngige Prognosegtemae';
	TITLE4 "(bei neuem, optimalem Cutpoint (cp=&prob_cross):";
	LABEL 	sens_ori		='Sensitivitt O' 
			sens_cross		='Sensitivitt J'
			sens_diff 		='Sensitivitt D'
			sens_percent	='%';
RUN;

PROC PRINT DATA= spez NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR spez_ori spez_cross spez_diff spez_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	spez_ori 		='Spezifitt O'
			spez_cross	 	='Spezifitt J'
			spez_diff 		='Spezifitt D'
			spez_percent	='%';
RUN;

PROC PRINT DATA= youden NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR maxyouden_ori maxyouden_cross maxyouden_diff maxyouden_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	maxyouden_ori 		='Max. YoudenIndex O'
			maxyouden_cross		='YoudenIndex J'
			maxyouden_diff 		='YoudenIndex D'
			maxyouden_percent	='%';
RUN;

PROC PRINT DATA= ppv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR ppv_ori ppv_cross ppv_diff ppv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	ppv_ori			='Positive Korrektheit O'
			ppv_cross	 	='Positive Korrektheit J'
			ppv_diff 		='Positive Korrektheit D'
			ppv_percent		='%';
RUN;

PROC PRINT DATA= npv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  npv_ori npv_cross npv_diff npv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	npv_ori 		='Negative Korrektheit O'
			npv_cross	 	='Negative Korrektheit J'
			npv_diff 		='Negative Korrektheit D'
			npv_percent		='%';
RUN;

PROC PRINT DATA= psep NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR psep_ori psep_cross psep_diff psep_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	psep_ori		='Separation Index PSEP O'
			psep_cross		='Separation Index PSEP J'
			psep_diff 		='Separation Index PSEP D'
			psep_percent	='%';
RUN;

/*Temporre Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE rsquare youden spez som sens roc_ori
		   result_roc_cross1 result_roc_cross2 result_roc_cross3 result2 result psep ppv
 		   gesamt2 npv maxresca index_ori gof_ori slope
		   gesamt3 gesamt1 emax brier auc association_ori;

%IF &exact=0 %THEN %DO;

    DELETE result_roc_ori2 result_roc_ori1;

%END;

	RUN;
QUIT;

%END;

DM "clear log;";

/*Falls genaue Berechnung erwnscht:*/

%IF &exact=1 %THEN %DO;

/*2. Genaue Berechnung:*/

/*Anzahl der Beobachtungen im angegebenen Datensatz bestimmen:*/

%NUMBEROBS(datensatz=&datensatzz);

%LET countera=&nnnn1;

/*Ergebnisdatei nullinitialisieren*/

DATA gof_cross_gesamt; SET _NULL_;

/*Schleife*/

%DO xyzb=1 %TO &countera;

/*neue Beobachtungsnummer einfhren*/

DATA tmp;
	SET &datensatzz;
	_obsneuxy_=_N_;
RUN;

DATA tmp2;
	SET tmp;
	IF _obsneuxy_=&xyzb THEN &resp_var=.;
RUN;

/*Output ausschalten*/

ODS SELECT NONE;

/*PROC LOGISTIC- Aufruf*/

%PM_LOGREG ( 	data			= tmp2,
				resp_var		= &resp_var,
				event			= &event,
				xvar			= &xvar,
				cvar			= &cvar,
				ref				= &ref,
				fxvar			= ,
				fcvar			= ,
				miss			= 1,
				sel				= NONE,
				sel_details		= 1,
				slstay			= 0.05,
				slentry			= 0.05,
				clodds			= 0,
				alpha			= 0.05,
				rsquare			= 1,
				ctable			= 0,
				ctable_pprob	= ,
				out_pred		= work.gof_cross,
				pred			= 1,
				predprobs_art	=INDIVIDUAL,
				roc				= 1,
				out_roc			= work.roc_cross,
				est				= 0,
				out_est			= ,
				inest_flag		= 0,
				inest_data		= ,
				boot_data		= ,
				c_wert			= 1,
				c_data			= work.association_cross,
				test			=,
				mi				=0,
				out_mi			=work.out_mi_log,
				macro_path		= &macro_path,
				binwidth		=&binwidth
		);

/*Wahrscheinlichkeit fr Patient i festhalten:*/

DATA gof_cross;
	SET gof_cross;
	IF _obsneuxy_ ne &xyzb THEN DELETE;
RUN;

/*An Ergebnisdatei anhngen*/

DATA gof_cross_gesamt;
	SET gof_cross_gesamt gof_cross;
RUN;

DM "clear log;";

%END;

/*Werte der Zielgre wieder einsetzen.*/

DATA gof_cross_gesamt (DROP=&resp_var);
	SET gof_cross_gesamt;
RUN;

DATA tmp;
	SET &datensatzz;
	_obsneuxy_=_N_;
RUN;

DATA ergebnis2;
	MERGE tmp gof_cross_gesamt;
	BY _obsneuxy_;
RUN;

/*Automatische ROC- Tabelle fr ROC- Analyse zuweisen*/

%IF &roc_auto=1 %THEN %DO;

%LET roc_data=roc_cross;

%END;

/*Falls manuelle Generierung der ROC Tabelle erwnscht Aufruf der Macros RECLASS*/
/*sowie Zuweisung der Ergebnisdatei fr ROC- Analyse*/

%IF &roc_auto=0 %THEN %DO;

%RECLASS (	macro_path	= &macro_path,
			pred_data	= ergebnis2,
			resp_var	= &resp_var,
			prob		= IP_&event
		 );

%LET roc_data=ergebnis;

%END;


/*ROC- Analyse: */

%PM_ROC			(
				 data				=&datensatzz,
				 resp_var			=&resp_var,
				 event				=&event,
				 data_roc			=&roc_data,
				 data_pred			=work.ergebnis2,
				 data_asso			=work.association_cross,
				 validation			=CROSSVALIDATION,
				 praevalenz			= ,
				 all_cp				=0,
			     cp					=&prob_original,
				 cibinom			=0,
				 rocdat				=0,
				 auroc				=0,
				 printit			=0,
				 alpha_cl			=0.95,
				 ext				=tiffp,
				 winkel				=1,
				 point				=1,
				 path				='c:\temp',
				 mi					=0,
				 out_mi				=work.out_mi_roc,
				 macro_path			=&macro_path,
				 binwidth			=&binwidth
			    );

/*Output wieder aktivieren*/

ODS SELECT ALL;

/*ROC- / Sonstige Kenngren der Crossvalidierung zusammenfassen*/

/*Cutpoint-abhngige Prognosegtemae*/

DATA result_roc_cross1;
	SET result;
	RENAME maxyouden 	= maxyouden_cross;
	RENAME _PROB_ 		= prob_cross;
	RENAME _SENSIT_		= sens_cross;
	RENAME spez 		= spez_cross;
	RENAME ppv 			= ppv_cross;
	RENAME npv 			= npv_cross;
	RENAME psep 		= psep_cross;
	dummy				= 1;
RUN;

/*Cutpoint- unabhngige Prognosegtemae:*/

DATA result_roc_cross2;
	SET index_ori;
	RENAME	brier_ori		= brier_cross;
	RENAME	emax_ori		= emax_cross;	
	RENAME  slope_ori		= slope_cross;
	RENAME  max_resca_ori	= max_resca_cross;
	RENAME  r_square_ori	= rsquare_cross;
	RENAME	auc_ori			= auc_cross;
	RENAME  som_ori			= som_cross;
	dummy					= 1;
RUN;

/*Cutpoint-abhngige Prognosegtemae (beim Cutpoint des Ausgangsmodells)*/

DATA result_roc_cross3 (DROP=_POS_ _NEG_ _FALNEG_ _FALPOS_);
	SET result2;
	RENAME	_SENSIT_		= sensit_cross_cp;
	RENAME	spez			= spez_cross_cp;			
	RENAME  youden 			= youden_cross_cp;
	RENAME  ppv				= ppv_cross_cp;
	RENAME	npv				= npv_cross_cp;
	RENAME  psep			= psep_cross_cp;
	dummy					= 1;
RUN;

/*Ergebnisdateien zusammenfhren:*/
/*1. Cutpoint-unabhngige Prognosegtemae:*/

DATA gesamt1;
	MERGE result_roc_cross2 result_roc_ori2;
	BY dummy;
RUN;

/*2. Cutpoint-abhngige Prognosegtemae:*/

DATA gesamt2;
	MERGE result_roc_cross1 result_roc_ori1;
	BY dummy;
RUN;

/*3. Prognosegtemae beim Cutpoint des Ausgangsmodells:*/

DATA gesamt3;
	MERGE result_roc_cross3 result_roc_ori1;
	BY dummy;
RUN;

/*Ergebnisdatei gesamt1 fr schneren Output aufsplitten*/

/*AUC*/

DATA auc (KEEP=auc_ori auc_cross auc_diff auc_percent);
	SET gesamt1;
	auc_diff=auc_ori-auc_cross;
	auc_percent=(auc_ori-auc_cross)/auc_ori;
RUN;

/*SomersD*/

DATA som (KEEP=som_ori som_cross som_diff som_percent);
	SET gesamt1;
	som_diff=som_ori-som_cross;
	som_percent=(som_ori-som_cross)/som_ori;
RUN;

/*Brier Score*/

DATA brier (KEEP=brier_ori brier_cross brier_diff brier_percent);
	SET gesamt1;
	brier_diff=brier_ori-brier_cross;
	brier_percent=(brier_ori-brier_cross)/brier_ori;
RUN;

/*Slope*/

DATA slope (KEEP=slope_ori slope_cross slope_diff slope_percent);
	SET gesamt1;
	slope_diff=slope_ori-slope_cross;
	slope_percent=(slope_ori-slope_cross)/slope_ori;
RUN;

/*EMax*/

DATA emax (KEEP=emax_ori emax_cross emax_diff emax_percent);
	SET gesamt1;
	emax_diff=emax_ori-emax_cross;
	emax_percent=(emax_ori-emax_cross)/emax_ori;
RUN;

/*R-Square*/

DATA rsquare (KEEP=r_square_ori rsquare_cross rsquare_diff rsquare_percent);
	SET gesamt1;
	rsquare_diff=r_square_ori-rsquare_cross;
	rsquare_percent=(r_square_ori-rsquare_cross)/r_square_ori;
RUN;

/*MaxRescaled R-Square*/

DATA maxresca (KEEP=max_resca_ori max_resca_cross max_resca_diff max_resca_percent);
	SET gesamt1;
	max_resca_diff=max_resca_ori-max_resca_cross;
	max_resca_percent=(max_resca_ori-max_resca_cross)/max_resca_ori;
RUN;

/*++++++++++++++++++++*/

/*Standardoutput ausgeben*/

/*Fallzahl bestimmen:*/

%IF &miss=1 %THEN %DO;

%NUMBEROBS(datensatz=&datensatzz);

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/
%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/


		DATA work.fallzahl;
		SET  &datensatzz;
		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

		RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatzz      ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE standardoutput fallzahl;
RUN;
QUIT;

/*++++++++++++++++++++*/

/*Ergebnisse (gesamt1)ausgeben*/

PROC PRINT DATA= auc NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR auc_ori auc_cross auc_diff auc_percent; 
    TITLE1 'Crossvalidierung-Zusammenfassung:';
	TITLE2 "Variante: Genaue Berechnung mittels 'Leaving-One-Out'";
	TITLE3 'O=Original / J=Jackknife-Crossvalidiert /D=Different (Original-JackknifeCrossval.)';
	TITLE4 'Cutpoint-unabhngige Prognosegtemae:';
	LABEL 	auc_ori			='AUC O' 
			auc_cross 		='AUC J'
			auc_diff 		='AUC D'
			auc_percent		='%';
RUN;

PROC PRINT DATA= som NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR som_ori som_cross som_diff som_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	som_ori 		='SomersD O'
			som_cross 		='SomersD J'
			som_diff 		='SomersD D'
			som_percent		='%';
RUN;

PROC PRINT DATA= brier NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  brier_ori brier_cross brier_diff brier_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	brier_ori 		='Brier Score O'
			brier_cross 	='Brier Score J'
			brier_diff 		='Brier Score D'
			brier_percent	='%';
RUN;

PROC PRINT DATA= slope NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  slope_ori slope_cross slope_diff slope_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	slope_ori 		='Slope O'
			slope_cross 	='Slope J'
			slope_diff 		='Slope D'
			slope_percent	='%';
RUN;

PROC PRINT DATA= emax NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR emax_ori emax_cross emax_diff emax_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	emax_ori 		='Emax O'
			emax_cross 		='Emax J'
			emax_diff 		='Emax D'
			emax_percent	='%';
RUN;

PROC PRINT DATA= rsquare NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR r_square_ori rsquare_cross rsquare_diff rsquare_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	r_square_ori 		='R-Square O'
			rsquare_cross 		='R-Square J'
			rsquare_diff 		='R-Square D'
			rsquare_percent		='%';
RUN;

PROC PRINT DATA= maxresca NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR max_resca_ori max_resca_cross max_resca_diff max_resca_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	max_resca_ori 		='Max-Rescaled R-Square O'
			max_resca_cross 	='Max-Rescaled R-Square J'
			max_resca_diff 		='Max-Rescaled R-Square D'
			max_resca_percent	='%';
			
RUN;

/*Ergebnisdatei gesamt3 fr schneren Output aufsplitten*/


/*Sensitivitt*/

DATA sens (KEEP=sens_ori sensit_cross_cp sens_diff sens_percent);
	SET gesamt3;
	sens_diff=sens_ori-sensit_cross_cp;
	sens_percent=(sens_ori-sensit_cross_cp)/sens_ori;
RUN;

/*Spezifitt*/

DATA spez (KEEP=spez_ori spez_cross_cp spez_diff spez_percent);
	SET gesamt3;
	spez_diff=spez_ori-spez_cross_cp;
	spez_percent=(spez_ori-spez_cross_cp)/spez_ori;
RUN;

/*YoudenIndex*/

DATA youden (KEEP=maxyouden_ori youden_cross_cp youden_diff youden_percent);
	SET gesamt3;
	youden_diff=maxyouden_ori-youden_cross_cp;
	youden_percent=(maxyouden_ori-youden_cross_cp)/maxyouden_ori;
RUN;

/*PPV*/

DATA ppv (KEEP=ppv_ori ppv_cross_cp ppv_diff ppv_percent);
	SET gesamt3;
	ppv_diff=ppv_ori-ppv_cross_cp;
	ppv_percent=(ppv_ori-ppv_cross_cp)/ppv_ori;
RUN;

/*NPV*/

DATA npv (KEEP=npv_ori npv_cross_cp npv_diff npv_percent);
	SET gesamt3;
	npv_diff=npv_ori-npv_cross_cp;
	npv_percent=(npv_ori-npv_cross_cp)/npv_ori;
RUN;

/*PSEP*/

DATA psep (KEEP=psep_ori psep_cross_cp psep_diff psep_percent);
	SET gesamt3;
	psep_diff=psep_ori-psep_cross_cp;
	psep_percent=(psep_ori-psep_cross_cp)/psep_ori;
RUN;

/*Ergebnisse (gesamt3) ausgeben*/

PROC PRINT DATA= sens NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR sens_ori sensit_cross_cp sens_diff sens_percent; 
	TITLE1;
	TITLE2;
	TITLE3 'Cutpoint-abhngige Prognosegtemae';
	TITLE4 "(bei Cutpoint des Ausgangsmodells (cp=&prob_original):";
	LABEL 	sens_ori		='Sensitivitt O' 
			sensit_cross_cp	='Sensitivitt J'
			sens_diff 		='Sensitivitt D'
			sens_percent	='%';
RUN;

PROC PRINT DATA= spez NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR spez_ori spez_cross_cp spez_diff spez_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	spez_ori 		='Spezifitt O'
			spez_cross_cp 	='Spezifitt J'
			spez_diff 		='Spezifitt D'
			spez_percent	='%';
RUN;

PROC PRINT DATA= youden NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR maxyouden_ori youden_cross_cp youden_diff youden_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	maxyouden_ori 	='Max. YoudenIndex O'
			youden_cross_cp	='YoudenIndex J'
			youden_diff 	='YoudenIndex D'
			youden_percent	='%';
RUN;

PROC PRINT DATA= ppv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR ppv_ori ppv_cross_cp ppv_diff ppv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	ppv_ori			='Positive Korrektheit O'
			ppv_cross_cp 	='Positive Korrektheit J'
			ppv_diff 		='Positive Korrektheit D'
			ppv_percent		='%';
RUN;

PROC PRINT DATA= npv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  npv_ori npv_cross_cp npv_diff npv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	npv_ori 		='Negative Korrektheit O'
			npv_cross_cp 	='Negative Korrektheit J'
			npv_diff 		='Negative Korrektheit D'
			npv_percent		='%';
RUN;

PROC PRINT DATA= psep NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR psep_ori psep_cross_cp psep_diff psep_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	psep_ori		='Separation Index PSEP O'
			psep_cross_cp 	='Separation Index PSEP J'
			psep_diff 		='Separation Index PSEP D'
			psep_percent	='%';
RUN;


/*Ergebnisdatei gesamt2 fr schneren Output aufsplitten*/

/*Sensitivitt*/

DATA sens (KEEP=sens_ori sens_cross sens_diff sens_percent);
	SET gesamt2;
	sens_diff=sens_ori-sens_cross;
	sens_percent=(sens_ori-sens_cross)/sens_ori;
RUN;

/*Spezifitt*/

DATA spez (KEEP=spez_ori spez_cross spez_diff spez_percent);
	SET gesamt2;
	spez_diff=spez_ori-spez_cross;
	spez_percent=(spez_ori-spez_cross)/spez_ori;
RUN;

/*YoudenIndex*/

DATA youden (KEEP=maxyouden_ori maxyouden_cross maxyouden_diff maxyouden_percent);
	SET gesamt2;
	maxyouden_diff=maxyouden_ori-maxyouden_cross;
	maxyouden_percent=(maxyouden_ori-maxyouden_cross)/maxyouden_ori;
RUN;

/*PPV*/

DATA ppv (KEEP=ppv_ori ppv_cross ppv_diff ppv_percent);
	SET gesamt2;
	ppv_diff=ppv_ori-ppv_cross;
	ppv_percent=(ppv_ori-ppv_cross)/ppv_ori;
RUN;

/*NPV*/

DATA npv (KEEP=npv_ori npv_cross npv_diff npv_percent);
	SET gesamt2;
	npv_diff=npv_ori-npv_cross;
	npv_percent=(npv_ori-npv_cross)/npv_ori;
RUN;

/*PSEP*/

DATA psep (KEEP=psep_ori psep_cross psep_diff psep_percent);
	SET gesamt2;
	psep_diff=psep_ori-psep_cross;
	psep_percent=(psep_ori-psep_cross)/psep_ori;
RUN;

/*Ergebnisse (gesamt2) ausgeben*/

PROC PRINT DATA= sens NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR sens_ori sens_cross sens_diff sens_percent; 
	TITLE1;
	TITLE2;
	TITLE3 'Cutpoint-abhngige Prognosegtemae';
	TITLE4 "(bei neuem, optimalem Cutpoint (cp=&prob_cross):";
	LABEL 	sens_ori		='Sensitivitt O' 
			sens_cross		='Sensitivitt J'
			sens_diff 		='Sensitivitt D'
			sens_percent	='%';
RUN;

PROC PRINT DATA= spez NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR spez_ori spez_cross spez_diff spez_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	spez_ori 		='Spezifitt O'
			spez_cross	 	='Spezifitt J'
			spez_diff 		='Spezifitt D'
			spez_percent	='%';
RUN;

PROC PRINT DATA= youden NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR maxyouden_ori maxyouden_cross maxyouden_diff maxyouden_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	maxyouden_ori 		='Max. YoudenIndex O'
			maxyouden_cross		='YoudenIndex J'
			maxyouden_diff 		='YoudenIndex D'
			maxyouden_percent	='%';
RUN;

PROC PRINT DATA= ppv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR ppv_ori ppv_cross ppv_diff ppv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	ppv_ori			='Positive Korrektheit O'
			ppv_cross	 	='Positive Korrektheit J'
			ppv_diff 		='Positive Korrektheit D'
			ppv_percent		='%';
RUN;

PROC PRINT DATA= npv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  npv_ori npv_cross npv_diff npv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	npv_ori 		='Negative Korrektheit O'
			npv_cross	 	='Negative Korrektheit J'
			npv_diff 		='Negative Korrektheit D'
			npv_percent		='%';
RUN;

PROC PRINT DATA= psep NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR psep_ori psep_cross psep_diff psep_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	psep_ori		='Separation Index PSEP O'
			psep_cross		='Separation Index PSEP J'
			psep_diff 		='Separation Index PSEP D'
			psep_percent	='%';
RUN;

/*Temporre Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE youden tmp2 tmp spez som sens rsquare_max rsquare 
		   result_roc_ori1 result_roc_ori2 slope 
		   result_roc_cross1 result_roc_cross2 result_roc_cross3
		   result2 result psep ppv out_roc out_est npv maxresca index_ori
		   gof_cross_gesamt gof_cross gesamt3 gesamt1 gesamt2 
		   ergebnis emax brier auc association ergebnis2
		   ;
	RUN;
QUIT;

/*Ende der genauen Berechnung*/

%END;

%MEND JACKKNIFE_CROSSVALIDATION;
