/************************************************************************************/
/*																				  	*/
/*	NAME:		logreg2.mac.sas														*/
/*  TITLE: 		Multiple Logistische Regression										*/
/*  KEYS:		logistic regression, variables selection 							*/
/*	VERSION:	SAS 8.2																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		LOGISTIC															*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\logreg2.mac.sas						*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*	Programmierer:					Christoph Ziegler								*/
/*	Datum der Ersterstellung:		9/2002-8/2003									*/
/*  Aktuelle Makro- Version:		12/2003											*/
/*																					*/
/*	Kurzbeschreibung:				Allgemeines Makro zur multiplen logistischen	*/
/*							    	Regressionanalyse mit/ ohne Variablenselektion	*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_LOGREG.MAC.SAS								*/
/*																					*/
/*  Genauere Beschreibung siehe:	PM_LOGREG.MAC.SAS								*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*									dummy_friendly.mac.sas							*/
/*																					*/
/*	Bentigte Dateien:				Datensatz (Parameter 'data=')					*/
/*																					*/
/*																					*/
/*  Erzeugte Dateien:				work.out_pred		(falls Parameter pred  =1)	*/
/*									work.out_roc		(falls Parameter roc   =1)	*/
/*	(bei Standardeinstellung;		work.out_est		(falls Parameter est   =1)	*/
/*   Dateinamen frei whlbar)		work.association	(falls Parameter c_wert=1)	*/
/*									work.out_mi_log		(falls Parameter mi    =1)	*/
/*																					*/
/*																					*/
/*  Programmnderungen 				Sandra Mller, 02.05.2016, Makrovariable 		*/
/*  und -ergnzungen 				binwidth hinzugefgt und Grafikausgabe der 		*/
/*  (Name, Datum, Beschreibung):    ROC-Kurven unterbunden							*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%MACRO LOGREG2	 (  data			= ,
					resp_var		= ,
					event			= 1,
					xvar			= ,
					cvar			= ,
					ref				= ,
					fxvar			= ,
					fcvar			= ,
					miss			= 0,
					sel				= NONE,
					sel_details		= 1,
					slstay			= 0.05,
					slentry			= 0.05,
					clodds			= 0,
					alpha			= 0.05,
					rsquare			= 1,
					ctable			= 0,
					ctable_pprob	= ,
					pred			= 1,
					out_pred		= work.out_pred,
					predprobs_art	= INDIVIDUAL,
					roc				= 1,
					out_roc			= work.out_roc,
					est				= 1,
					out_est			= work.out_est,
					inest_flag		= 0,
					inest_data		= ,
					boot_data		= ,
					c_wert			= 1,
					c_data			= work.association,
					test			= ,
					mi				= 0,
					out_mi			= work.out_mi_log,
					macro_path		=,
					binwidth		= 0
				);

FILENAME y &macro_path;

%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('dummy_friendly.mac.sas');

%GLOBAL nnnn1;
%GLOBAL status_separation;

%IF &status_separation=0 %THEN %DO;

/*++++++++++++++++++++*/

/*Standardoutput ausgeben*/

%LET datensatz=&data;

/*Fallzahl bestimmen:*/

%IF &miss=1 %THEN %DO;

%NUMBEROBS(datensatz=&data);

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/


		DATA work.fallzahl;
		SET  &data;
		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

		RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              PM_LOGREG        ";
	TITLE6 "Analysierter Datensatz: &datensatz       ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE 

%IF &miss=0 %THEN %DO;

fallzahl 

%END;

standardoutput
;
RUN;
QUIT;

/*++++++++++++++++++++*/
%END;

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The cvar contains &count variables.;

/*Anzahl der Referenzen ermitteln*/

%GLOBAL count_ref var_ref;
%LET count_ref=1;
%LET var_ref=%QSCAN(&ref,&count_ref,*);
%DO %WHILE(&var_ref ne);
     %LET count_ref=%EVAL(&count_ref+1);
     %LET var_ref=%QSCAN(&ref,&count_ref,*);
%END;
%LET count_ref=%EVAL(&count_ref-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The refs contains &count_ref variables.;

/*Anzahl der fixen stetigen Variablen ermitteln*/

%GLOBAL count_fxvar var_fxvar;
%LET count_fxvar=1;
%LET var_fxvar=%QSCAN(&fxvar,&count_fxvar,%STR( ));
%DO %WHILE(&var_fxvar ne);
     %LET count_fxvar=%EVAL(&count_fxvar+1);
     %LET var_fxvar=%QSCAN(&fxvar,&count_fxvar,%STR( ));
%END;
%LET count_fxvar=%EVAL(&count_fxvar-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The fxvar contains &count_fxvar variables.;

/*Anzahl der fixen kategoriellen Variablen ermitteln*/

%GLOBAL count_fcvar var_fcvar;
%LET count_fcvar=1;
%LET var_fcvar=%QSCAN(&fcvar,&count_fcvar,%STR( ));
%DO %WHILE(&var_fcvar ne);
     %LET count_fcvar=%EVAL(&count_fcvar+1);
     %LET var_fcvar=%QSCAN(&fcvar,&count_fcvar,%STR( ));
%END;
%LET count_fcvar=%EVAL(&count_fcvar-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The fcvar contains &count_fcvar variables.;

/*Addition der Anzahl der fixen stetigen Variablen mit*/
/*der Anzahl der fixen kategoriellen Variablen fr INCLUDE- Option beim MODEL- Statement*/

%GLOBAL count_include;
%LET count_include=%EVAL(&count_fcvar+&count_fxvar);
/*Summe ausgeben*/
%PUT Summe: &count_include;

/*Anzahl der stetigen Variablen ermitteln:*/
%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The xvar contains &count_x variables.;

/*Addition der Anzahl der fixen stetigen Variablen mit*/
/*der Anzahl der fixen kategoriellen Variablen */

%GLOBAL count_include;
%LET count_include=%EVAL(&count_fcvar+&count_fxvar);
/*Summe im LOG- Fenster ausgeben*/
%PUT Summe: &count_include;

/*Elimination aller missing values falls miss=0 angegeben wurde*/

%IF &miss=0 %THEN
	%DO;

		DATA work.nomiss;
		SET  &data;
		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

%LET data=nomiss; 
RUN;
%END;

/*Falls TEST- Statement erwnscht: Fhre nur dieses aus!*/

%IF &test = %THEN %DO;


/*Falls SELECTION = NONE angegeben wurde soll Output der ersten PROC LOGISTIC (Variablen-*/
/*selektion in diesem Fall NONE) nicht ausgegeben werden, da dieser mit dem OUTPUT der zweiten*/
/*PROC LOGISTIC bereinstimmt. */

%IF %UPCASE(&sel)=NONE %THEN 
	%DO;

ODS LISTING CLOSE;

%END;

%IF %UPCASE(&sel)=N %THEN 
	%DO;

ODS LISTING CLOSE;

%END;

%IF %UPCASE(&sel) ne N %THEN %DO;
%IF %UPCASE(&sel) ne NONE %THEN %DO;
%IF &sel_details=0 %THEN %DO;

ODS LISTING CLOSE;

ODS OUTPUT ModelInfo=ModelInfo;
ODS OUTPUT ResponseProfile=ResponseProfile;
ODS OUTPUT ModelBuildingSummary=ModelBuildingSummary;

%END;
%END;
%END;

/*Falls class- Variablen angegeben wurden:*/

%IF &count ne 0 %THEN %DO;

/*ODS Output fr TypeIII- und Class- Variablen*/

ODS OUTPUT ClassLevelInfo=ClassLevelInfo;
ODS OUTPUT Type3=TypeIII;

%END;

/*Falls keine class- Variablen angegeben wurden erhlt man die Variablennamen mittels*/
/*ODS aus einer anderen Tabelle*/

%ELSE %DO;

ODS OUTPUT ParameterEstimates=ParameterEstimates;

%END;

%IF %UPCASE(&sel) ne NONE %THEN 
	%DO;

%IF %UPCASE(&sel) ne N %THEN 
	%DO;

/*Konvergenzstatus von PROC LOGISTIC #1 berprfen*/

ODS OUTPUT ConvergenceStatus=cs;

%END;
%END;




/*Start von Proc Logistic auf die angegebenen Daten*/

PROC LOGISTIC DATA=

/*Wahl der Datenquelle. Entweder Original- Datensatz oder Bootstrap- Datensatz*/

%IF &boot_data= %THEN
	%DO;
		&data
%END;

%ELSE %DO;
		&boot_data
%END;

/*Falls Ausgabe ber OUTEST erwnscht -->work.out_boot*/

%IF &inest_flag=1 %THEN
	%DO;
		OUTEST=work.out_boot
%END;

PLOTS=NONE			/* unterdrckt Ausgabe von ROC-Kurven fr jeden Selektionsschritt */

NAMELEN=32;

TITLE "1. PROC LOGISTIC, Variablenselektion (SELECTION=&sel)";

/*Falls class- Variablen angegeben wurden:*/

%IF &count ne 0 %THEN %DO;

/*   PROC LOGISTIC CLASS- Statement ohne dazugehrigen Referenzkategorien, falls keine */
/*   angegeben wurden*/

%IF &count_ref=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &count;
		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS
	%DO i=1 %TO &count;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;

%END;

/*Elimination der fixen stetigen Variablen aus xvar fr MODEL- Statement*/

%GLOBAL var_x;
%LET x='                                                               ';
%LET y='                                                               ';
%DO i=1 %TO &count_fxvar %BY 1;
	%DO j=1 %TO &count_x %BY 1;
		%LET x=%scan(&fxvar,&i," ");
		%LET y=%scan(&xvar,&j," ");

		%IF (%LOWCASE(&x) ne %LOWCASE(&y)) %THEN
		%DO;
			%LET var_x=&var_x &y;
		%END;
	%END;
	%LET xvar=&var_x;
	%LET var_x=;
%END;
%PUT &xvar;

/*Elimination der fixen kategoriellen Variablen aus cvar fr MODEL- Statement*/

%GLOBAL var_c;
%LET x='                                                               ';
%LET y='                                                               ';
%LET el_cvar=&cvar;
%DO i=1 %TO &count_fcvar %BY 1;
	%DO j=1 %TO &count %BY 1;
		%LET x=%scan(&fcvar,&i," ");
		%LET y=%scan(&el_cvar,&j," ");

        %IF (%LOWCASE(&x) ne %LOWCASE(&y)) %THEN
		%DO;
			%LET var_c=&var_c &y;
		%END;
	%END;
	%LET el_cvar=&var_c;
	%LET var_c=;
%END;
%PUT &el_cvar;

/*Modellgleichung*/

MODEL &resp_var(

%IF &event=1 %THEN
	%DO;
		event='1'
%END;

%IF &event=0 %THEN
	%DO;
		event='0'
%END;

)

= &fcvar &fxvar &el_cvar &xvar/ INCLUDE=&count_include BINWIDTH=&binwidth  /* BINWIDTH Korrektur */

/*Auswahl der Methode der Variablenselektion und Angabe der passenden Option(en)*/

%IF %UPCASE(&sel)=BACKWARD %THEN
	%DO;
		SELECTION=&sel SLSTAY=&slstay
%END;

%IF %UPCASE(&sel)=B %THEN
	%DO;
		SELECTION=&sel SLSTAY=&slstay
%END;


%IF %UPCASE(&sel)=FORWARD %THEN
	%DO;
		SELECTION=&sel SLENTRY=&slentry
%END;

%IF %UPCASE(&sel)=F %THEN
	%DO;
		SELECTION=&sel SLENTRY=&slentry
%END;


%IF %UPCASE(&sel)=STEPWISE %THEN
	%DO;

		SELECTION=&sel SLSTAY=&slstay SLENTRY=&slentry
%END;

%IF %UPCASE(&sel)=S %THEN
	%DO;

		SELECTION=&sel SLSTAY=&slstay SLENTRY=&slentry
%END;

/*Falls keine Selektionsmethode angegeben wurde, Verwendung des Standards*/

%IF %UPCASE(&sel)=NONE %THEN 
	%DO;

		SELECTION=&sel
%END;

%IF %UPCASE(&sel)=N %THEN 
	%DO;

		SELECTION=&sel
%END;

;
RUN;

/*ODS- Output schlieen*/

ODS OUTPUT close;

/*Umbenennung der neuen Variablen Effect in Variable*/

DATA TypeIII (RENAME=(Effect=Variable));
	SET TypeIII;
RUN;

%IF %UPCASE(&sel)=NONE %THEN 
	%DO;

ODS LISTING;

%END;

%IF %UPCASE(&sel)=N %THEN 
	%DO;

ODS LISTING;

%END;

%IF &sel_details=0 %THEN 
	%DO;

ODS LISTING;

%END;

%IF &count_ref>0 %THEN
     %DO;


/*Angegebene kategorielle Variablen und zugehrige Referenzkategorie in Tabelle schreiben*/

DATA cvar_refs;
	variable='                                                               ';
	ref_cat='                                                                ';
		%DO i=1 %TO &count_ref %BY 1; 
			variable="%scan(&cvar,&i," ")";
			ref_cat="%scan(&ref,&i,*)";
			output cvar_refs;
		%END;
RUN;

%END;

/*Ermittlung der Variablen, die im finalen Modell enthalten sind und Ergnzung durch */
/*Obs- Nummer*/

%IF &count = 0 %THEN %DO;

DATA variables (KEEP=variable obs); 
	SET ParameterEstimates;
	obs=_N_;
	IF variable='Intercept' THEN DELETE;
RUN;

%END;
%ELSE %DO;

DATA variables (KEEP=variable obs); 
	SET typeIII;
	obs=_N_;
RUN;

/*Ermittlung der Class- Variablen mit Leerzeilen- Elimination*/

DATA class1 (KEEP=class); 
	SET ClassLevelInfo;
	IF class='' THEN DELETE;
RUN;

/*Ergnzung durch Obs- Nummer und Umbenennung der Class- Spalte fr spteren Vergleich */

DATA class (KEEP=variable obs);
	SET class1;
	obs=_N_;
	RENAME class=variable;
RUN;

%END;

/*falls keine Variablen bei Selektion brig bleiben: Keine zweite PROC LOGISTIC mit finalem Modell!*/

%NUMBEROBS(datensatz=variables);

%GLOBAL selection_check;

%LET selection_check=&nnnn1;

%IF &selection_check=0 %THEN %DO;

%GOTO selection_check_false;

%END;

/*Speicherung aller Variablen des finalen Modells unter den Namen "names1" bis "namesn"*/

DATA names;
	SET variables;
	CALL SYMPUT('names'||left(_n_), variable);
RUN;

/*Ermittlung der Anzahl der finalen Variablen des Modells fr Schleife*/


%NUMBEROBS(datensatz=variables);

%LET nobs=&nnnn1;

/*Zusammenfassung aller finalen Variablen in der Variable "var" fr Model- Statement von*/
/*PROC LOGISTIC*/

%GLOBAL var names;
	%LET names=NAMES;
		%DO i=1 %TO &nobs;
			%LET var=&var &&names&i;
		%END;

/*Sortierung der Class- Variablen sowie aller finalen Variablen fr PROC COMPARE*/

%IF &count ne 0 %THEN %DO;

PROC SORT DATA=class;
	BY variable;
RUN;

%END;

PROC SORT DATA=variables;
	BY variable;
RUN;

%IF &count_ref>0 %THEN
     %DO;
/*Sortierung der Tabelle mit den angegebenen kategoriellen Variablen und den dazugehrigen*/
/*Referenzkategorien*/

PROC SORT DATA=cvar_refs;
	BY variable;
RUN;

%END;

%IF &count ne 0 %THEN %DO;

/*Vergleich um finale Class- Variablen zu ermitteln. Unterdrckung der Ausgabe im Output*/

PROC COMPARE OUT=outdif BASE=class COMPARE=variables NOPRINT;
	ID variable;
RUN;

/*Speicherung aller Class- Variablen des finalen Modells unter den Namen*/
/*"namesclass1" bis "namesclassn"*/

DATA namesclass;
	SET outdif;
	CALL SYMPUT('namesclass'||left(_n_), variable);
RUN;

/*Ermittlung der Anzahl der finalen Class-Variablen des Modells fr Schleife*/

/*%ANZOBS_class;*/
%NUMBEROBS(datensatz=outdif);
%LET nobs_class=&nnnn1;

/*Selektion der finalen Class- Variablen ohne dazugehrige passende Referenzkategorie*/
%IF &count_ref=0 %THEN
     %DO;

%GLOBAL var_class1;
%LET var_class1=;
%GLOBAL namesclass; 
%LET namesclass=;
%LET namesclass=NAMESCLASS;
%LET x='                                                               ';
%DO i=1 %TO &nobs_class %BY 1;
	%DO j=1 %TO &count %BY 1;
		%LET x=%scan(&cvar,&j," ");
	    %IF %LOWCASE(&&namesclass&i)=%LOWCASE(&x) %THEN
		%DO;
			%LET var_class1=&&namesclass&i &var_class1;
		%END;
	%END;
%END;

%END;

%ELSE %DO;

/*Selektion der finalen Class- Variablen und dazugehrige passende Referenzkategorie*/

%GLOBAL var_class1;
%LET var_class1=;
%GLOBAL var_ref1;
%LET var_ref1=;
%LET x='                                                               ';
%LET y='                                                               ';
%DO i=1 %TO &nobs_class %BY 1;
	%DO j=1 %TO &count %BY 1;
		%LET x=%scan(&cvar,&j," ");
		%LET y=%scan(&ref,&j,*);
	    %IF %LOWCASE(&&namesclass&i)=%LOWCASE(&x) %THEN
		%DO;
			%LET var_class1=&&namesclass&i &var_class1;
			%LET var_ref1=&y &var_ref1;
		%END;
	%END;
%END;

%END;

/*Zusammenfassung aller finalen class- Variablen in der Variable "var_class" fr*/
/*Class- Statement von PROC LOGISTIC*/

/*%VARIABLES_CLASS;*/

%GLOBAL var_class;
%LET var_class=;
%DO i=1 %TO &nobs_class;
	%LET var_class=&var_class &&namesclass&i;
%END;

%END;

/*-------------------------------------*/

%IF &sel_details=0 %THEN %DO;
%IF %UPCASE(&sel) ne N %THEN %DO;
%IF %UPCASE(&sel) ne NONE %THEN %DO;

PROC PRINT DATA=ModelInfo NOOBS LABEL HEADING=H WIDTH=UNIFORM;
    TITLE1 "1. PROC LOGISTIC, Variablenselektion (SELECTION=&sel)";
	TITLE2 "(Details unterdrckt)";
	TITLE3 "Model Information";
	TITLE4 ;
	TITLE5 ; 
	TITLE6 ;
	TITLE7 ;
	TITLE8 ;
	TITLE9 ;
	TITLE10;
	LABEL description	='---';
	LABEL value			='---';
RUN;

PROC PRINT DATA=ResponseProfile NOOBS LABEL HEADING=H WIDTH=UNIFORM;
    TITLE1 "Response Profile";
	TITLE2 ;
	TITLE3 ;
	TITLE4 ;
	TITLE5 ; 
	TITLE6 ;
	TITLE7 ;
	TITLE8 ;
	TITLE9 ;
	TITLE10;
	LABEL count='Total Frequency';
RUN;

PROC PRINT DATA=ModelBuildingSummary NOOBS LABEL HEADING=H WIDTH=UNIFORM;
    TITLE1 "Summary of &sel -variable selection process";
	TITLE2 ;
	TITLE3 ;
	TITLE4 ;
	TITLE5 ; 
	TITLE6 ;
	TITLE7 ;
	TITLE8 ;
	TITLE9 ;
	TITLE10;
RUN;

PROC DATASETS NOLIST;
	DELETE ResponseProfile ModelBuildingSummary ModelInfo;
RUN;
QUIT;

%END;
%END;
%END;
/*-------------------------------------*/

/*Falls c_wert=1, Ausgabe der c-Statistik in die unter c_data angegebene Datei*/

%IF &c_wert=1 %THEN %DO;
	ODS OUTPUT Association=&c_data;
%END;

%IF &rsquare=1 %THEN %DO;
	ODS OUTPUT rsquare=rsquare;
%END;

/*Falls out_mi erwnscht, TYPE III ausgeben*/

%IF &mi=1 %THEN %DO;

/*berprfen, ob CLASS- Variablen angegeben wurden*/

%IF &count ne 0 %THEN %DO;

	ODS OUTPUT Type3=&out_mi;

%END;
%ELSE %DO;
	
	ODS OUTPUT ParameterEstimates=&out_mi;

%END;
%END;

/*Konvergenzstatus berprfen*/

    ODS OUTPUT ConvergenceStatus=cs2;

/*Erneute Ausfhrung von PROC LOGISTIC mit finalem Modell um richtige Daten fr ROC- Kurve zu*/
/*erhalten.*/

PROC LOGISTIC DATA=&data

%IF &est=1 %THEN %DO;

		OUTEST=&out_est

%END;

%IF &inest_flag=1 %THEN
	%DO;
		INEST=work.out_boot
%END;

%IF &inest_flag=2 %THEN
	%DO;
		INEST=&inest_data
%END;

PLOTS=NONE   /* unterdrckt Ausgabe der ROC-Kurve */

NAMELEN=32;

%IF %UPCASE(&sel)=NONE %THEN 
	%DO;

TITLE "PROC LOGISTIC, ohne Variablenselektion (SELECTION=NONE)";

%END;
%ELSE %DO;

TITLE "2. PROC LOGISTIC mit finalem Modell";

%END;

%IF %UPCASE(&sel)=N %THEN 
	%DO;

TITLE "PROC LOGISTIC, ohne Variablenselektion (SELECTION=NONE)";

%END;
%ELSE %DO;

TITLE "2. PROC LOGISTIC mit finalem Modell";

%END;

%IF &count ne 0 %THEN %DO;

%IF &count_ref>0 %THEN
	%DO;

	CLASS
		%DO i=1 %TO &nobs_class;
  		%LET var_liste=%SCAN(&var_class1,&i,%STR( ));
  		%LET ref_liste=%SCAN(&var_ref1,&i,%STR( ));

    &var_liste (PARAM=REF REF=&ref_liste)

		%END;

	;
%END;
%ELSE %DO;

CLASS
		%DO i=1 %TO &nobs_class;
  		%LET var_liste=%SCAN(&var_class1,&i,%STR( ));

    &var_liste (PARAM=REF REF=FIRST)

		%END;

	;

%END;

%END;

    MODEL &resp_var (

%IF &event=1 %THEN
	%DO;
		event='1'
%END;

%IF &event=0 %THEN
	%DO;
		event='0'
%END;

)
= &var/ 

BINWIDTH=&binwidth  /* BINWIDTH Korrektur */
ALPHA=&alpha

%IF &clodds=1 %THEN
	%DO;
		CLODDS=PL 
%END;

%IF &rsquare=1 %THEN
	%DO;
		RSQUARE 
%END;

%IF &ctable=1 AND &ctable_pprob ne %THEN
	%DO;
		CTABLE PPROB=(&ctable_pprob)

%END;

%IF &roc=1 %THEN
	%DO;
		OUTROC=&out_roc ROCEPS=1E-16
/*ROCEPS gibt das Gruppierungskriterium fr geschtzte event WS-keiten an*/
%END;

/*MAXITER unterdrckt erneute Parameterschtzung*/

%IF &inest_flag=1 OR &inest_flag=2 %THEN
	%DO;
		MAXITER=0
%END;


;

%IF &pred=1 %THEN
	%DO;

		OUTPUT OUT=&out_pred PREDPROBS=&predprobs_art;
%END;
;

RUN;


ODS OUTPUT CLOSE;

/*Umbenennung der neuen Variablen Effect in Variable*/
%IF (&mi=1 && &count ne 0) %THEN %DO;			/*Nur wenn der Datensatz auch wirklich erstellt*/
	DATA &out_mi (RENAME=(Effect=Variable));    /*wurde (nach den angegebenen Bedingungen, soll*/
		SET &out_mi;							/*umbenannt werden*/
	RUN;
%END;


/*C-Wert/Somer's D aus dem Output selektieren*/

%IF &c_wert = 1 %THEN %DO;
	DATA &c_data (DROP=Label1 cValue1 nValue1);
		SET &c_data;
	RUN;
	DATA &c_data;
		SET &c_data;
		IF Label2='Gamma' THEN DELETE;
		IF Label2='Tau-a' THEN DELETE;
	RUN;

%IF &rsquare=1 %THEN %DO;
/*R-Quadrat- Werte hinzufgen*/

DATA rsquare_max (DROP=Label1 nValue1 cValue1);
	SET rsquare;
RUN;

DATA rsquare (DROP=Label2 nValue2 cValue2);
	SET rsquare;
RUN;

DATA rsquare;
	SET rsquare;
	RENAME Label1=Label2
		   nValue1=nValue2
		   cValue1=cValue2
;

DATA &c_data;
	SET &c_data rsquare_max rsquare;
RUN;

%END;
%ELSE %DO;

DATA rsquare_max;
	Label2='Max-resca';
RUN;

DATA rsquare;
	Label2='R-Square';
RUN;

DATA &c_data;
	SET &c_data rsquare_max rsquare;
RUN;

DATA &c_data;
	SET &c_data;
	IF Label2='Max-resca' THEN nValue2=.;
	IF Label2='Max-resca' THEN cValue2=.;
	IF Label2='R-Square'  THEN nValue2=.;
	IF Label2='R-Square'  THEN cValue2=.;
RUN;

%END;

%END;

/*p-Werte behalten*/

%IF &count ne 0 %THEN %DO;

DATA pwerte;
	SET typeiii;
RUN;

%END;
%ELSE %DO;

DATA pwerte;
	SET parameterestimates;
RUN;

%END;

/*Lschung von Zwischendateien und temporren Dateien*/

PROC DATASETS NOLIST;
	DELETE variables names ;

	%IF &count ne 0 %THEN %DO;

	DELETE typeiii outdif namesclass classlevelinfo class1 class;

	%END;
	%ELSE %DO;

	DELETE parameterestimates;

	%END;

	%IF &count_ref>0 %THEN 
      %DO;

    DELETE cvar_refs ;

    %END;

    %IF &miss=0 %THEN 
      %DO;

    DELETE nomiss ;

    %END;

	%IF &inest_flag=1 %THEN 
      %DO;

    DELETE out_boot ;

    %END;

	%IF &c_wert=1 %THEN
	  %DO;

	*DELETE rsquare_max rsquare;

	%END;

	RUN;
	QUIT;

/*--------------------------------------------------------------*/

/*Auf Basis der Angaben unter predprobsart= (c/i/x) die predicted values*/
/*in der generierten ROC- Tabelle anpassen! (diese sind stets 'INDIVIDUAL')*/

/*nur ausfhren, falls roc=1 & pred=1*/

%IF &roc=1 %THEN %DO;
%IF &pred=1 %THEN %DO;

/*falls predicted values=individual, dann ist die Anpassung nicht ntig (bereits korrekt)*/

%IF %UPCASE(&predprobs_art) ne I %THEN %DO;
%IF %UPCASE(&predprobs_art) ne INDIVIDUAL %THEN %DO;

/*Zielgre fr PROC LOGISTIC bestimmen*/

%IF %UPCASE(&predprobs_art)=X %THEN %DO;

	%LET predicted= XP_&event;

%END;

%IF %UPCASE(&predprobs_art)=XVALIDATE %THEN %DO;

	%LET predicted= XP_&event;

%END;

%IF %UPCASE(&predprobs_art)=CROSSVALIDATE %THEN %DO;

	%LET predicted= XP_&event;

%END;

%IF %UPCASE(&predprobs_art)=C %THEN %DO;

	%LET predicted= CP_&event;

%END;

%IF %UPCASE(&predprobs_art)=CUMULATIVE %THEN %DO;

	%LET predicted= CP_&event;

%END;

/*Zuerst: Regressionskoeffizienten bestimmen*/
/*Zielgre = entsprechende predicted values*/

PROC LOGISTIC DATA=&out_pred OUTEST=inest_correct_roc NOPRINT NAMELEN=32;

MODEL &resp_var(

%IF &event=1 %THEN
	%DO;
		event='1'
%END;

%IF &event=0 %THEN
	%DO;
		event='0'
%END;

)

= &predicted

;
RUN;

/*Outest- Datei bearbeiten*/

/*Regressionskoeffizienten abndern:*/
/*Steigung (Intercept)=0, Beta1=1*/

DATA inest_correct_roc (DROP=_LNLIKE_);
	SET inest_correct_roc;
	&predicted=1;
	Intercept=0;
	RENAME &predicted=logit_&predicted;
RUN;

/*Logit- Funktion (ln(p/1-p)) der Wahrscheinlichkeiten bestimmen*/

DATA predx;
	SET &out_pred;
	logit_&predicted=log(&predicted/(1-&predicted));
RUN;

/*korriegierte ROC- Daten herausschreiben*/

PROC LOGISTIC DATA=predx INEST=inest_correct_roc NOPRINT NAMELEN=32;

MODEL &resp_var(

%IF &event=1 %THEN
	%DO;
		event='1'
%END;

%IF &event=0 %THEN
	%DO;
		event='0'
%END;

)

= logit_&predicted / OUTROC=&out_roc ROCEPS=1E-16 MAXITER=0 BINWIDTH=&binwidth; /* BINWIDTH Korrektur */
RUN;

/*temporre zwischendateien lschen*/

PROC DATASETS NOLIST;
DELETE predx inest_correct_roc;
RUN;
QUIT;

%END;
%END;
%END;
%END;

/*--------------------------------------------------------------*/

/*Sprungmarke, falls bei 1.PROC LOGISTIC keine Variablen brig bleiben*/

%selection_check_false:

%IF &selection_check=0 %THEN %DO;

/*-------------------------------------*/

%IF &sel_details=0 %THEN %DO;
%IF %UPCASE(&sel) ne N %THEN %DO;
%IF %UPCASE(&sel) ne NONE %THEN %DO;

PROC PRINT DATA=ModelInfo NOOBS LABEL HEADING=H WIDTH=UNIFORM;
    TITLE1 "1. PROC LOGISTIC, Variablenselektion (SELECTION=&sel)";
	TITLE2 "(Details unterdrckt)";
	TITLE3 "Model Information";
	TITLE4 ;
	TITLE5 ; 
	TITLE6 ;
	TITLE7 ;
	TITLE8 ;
	TITLE9 ;
	TITLE10;
	LABEL description	='---';
	LABEL value			='---';
RUN;

PROC PRINT DATA=ResponseProfile NOOBS LABEL HEADING=H WIDTH=UNIFORM;
    TITLE1 "Response Profile";
	TITLE2 ;
	TITLE3 ;
	TITLE4 ;
	TITLE5 ; 
	TITLE6 ;
	TITLE7 ;
	TITLE8 ;
	TITLE9 ;
	TITLE10;
	LABEL count='Total Frequency';
RUN;

PROC PRINT DATA=ModelBuildingSummary NOOBS LABEL HEADING=H WIDTH=UNIFORM;
    TITLE1 "Summary of &sel -variable selection process";
	TITLE2 "NOTE: All effects have been removed from the model.";
	TITLE3 ;
	TITLE4 ;
	TITLE5 ; 
	TITLE6 ;
	TITLE7 ;
	TITLE8 ;
	TITLE9 ;
	TITLE10;
RUN;

PROC DATASETS NOLIST;
	DELETE ResponseProfile ModelBuildingSummary ModelInfo;
RUN;
QUIT;

%END;
%END;
%END;
/*-------------------------------------*/


%END;

/*(Quasi)- Separation von PROC LOGISTIC #1 (nur wenn sel nicht NONE) und #2 immer berprfen*/

%IF %UPCASE(&sel) ne NONE %THEN 
	%DO;

%IF %UPCASE(&sel) ne N %THEN 
	%DO;

%NUMBEROBS(datensatz=cs);

%LET cs_check=&nnnn1;

%END;
%END;

%IF &selection_check ne 0 %THEN %DO;

%NUMBEROBS(datensatz=cs2);

%LET cs2_check=&nnnn1;

%END;

%IF %UPCASE(&sel) ne NONE %THEN 
	%DO;
%IF %UPCASE(&sel) ne N %THEN 
	%DO;

DATA est_check;
	SET cs;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

%END;
%END;

%IF &selection_check ne 0 %THEN %DO;

DATA est_check2;
	SET cs2;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

%END;

%IF %UPCASE(&sel) ne NONE %THEN 
	%DO;

%IF %UPCASE(&sel) ne N %THEN 
	%DO;

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

%NUMBEROBS (datensatz=est_check);

%LET est_count=&nnnn1;

%END;
%END;

%IF &selection_check ne 0 %THEN %DO;

%NUMBEROBS (datensatz=est_check2);

%LET est_count2=&nnnn1;

%END;

PROC DATASETS NOLIST;
	DELETE 

	%IF %UPCASE(&sel) ne NONE %THEN 
		%DO;

	%IF %UPCASE(&sel) ne N %THEN 
		%DO;


		est_check cs 

	%END;

	%END;

	%IF &selection_check ne 0 %THEN %DO;

		est_check2  cs2

	%END;


;
RUN;
QUIT;

%GLOBAL status_separation;

%IF %UPCASE(&sel) ne NONE  %THEN 
	%DO;

%IF %UPCASE(&sel) ne N  %THEN 
	%DO;

%IF &est_count ne &cs_check %THEN %DO;

%LET status_separation=1;
%GOTO ende;

%END;
%ELSE %DO;

%LET status_separation=0;

%END;

%END;
%END;

%IF &selection_check ne 0 %THEN %DO;

%IF &est_count2 ne &cs2_check %THEN %DO;

%LET status_separation=1;

%END;
%ELSE %DO;

%LET status_separation=0;

%END;
%END;

%IF &selection_check = 0 %THEN %DO;

/*Lschung von Zwischendateien und temporren Dateien*/

PROC DATASETS NOLIST;
	DELETE variables ;

	%IF &count ne 0 %THEN %DO;

	DELETE typeiii classlevelinfo class1 class;

	%END;
	%ELSE %DO;

	DELETE parameterestimates;

	%END;

	%IF &count_ref>0 %THEN 
      %DO;

    DELETE cvar_refs ;

    %END;

    %IF &miss=0 %THEN 
      %DO;

    DELETE nomiss ;

    %END;


	RUN;
	QUIT;

%END;

%ende:


%END;
%ELSE %DO;

/*Implementierung des PROC LOGISTIC-Test- Statements*/

/*Falls kategorische Variablen angegeben wurden:*/
/*zuerst manuelle Dummy- Codierung, da TEST- Statement mit CLASS- Statement nicht funtioniert*/

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count_c varc;
%LET count_c=1;
%LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%DO %WHILE(&varc ne);
     %LET count_c=%EVAL(&count_c+1);
     %LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%END;
%LET count_c=%EVAL(&count_c-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The cvar contains &count_c variables.;

/*Dummy- Kodierung (nicht automatisch via CLASS- Statement bei PROC LOGISTIC*/

%LET all_variables=;

DATA ergebnis_ende (KEEP=&resp_var &cvar &xvar);
	SET &data;
RUN;

/*Referenzen zu jeweiliger Variable zuordnen*/

%DO m=1 %TO &count_c;
	%LET var=%SCAN(&cvar,&m,%STR( ));
	%LET var_ref=%SCAN(&ref,&m,*);

/*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

	PROC FREQ DATA=&data NOPRINT;
		TABLES &var /  SPARSE OUT=&var;
	RUN;

/*Ausgabedatensatz von PROC FREQ anpassen*/

	DATA &var (DROP=percent count);
		SET &var;
		IF &var=. THEN DELETE;
		IF count=0 THEN DELETE;
	RUN;

/*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/
	
	%NUMBEROBS(datensatz=&var);

	%LET count_var=&nnnn1;

		%LET underscore=_;

			/*Falls keine Referenzkategorie angegeben wurde, die erste nehmen*/
	
			%IF &count_ref=0 %THEN %DO;
				%LET var_ref=_FIRST_;
			%END;

			/*Referenzkategorie "FIRST" fr bergabe an Dummy- Makro anpassen ("_FIRST_")*/

			%IF &var_ref=FIRST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*Referenzkategorie "LAST" fr bergabe an Dummy- Makro anpassen ("_LAST_")*/

	        %IF &var_ref=LAST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*jeweilige Basiskategorie lschen*/

				DATA var2;
					SET &var end=_eof_;
					%IF 		&var_ref = _FIRST_ | &var_ref = LOW 
					%THEN 		%STR( IF _n_ = 1 THEN DELETE;);
					%ELSE %IF 	&var_ref = _LAST_ | &var_ref = HIGH
					%THEN 		%STR( IF _eof_ THEN DELETE;);
					%ELSE 		%STR(IF &var = &var_ref THEN DELETE;);
				RUN;

			/*Anzahl Kategorien der jeweiligen Variable ermitteln*/

				%NUMBEROBS(datensatz=var2);

			/*mgliche Ausprgungen pro Variable als Makro- Variable zwischenspeichern*/

			%LET count_var2=&nnnn1;

			DATA var2;
				SET var2;
				CALL SYMPUT('auspraeg'||left(_n_), trim(left(put(&var,BEST8.))));
			RUN;

/*Aufruf des Dummy- Kodierungs- Makros*/

	ODS LISTING CLOSE;

	%DUMMY( 
 	 		 data		=work.ergebnis_ende,   /* name of input dataset                  */
   			 out		=work.ergebnis_ende,   /* name of output dataset                 */
  			 var		= &var,         	   /* variable(s) to be dummied              */
  			 base		=&var_ref,     		   /* base category                          */
   			 prefix 	=,    				   /* prefix for dummy variable names        */
   			 format 	=,      			   /* format used to categorize variable     */
  			 name  		= VAL,  			   /* VAL: variable names are D_value        */
  			 fullrank	=1     				   /* Eliminate dummy for baseline category? */
   );

   ODS LISTING;


/*Falls fr eine Variable Dummies generiert wurden, Originalvariable lschen*/

	DATA ergebnis_ende (DROP=&var);
		SET ergebnis_ende;

		/*zustzliche Umbenennung (Anpassung an automatische Ausgabe der Dummy- Variablen*/
		/*von PROC LOGISTIC)*/

		%DO o=1 %TO &count_var2;

		RENAME &var._&&auspraeg&o = &var&&auspraeg&o;

		%END;
	RUN;


/*Zwischendatensatz lschen*/

	PROC DATASETS NOLIST;
		DELETE &var;
	RUN;
	QUIT;


%END;

/*Mittels PROC CONTENTS die Variablen, die fr PROC LOGISTIC selektieren*/
/*ohne Event (resp_var)*/

	DATA ergebnis_ende2 (DROP=&resp_var);
		SET ergebnis_ende;
	RUN;

	PROC CONTENTS DATA=ergebnis_ende2 NOPRINT OUT=ergebnis2 (KEEP=name);
	RUN;

	%NUMBEROBS(datensatz=ergebnis2);

	%LET count_ergebnis2=&nnnn1;

/*Variablennamen als Makro-Variablen speichern*/

	DATA ergebnis2;
		SET ergebnis2;
		CALL SYMPUT('y'||left(_N_), name);
	RUN;

/*Variablennamen fr PROC LOGISTIC zusammenfassen*/

	%DO s=1 %TO &count_ergebnis2;

	%LET all_variables=&all_variables &&y&s;
	%PUT &all_variables;
	%END;


/*Automatischer Output unterdrcken*/

ODS LISTING CLOSE;

/*(Quasi-) complete separation (Konvergenzstatus) auch hier berprfen*/

ODS OUTPUT ConvergenceStatus=cs3;

/*Daten von TEST- Statement mittels ODS ermitteln*/

ODS OUTPUT TestStmts=stmts;

PROC LOGISTIC DATA=ergebnis_ende

/*falls (quasi)- complete separation entdeckt wurde mit INEST- Statement arbeiten*/
/*---> bergabe der korrigierten Regressionskoeffizienten aus Macro FL*/

%IF &status_separation=1 %THEN %DO;

INEST=&inest_data

%END;

NAMELEN=32;
TITLE1;
TITLE2;
TITLE3;
TITLE4;
TITLE5;
TITLE6;
TITLE7;
TITLE8;
TITLE9;
TITLE10;
MODEL &resp_var (

%IF &event=1 %THEN
	%DO;
		event='1'
%END;

%IF &event=0 %THEN
	%DO;
		event='0'
%END;

				)
=&all_variables / SELECTION=NONE BINWIDTH=&binwidth; /* BINWIDTH Korrektur */

/*berprfen, wie viele Test- Statements ausgefhrt werden mssen*/

%GLOBAL count_test var_test;
%LET count_test=1;
%LET var_test=%QSCAN(&test,&count_test,TEST);
%DO %WHILE(&var_test ne);
     %LET count_test=%EVAL(&count_test+1);
     %LET var_test=%QSCAN(&test,&count_test,TEST);
%END;
%LET count_test=%EVAL(&count_test-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The test- Statement contains &count_test variables.;

/*Initialisierung des Ergebnis-Strings*/

%LET final_test_string=;

/*Schleife ber die Anzahl der Tests, die ausgefhrt werden sollen*/

%DO rt=1 %TO &count_test;

/*berprfen, wie viele Variablen pro Test- Statement vorkommen*/

%LET string_to_test=%QSCAN(&test,&rt,TEST);

%GLOBAL count_string var_string;
%LET count_string=1;
%LET var_string=%QSCAN(&string_to_test,&count_string,%STR( ));
%DO %WHILE(&var_string ne);
     %LET count_string=%EVAL(&count_string+1);
     %LET var_string=%QSCAN(&string_to_test,&count_string,%STR( ));
%END;
%LET count_string=%EVAL(&count_string-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The substring contains &count_string variables.;

/*Schleife ber die Variablen pro Test_Statement*/

%DO ld=1 %TO &count_string;

%LET var_to_test=%QSCAN(&string_to_test,&ld,%STR( ));

/*berprfen ob die jeweilige Variable stetig ist oder nicht (xvar)*/

	%LET xvar_ok=0;

	%DO uz=1 %TO &count_x;

		%LET xvar_check=%QSCAN(&xvar,&uz,%STR( ));

		%IF %UPCASE(&var_to_test)= %UPCASE(&xvar_check) %THEN %DO;

			%LET xvar_ok=1;

		%END; 
	
	%END;

    /*stetige Variable kann ohne weiteres an den Ergebnis- String angehngt werden*/
    /*Keine Dummies*/

	%IF &xvar_ok=1 %THEN %DO;

    /*Runde 1: Ohne = Zeichen*/

	%IF &ld=1 %THEN %DO;

	%LET final_test_string=&final_test_string &var_to_test;

	%END;
	%ELSE %DO;

    /*brige Runden: Mit = Zeichen*/

	%LET final_test_string=&final_test_string = &var_to_test;

	%END;


	%END;
	%ELSE %DO;

	/*es sind kategorielle Variablen enthalten, deshalb dummies anstatt normaler Variable fr*/
	/*Test- Statement nehmen*/

	%LET length_var_to_test=%LENGTH(&var_to_test);

	%LET check=0;

	%DO zr=1 %TO &count_ergebnis2;

	%LET all_var_scan=%QSCAN(&all_variables,&zr,%STR( ));

		%LET length_var_to_scan=%LENGTH(&all_var_scan);

		%IF &length_var_to_scan > &length_var_to_test %THEN %DO;

		
		%IF &var_to_test=%QSUBSTR(&all_var_scan,1,&length_var_to_test) %THEN %DO;

		%LET check=%EVAL(&check+1);
		%IF &check=1 AND &ld=1 %THEN %DO;

		%LET final_test_string=&final_test_string  &all_var_scan ;

		%END;
		%ELSE %DO;

		%LET final_test_string= &final_test_string = &all_var_scan;

		%END;

		%END;

		%END;

	%END;

	%END;

%END;

Test&rt: TEST &final_test_string=0;

%LET fortitle&rt=&final_test_string=0;

%LET final_test_string=;

%END;

RUN;

ODS OUTPUT CLOSE;

ODS LISTING;

DATA null;
	null='';
RUN;

/*Proc Logistic- Ausgabe um TypeIII zu erhalten (mit CLASS- Statement):*/

ODS OUTPUT ConvergenceStatus=cs4;

PROC LOGISTIC DATA=&data


/*falls (quasi)- complete separation entdeckt wurde mit INEST- Statement arbeiten*/
/*---> bergabe der korrigierten Regressionskoeffizienten aus Macro FL*/

%IF &status_separation=1 %THEN %DO;

INEST=&inest_data

%END;

NAMELEN=32;

/*Falls class- Variablen angegeben wurden:*/

%IF &count ne 0 %THEN %DO;

/*   PROC LOGISTIC CLASS- Statement ohne dazugehrigen Referenzkategorien, falls keine */
/*   angegeben wurden*/

%IF &count_ref=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &count;
		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS
	%DO i=1 %TO &count;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;

%END;
		MODEL &resp_var (

			%IF &event=1 %THEN
				%DO;
					event='1'
			%END;

			%IF &event=0 %THEN
				%DO;
					event='0'
			%END;

				)
=&xvar &cvar / SELECTION=NONE BINWIDTH=&binwidth; /* BINWIDTH Korrektur */
RUN;

ODS OUTPUT CLOSE;


/*Ergebnisse des/der Test(s) ausgeben*/

PROC PRINT DATA=stmts NOOBS LABEL;
	TITLE 'Linear Hypothesis/Hypotheses Testing Results';
	VAR label waldchisq df probchisq;
	LABEL label		='Label';
	LABEL waldchisq	='Wald Chi-Square';
	LABEL df		='DF';
	LABEL probchisq		='Pr > ChiSq';
RUN;

/*Ergebnisse des/ der Test(s) ausgeben*/

PROC PRINT DATA=null NOOBS LABEL;

	%DO rt=1 %TO &count_test;
	

    TITLE&rt "Test&rt: &&fortitle&rt";
		

	%END;
	VAR null;
	LABEL null ="---------------------------------------------";
RUN;

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis2 ergebnis_ende2 ergebnis_ende null stmts

	%IF &cvar ne %THEN %DO;

	var2 _cats_

	%END;

    ;
	%IF &miss=0 %THEN 
      %DO;

    DELETE nomiss ;

    %END;

	RUN;
QUIT;

%NUMBEROBS(datensatz=cs3);

%LET cs_check3=&nnnn1;

DATA est_check3;
	SET cs3;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

%NUMBEROBS (datensatz=est_check3);

%LET est_count3=&nnnn1;

%GLOBAL status_separation;


%IF &est_count3 ne &cs_check3 %THEN %DO;

%LET status_separation=1;

%END;
%ELSE %DO;

%LET status_separation=0;

%END;


%NUMBEROBS(datensatz=cs4);

%LET cs_check4=&nnnn1;

DATA est_check4;
	SET cs4;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

%NUMBEROBS (datensatz=est_check4);

%LET est_count4=&nnnn1;

%GLOBAL status_separation;


%IF &est_count4 ne &cs_check4 %THEN %DO;

%LET status_separation=1;

%END;
%ELSE %DO;

%LET status_separation=0;

%END;


/*Temporren Datenstze der Separation- berprfung lschen*/

PROC DATASETS NOLIST;
	DELETE cs3 est_check3 cs4 est_check4;
RUN;
QUIT;

/*Selection_check automatisch auf 0 setzen*/
/*(es fand keine Variablenselektion statt, es blieben somit keine Variablen brig)*/

%GLOBAL selection_check;
%LET selection_check=0;

%END;

%MEND LOGREG2;


