/************************************************************************************/
/*						        													*/
/*     NAME: pm_uni_logreg.mac.sas	         										*/
/*    TITLE: Univariate Logistische Regression mit univariater					    */
/*			 Betrachtung der stetigen Variablen mit Fractional Polynomials			*/
/*     KEYS: univariate logistic regression, fractional polynomials,	   		    */
/*			 (quasi-) complete separation, firth- procedure							*/
/*  VERSION: SAS 8.2												      			*/
/*  PRODUCT: BASE, STAT									        					*/
/*    PROCS: LOGISTIC											        			*/
/*															         				*/
/*																					*/
/*  Verzeichnis/Dateiname:			F:\Prognose\pm_uni_logreg.mac.sas	        	*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*  Programmierer:					Christoph Ziegler								*/
/*  Datum der Ersterstellung:		10/2002											*/
/*  Aktuelle Makro-Version:			12/2003											*/
/*																					*/
/*  Kurzbeschreibung:				Makro zur univariaten Logistischen Regression	*/
/*									sowie zur univariaten Betrachtung				*/
/*			  						der stetigen Variablen mit Fractional 			*/
/*									Polynomials										*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*									fl_heinze.mac.sas								*/
/*									dummy_friendly.mac.sas							*/
/*									fracpoly.mac.sas								*/
/*									exist.mac.sas									*/
/*									numchar.mac.sas									*/
/*									exist_var.mac.sas								*/
/*									ref.mac.sas										*/
/*									error1.sas	error2.sas	error3.sas	error4.sas	*/
/*									error5.sas	error6.sas	error7.sas	error8.sas	*/
/*									error24.sas error11.sas	error60.sas				*/
/*																					*/
/*  Bentigte Dateien:				Datensatz (Parameter 'data=')			   		*/
/*															           				*/
/*																					*/
/*  Erzeugte Dateien:				-----										 	*/
/*  									            								*/
/*  Programmnderungen																*/
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):											 		*/
/*  																				*/
/*        																			*/
/************************************************************************************/
/*

 TITLE
 -----
 pm_uni_logreg.mac.sas

 SAS Makro zur univariaten Logistischen Regression sowie zur univariaten Betrachtung
 der stetigen Variablen mit Fractional Polynomials.								


 BACKGROUND
 ----------
 Univariater Einfluss von Variablen
 Das Ziel bei der Entwicklung eines Prognosemodells ist es, ein multiples Modell 
 aufzustellen, welches zur Berechnung der Wahrscheinlichkeit fr das zu untersuchende 
 Ereignis genutzt werden kann. Ein Problem bei der Nutzung der multiplen logistischen 
 Regression ist unter anderem, dass fr die Datensituation oft zu viele Variablen 
 mit dem daraus resultierenden Problem des "Overfitting" vorhanden sind. In vielen 
 Forschungssituationen ist dies auch nach Auswahl der inhaltlich relevanten 
 Variablen noch der Fall. Deshalb kann in einem ersten Schritt der jeweilige univariate 
 Einfluss der Variablen untersucht werden. Der Begriff "univariat" bezeichnet dabei 
 jeweils die Untersuchung des Zusammenhangs je einer Einflussgre auf die Zielgre. 

 Diese Untersuchung hat zwei Ziele. Einmal knnen die Voraussetzungen fr den Einsatz 
 einer Variablen untersucht werden und zweitens kann dieses Vorgehen fr eine Vor-
 selektion der Variablen dienen, die spter in die multiple Regression eingesetzt werden. 
 Eine detaillierte Beschreibung der Vorgehensweise findet sich im Buch von Hosmer und 
 Lemeshow [hosmer2000] auf den Seiten 92 bis 98. Hier sollen anschlieend die wesent-
 lichen Aspekte kurz aufgezeigt und in den Bezug zur Umsetzung mit dem SAS-Makro 
 gebracht werden. Dabei wird oft gerade die Deskription der Variablen genutzt, die mit 
 dem Makro PM_DESCRIPTION.MAC.SAS (s. dort) erzeugt wird.   

 Modellvoraussetzungen 
 Es gibt einige Voraussetzungen, die die Beziehung zwischen der Einflussgre und 
 Zielgre im logistischen Modell stark beeinflussen. Die Untersuchung dieser  
 Voraussetzungen unterscheidet sich erheblich fr kategorielle/dichotome Variablen 
 von der der stetigen Variablen. Deshalb wird hier auf beide Situationen getrennt 
 eingegangen. 

 Kategorielle / dichotome Einflussgren
 Ein Problem fr die Modellierung besteht z.B. darin, dass bestimmte Merkmals-
 kombinationen zwischen Einfluss- und Zielgre gar nicht oder nur sehr selten 
 beobachtet worden sind. Diese Situation kann durch die Betrachtung der Hufigkeits-
 analysen des Makros fr die Deskription entdeckt werden (s. dort). Eine nicht 
 beobachtete Kombination fhrt zu Koeffizientenschtzungen, die Unendlich oder Null 
 sind. Meist kann durch geeignetes Zusammenlegen von Kategorien dies Problem behoben 
 werden. 
 Ein weiteres Problem ist die Modellierung der Odds ratios und Koeffizienten fr 
 kategorielle Variablen. Standardmig wird eine monotone Risikobeziehungen durch 
 das logistische Regressionsmodell modelliert, in die bei direkter Nutzung der Variablen 
 nur die Abstnde zwischen den Kategorien in die Berechnung eingehen. Dies ist meist nicht 
 sinnvoll, da die Risiken zwischen verschiedenen Kategorien nicht immer als gleich 
 vorausgesetzt werden knnen. Hier kommt erschwerend hinzu, dass meist die Ausprgungen 
 fr die Kategorien frei gewhlt werden knnen und demnach schon durch diese Wahl die 
 Regressionskoeffizienten und somit das Odds ratio verndert werden. Abhilfe schaffen 
 kann dann eine Dummy-Codierung, indem die einzelnen Risikosprnge einzeln modelliert 
 werden [cohen1991]. Allerdings erhht sich dadurch die Anzahl der Koeffizienten, die 
 zu schtzen sind. Die Anzahl Koeffizienten pro kategorieller Variable bei Dummy-Codierung 
 ergibt sich als die  Anzahl Ausprgungen - 1. Das bedeutet, dass wenn viele kategorielle 
 Variablen zu modellieren sind, die Anzahl Koeffizienten stark ansteigt. Deshalb ist es 
 sicher sinnvoll, an dieser Stelle durch Betrachtung der Anteile der Outcomevariablen fr 
 jede Merkmalsausprgung in der Hufigkeitstabelle nachzuschauen, ob nicht doch die 
 Annahme einer monotonen, linearen Risikofunktion gerechtfertigt ist. In der Regel sollten 
 die Variablen aber fr eine inhaltlich sinnvolle Modellierung kategorieller Variablen 
 durch entsprechende Dummy-Variablen ersetzt und in das Modell eingebracht werden. Zur 
 berprfung des Einflusses einer Einflussgre ist jetzt allerdings nicht der p-Wert der 
 einzelnen Dummy-Variablen sondern der p-Wert eines simultanen Tests mit der Hypothese, 
 dass alle Regressionskoeffizienten der zusammengehrigen Dummy-Variablen gleich Null sind, 
 zu bercksichtigen (Type-III-Analyse) (s. [saslogistic]).   

 Stetige Variablen (Monotonie / fractional polynomials)
 Bei "naivem" Einsetzen der Variablen in das logistische Regressionsmodell wird ein linearer
 und monotoner Einfluss modelliert. D.h. dass der Unterschied zwischen zwei Merkmals-
 ausprgungen relevant ist, aber nicht der Wert an sich (die Risikosteigerung hngt nur 
 von der Differenz zweier Ausprgungen, aber nicht von den Absolutwerten der Variablen-
 ausprgungen ab. Das ist nicht immer sinnvoll und muss deshalb untersucht bzw. 
 bercksichtigt werden. 
 Dies kann direkt anhand der Ausgabe des Makros PM_DESCRIPTION.MAC.SAS erfolgen, welches 
 die Risikobeziehung fr stetige Variablen u.a. anhand der Quintile darstellt. Sollte eine 
 Monotonie nicht vorrausgesetzt werden knnen bzw. sprechen inhaltliche Grnde dagegen, so 
 knnen stetige Variablen in der Regel entsprechend kategorisiert werden und als Dummy-
 Variablen (s.o.) in die Modellierung eingehen. Dabei ist die Wahl der Trennpunkte 
 problematisch und genau zu beachten. Einer eventuell besseren Modellierung der Risiko-
 beziehung steht dabei ein Informationsverlust durch diese Kategorisierung entgegen. 
 Ein weiterer Vorschlag zum Umgang mit diesem Problem ist die Modellierung nichtlinearer 
 Zusammenhnge im logistischen Regressionsmodells durch Transformationen der Einflussgren.
 Dabei hngt dann die Risikobeziehung von der Wahl dieses nichtlinearen Zusammenhanges ab. 
 Es gibt dafr sehr viele Mglichkeiten, die in der Praxis nicht alle untersucht werden 
 knnen. Roysten und Altman schlagen vor, eine Auswahl bestimmter Polynome, sogenannte 
 "fractional polynomials", auf ihre Modelltauglichkeit zu prfen [roysten1994]. Dabei 
 wird eine Klasse von 8 mglichen Transformationen vorgegeben und in Kombination geprft. 
 Fr fractional polynomials vom Grad m=1 sind dies diese 8 Modelle, die zu prfen sind, 
 fr den Grad m=2 werden die Transformationen kombiniert (z.B.: (0,2): log(X) + X2, wobei 
 im Falle eines doppelten Wertes (p,p) das Polynom Xp + Xplog(X) benutzt wird (36 Modelle). 
 Polynome vom Grad m>2 werden selten genutzt, da die Klasse der mglichen Funktionsformen 
 in der Regel ausreicht, die Risikobeziehung zwischen der Variablen und der Zielgre 
 adquat zu beschreiben. Geprft wird anschlieend anhand der Likelihoodwerte der Modelle, 
 ob das beste Modell mit Polynom zweiten Grades besser ist als das mit einem Polynom 
 ersten Grades bzw. dieses besser ist als die lineare Modellierung (p=1) [sauerbrei1999]. 
 Im Makro fr die Untersuchung des univariaten Einflusses der Einflussgren ist dies 
 Vorgehen implementiert. 

 Variablenselektion
 Zur Auswahl, welche Variablen nach berprfung der Modellierungsaspekte in das multiple 
 Regressionsmodell aufgenommen werden, kann der jeweilige p-Wert des Tests auf einen 
 signifikanten Einfluss dieser univariaten Betrachtung der Variablen genutzt werden 
 (p-Wert fr simultanen Einfluss aller Dummys bei kategoriellen Variablen). So knnen in 
 diesem Analyseschritt die Variablen aus der weiteren Betrachtung herausgenommen werden, 
 die zumindest univariat keinen wesentlichen Einfluss auf die Zielgre zeigen. 
 Da durch die Korrelationsstrukturen im Datensatz es dennoch dazu kommen kann, dass  
 eine Variable univariat keinen Einfluss zeigt aber in Kombination mit weiteren Variablen 
 relativ wichtig fr eine Prognose wird (s. [rothman1998] [sun1996]), sollte fr diese 
 Vorselektion der einflussreichen Variablen die Grenze fr die Selektion relativ hoch 
 gesetzt werden. Hosmer und Lemeshow geben in [hosmer2000] eine Grenze von 0.25 an, die 
 einen Kompromiss darstellt, eventuell einflussreiche Variablen erst einmal zu behalten 
 ohne zu viele Variablen mit sehr geringem Informationsinhalt im multiplen Modell zu 
 belassen. Bei Nutzung des in der Medizin blichen Grenzwertes von 0.05 in dieser 
 univariaten Betrachtung zeigt sich, dass oft wichtige Variablen nicht identifiziert 
 werden [hosmer2000], [mickey1989], [sun1996], so dass ein so niedriger Grenzwert in 
 der Regel nicht genutzt werden sollte. 
 Bei der Berechnung der p-Werte fr den univariaten Einfluss ist zu beachten, dass 
 dabei in jeder dieser Regressionen jeweils die Daten aller zur Verfgung stehenden 
 Beobachtungen genutzt werden. Somit basieren die p-Werte eventuell auf unterschiedlichen 
 Fallzahlen und sind nicht direkt vergleichbar. Deshalb sollte fr eine Complete Case 
 Analyse der Datensatz auf alle vollstndigen Beobachtungen reduziert werden. Im 
 SAS-Makro ist diese Mglichkeit durch die Angabe des Parameters miss= integriert. 
 Andernfalls beruhen die Auswertungen der univariaten und der multiplen Regression 
 auf unterschiedlichen Datenstzen.


 DESCRIPTION
 -----------

 Im Makro sind die angesprochenen Methoden der Untersuchung des univariaten Einflusses 
 der mglichen Einflussgren folgendermaen implementiert worden:
 - Jede Variable kann entweder linear (stetig) oder Dummy-codiert (kategoriell) in die 
   SAS Prozedur PROC LOGISTIC eingesetzt werden. Es wird dann der entsprechende p-Wert 
   aus der Type-III-Analyse ausgeben (fr kategorielle Variablen der simultane p-Wert).
 - Sollte wegen des Problems einer Datenseparation keine Konvergenz der Regressions-
   koeffizienten erreicht werden, wird die Firth-Korrektur mit dem SAS-Makro von Heinze 
   [heinze2002] durchgefhrt. Dabei wird eine PROC LOGISTIC gerechnet, wobei 
   anschliessend der Konvergenzstatus abgefragt werden kann. Falls dies eintritt, wird 
   der erzeugte Output gelscht und das FL-Makro (FL_HEINZE.MAC.SAS) gestartet. Die 
   darin erzeugten Koeffizienten werden dann PROC LOGISTIC mit INEST= und MAXITER=0 
   bergeben und eine weitere PROC LOGISTIC gestartet, die den gewnschten Output erzeugt. 
 - Fr die Durchfhrung einer Complete Case Analyse kann der Parameter miss= auf 0 
   gesetzt werden, so dass alle univariaten logistischen Regressionen auf demselben 
   Datensatz beruhen.
 - Die angegeben stetigen Variablen (xvar=) werden einzeln (univariat) mit fractional 
   polynomial Modellen (36 Modelle vom Grad 2 und 8 Modelle vom Grad 1) untersucht, ob 
   eine dieser Transformationsmglichkeiten univariat eine bessere Anpassung als die 
   lineare, direkte Nutzung der Variablen bringt. Die anhand der Devianz jeweils besten 
   Modelle vom Grad 2, Grad 1, linear und ohne Variable werden anschlieend verglichen. 
   Da die Differenz der Devianz chiquadrat verteilt ist, lt sich fr den jeweiligen 
   Vergleich ein entsprechender p-Wert angeben. Beim Vergleich beginnt man mit dem Modell 
   vom Grad 2 in Bezug auf das Modell vom Grad 1. Sollte sich das Modell vom Grad 2 als 
   besser herausstellen, sollte die Variable entsprechend transformiert und so in die 
   weitere Analyse einbezogen werden. Die weiteren Vergleich werden entsprechend durch-
   gefhrt. Im Makro werden alle Devianzen der 44 Modelle (der Gre nach sortiert) und 
   die entsprechenden p-Werte der Vergleichstests ausgegeben. Zustzlich wird der 
   simultane p-Wert der jeweiligen besten Transformationen bestimmt und ebenfalls 
   ausgegeben.  


 LITERATURE
 ----------
 - SAS Online Doc zu der Prozeduren PROC LOGISTIC
 - Cohen A (1991): Dummy variables in stepwise regression.  Am. Stat. 45: 226-228
 - Heinze G, Schemper M (2002): A solution to the problem of separation in logistic 
                     regression.  Statist. Med. 21: 2409-2419
 - Hosmer DW, Lemeshow S (2000): Applied logistic regression (2nd Ed.).  
                     John Wiley and Sons Inc., New York
 - Mickey J, Greenland S (1989): A study of the impact of confounderselection criteria 
                     on effect estimation.  Am. J. of Epidemiology 129: 125-137
 - Rothman KJ, Greenland S (1998): Modern Epidemiology, 3rd Edition.  
                     Lippincott-Raven, Philadelphia
 - Royston P, Altman D (1994): Regression using fractional polynomials of continuous 
                     covariates: parsimonious parametric modelling (with discussion).  
                     Applied Statistics 43: 429-467 
 - Sun GW, Shook TL, Kay GL (1996): Inappropriate use of bivariable analysis to screen 
                     risk factors for use in multivariable analysis.  
                     J. Clin. Epidemiol. 49: 907-916  
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
					 Prognosemodellen auf Basis der logistischen Regression. 
					 Diplomarbeit, FH Ulm 

 SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %PM_UNI_LOGREG (	
   *data			= ,				Datensatz
   *resp_var		= ,				Zielgre
	event			= 1,			Event- Kategorie der binren Zielgre (0/1)
   *xvar			= ,				stetige Variable(n)
   *cvar			= ,				kategorielle Variable(n)
	ref				= ,				Referenzkategorien fr kategorische Variablen
	miss			= 0,			Missing Values ausschliessen (ja/ nein)
   *macro_path		=				Pfad zur Makrosammlung
  );

  Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
  Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar.

   data:		    Parameter zur Angabe des zu analysierenden Datensatzes. Wurde 
					beispielsweise vor dem Aufruf des Makros ein Datensatz mit dem 
					Namen "dat300" angelegt, lautet der korrekte Aufruf: "data=dat300". 
					Zustzlich kann auch die dazugehrige SAS-Library bergeben werden 
					(Beispiel: "data=work.dat300"). Die bergabe dieses Parameters ist 
					zwingend erforderlich.
 
   resp_var:		"resp_var" steht fr "Response Variable", also fr die Zielgre. 
					Dieser bergabeparameter ist ebenfalls zwingend erforderlich. 
					Ferner wird eine binre Datenstruktur (0/1) der Zielgre bentigt. 
					Die Zielgre muss mit "0" und "1" kodiert sein. Sollte dies nicht 
					der Fall, ist eine vorherige Umkodierung erforderlich. Heit 
					beispielsweise die Zielgre "tod" lautet der Aufruf wie folgt: 
					"resp_var=tod".

   event:			Je nach dem welche Kategorie der binren Zielgre untersucht 
					werden soll, kann diese mit Hilfe dieses Parameters bergeben werden. 
					Soll beispielsweise die Zielgre "tod" analysiert werden und steht 
					"1" fr tot sowie "0" fr lebendig, lautet der korrekte Aufruf 
					"event=1". Dies ist zugleich die Standardeinstellung, da blicherweise
					"1" fr das Eintreten eines bestimmten Ereignisses steht.

   xvar:			Dieser Parameter ist zur Angabe der stetigen Variablen 
					(Einflussgren). Allgemein gesagt sollten hier alle Variablen 
					genannt werden, die nicht in das CLASS-Statement von PROC LOGISTIC 
					aufgenommen werden sollen, also beispielsweise auch quasi-stetige 
					Variablen. Werden mehrere stetige oder quasi-stetige Variablen 
					bergeben, so mssen diese durch ein Blank (Leerzeichen) getrennt 
					werden. Beispiel: "xvar=alter bmi".

   cvar:			"cvar" steht fr "categorical variables" und ist fr die bergabe 
					aller kategorischen sowie aller dichotomen Variablen verantwortlich. 
					Diese werden innerhalb der LOGISTIC-Prozedur in das CLASS-Statement 
					aufgenommen und werden automatisch Dummy-kodiert. Werden mehrere 
					kategorische Variablen bergeben, mssen diese getrennt durch ein 
					Leerzeichen (Blank) bergeben werden (Beispiel: "cvar=diagnose msnr").
					Wurden dem Parameter "cvar" dichotome Variablen bergeben, so werden 
					diese zwar in das CLASS-Statement von PROC LOGISTIC aufgenommen, 
					jedoch keine Dummy-Kodierung durchgefhrt, da dichotome Variablen nur 
					zwei Klassen besitzen.

   ref:				Mittels dieses Parameters, der fr "reference" steht, ist es 
					mglich, pro bergebener kategorischer Variable eine sogenannte 
					Referenzkategorie anzugeben. Werden keine Referenzkategorien 
					angegeben, wird der Standard bei jeder Variablen innerhalb des 
					CLASS-Statements von PROC LOGISTIC auf REF=FIRST gesetzt. Dies 
					bedeutet, dass die erste (sortierte) Kategorie jeder bergebenen 
					Variablen als Referenzkategorie gewhlt wird. Werden 
					Referenzkategorien bergeben, so muss deren Anzahl mit der Anzahl 
					der bergebenen kategorischen Variablen bereinstimmen. SAS bietet 
					standardmig die Kategorien FIRST und LAST an. Zustzlich ist es 
					mit Hilfe dieses Parameters mglich, beliebige Kategorien anzugeben. 
					Das Trennungszeichen ist der Stern (*). Eigens gewhlte 
					Referenzkategorien mssen zustzlich in Hochkommata eingeschlossen 
					werden. Die SAS-Standardkategorien FIRST / LAST nicht. Die ntige 
					Form lautet beispielsweise: "ref='1'*FIRST*'4'". Der dazugehrige 
					"cvar"-Parameter: "cvar=icdklass3 msnr_num beginn_b_num". Von der 
					bergebenen Variablen "icdklass" wird die Kategorie "1" als 
					Referenzkategorie gewhlt. Von der Variablen "msnr_num" die erste 
					(sortierte), unabhngig was diese fr einen Wert besitzt und von der 
					Variablen "beginn_b_num" die Kategorie "4". 

   miss:			Dieser Parameter legt fest, ob Beobachtungen mit fehlenden Werten 
					(missing values) in den unter "xvar", "cvar" und "resp_var" 
					bergebenen Variablen vor der Analyse aus dem Datensatz eliminiert 
					werden sollen (0) oder nicht (1). Der Standardwert dieses Parameters 
					ist 0 (Complete Case-Analyse). Hierbei ist zu beachten, dass fehlende 
					Werte im Datensatz durch einen Punkt (.) reprsentiert sein mssen.
					
   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den Pfad 
					zu dem Verzeichnis, wo die komplette Makrosammlung abgelegt ist,
					anzugeben. Der Pfad selber muss in Hochkommata eingeschlossen 
					sein. Beispiel: 'f:\makros'. 

  VALIDATION
  ----------

  Validiert durch: Stephanie Heiss
 				   University of Ulm 
                   Department of Biometry and Medical Documentation
                   89075 Ulm
  
  Validiert am:    25. November 2003

  Validierungsprogramm::      PM_UNI_LOGREG_VAL.SAS
  Validierungsdokumentation:  PM_UNI_LOGREG_VAL.DOC
   
  Validierungsvorgehen:
  - Das Testen der Makros wird durch eine von der Programmierung unabhngige 
    Person durchgefhrt.
  - Fr die Durchfhrung der Validierung wird auf einen in der Literatur 
    genutzten Datensatz zurckgegriffen:  LBW (Low Birth Weight), beschrieben
    in Hosmer, Lemeshow: Applied logistic regression [hosmer2000]
  - Der Datensatz wird fr verschiedene Testlufe verndert bzw. erweitert, 
    damit "die Testdaten neben den normalen Angaben auch Grenzwerte, ungltige
    Angaben, Fehlerbedingungen und Sonderflle beinhalten." [beiwanger2003]
  - Fehlerkontrolle:
    Hierbei werden nur die im LOG-Fenster auftretenden Fehler bzw. Warnungen 
    und offensichtlichen Fehler im OUTPUT betrachtet. Fr jeden Parameter des 
    Makros werden mehrere Durchlufe mit verschiedenen Wertangaben durchgefhrt, 
    wobei auch ungltige Werte angegeben werden, um das Verhalten des Makros
    bei fehlerhaften Aufrufen zu untersuchen. Gefundene Fehler werden 
    dokumentiert und dem Programmierer zur Verfgung gestellt. Nach der Korrektur 
    aller Fehler werden die Testlufe wiederholt, um sicherzustellen, dass 
    wirklich alle Fehler abgefangen wurden und dass keine neuen Fehler entstanden 
    sind.
  - Ergebniskontrolle:
    Um die Richtigkeit der Ergebnisse zu berprfen, werden soweit mglich, die
    Rechenbeispiele und bungsaufgaben aus Hosmer/Lemeshow durch vergleichbare
    Makroaufrufe nachgerechnet. Die Ergebnisse werden denen aus dem Begleitbuch 
    Hosmer/Lemeshow/Cook [hosmer2001] gegenbergestellt und direkt verglichen.
  - Alle Validierungen werden dokumentiert.

  Literatur: 
  - Beiwanger S (2003): Validierung von SAS-Makros. Diplomarbeit, FH Ulm
  - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. 
           J. Wiley & Sons, New York
  - Hosmer DW, Lemeshow S, Cook ED (2001): Solutions manual to accompany
           applied logistic regression 2nd. Ed. J. Wiley & Sons, New York



  AUTHOR
  ------
  R. Muche / C. Ring / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name.

----------------------------------------------------------------------------------*/

%MACRO PM_UNI_LOGREG (	data		= ,
					    resp_var	= ,
					    event		= 1,
					    xvar		= ,
					    cvar		= ,
						ref			= ,
				   		miss		= 0,
						macro_path	=
				 	);

%GLOBAL status_separation;
%GLOBAL var_separation;

/*Prfungen / Referenzierungen / Standardbelegungen*/

/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;

	%WINDOW error color=grey                                              
     #5  @28 'Fehler!' attr=highlight                                   
             color=blue    
     #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert"
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm welches berprft, ob dem Parameter 'macro_path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);

%LET infile=&macro_path;

%IF %bquote(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %bquote(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %bquote(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

/*falls die Syntax nicht stimmt, Anzeige eines Hinweisfensters*/

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey                
	 #5  @28 'Fehler!' attr=highlight                                                                                 
             color=blue                                                                                                                
     #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert" 
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"	 
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
	%DISPLAY error;

	/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das unter 'macro_path' angegebene Verzeichnis existiert*/

%LET dir=&macro_path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey                                                          
	 #5  @28 'Fehler!' attr=highlight                  
             color=blue    
	 #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';     
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Falls bestimmte Parameter ohne Wert bergeben wurden, diese mit Standard belegen*/

%IF &event= %THEN %DO;
	%LET event=1;
%END;

%IF &miss= %THEN %DO;
	%LET miss=0;
%END;

/*Externe Makros, die innerhalb des Makros 'PM_UNI_LOGREG.MAC.SAS'*/
/*verwendet werden (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('fl_heinze.mac.sas');
%INCLUDE y ('dummy_friendly.mac.sas');
%INCLUDE y ('fracpoly.mac.sas');
%INCLUDE y ('numchar.mac.sas');
%INCLUDE y ('exist.mac.sas');
%INCLUDE y ('exist_var.mac.sas');
%INCLUDE y ('ref.mac.sas');

/*globale Variable des Untermakros 'numberobs.mac.sas'*/

%GLOBAL nnnn1;

/*Test, ob bei bestimmten Parametern anstatt numerischen Werten*/
/*character- Werte bergeben wurden*/

/*berprfung erfolgt mit Hilfe des Untermakros 'numchar.mac.sas'*/

%NUMCHAR(parameterliste=&event &miss);

%IF &numcharok='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error8.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob der angegebene Datensatz (Parameter 'data=') auch wirklich existiert*/
/*(mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%EXIST(dsn=&data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error4.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Datenquelle (Parameter 'data=' UND response-Variable*/
/*(Zielgre / Parameter 'resp_var=') angegeben wurde*/

%IF &data= OR &resp_var= %THEN                                                                                   
	%DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

	%INCLUDE y ('error1.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob berhaupt Variablen (Parameter 'xvar=' / 'cvar=') angegeben wurden*/

%IF &xvar= and &cvar= %THEN                                                                                  
	%DO;

/*wenn keine Variablen angegeben wurden, Anzeige eines Fensters mit*/ 
/*Fehlermeldung/ Hinweis*/

	%INCLUDE y ('error2.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Prfung, ob alle bergebenen Variablen numerisch sind*/

/*Erzeugung eines temporren Zwischendatensatzes, welcher lediglich die*/
/*bergebenen Variablen enthlt*/

DATA test3;
	SET &data;
	KEEP &xvar &resp_var &cvar;
RUN;

/*mittels der Prozedur PROC CONTENTS kann der Typ einer Variablen bestimmt werden*/

/*Unterdrckung der Ausgabe von PROC CONTENTS*/

PROC CONTENTS DATA=test3 NOPRINT OUT=testcontent;
RUN;

/*lediglich die Variable 'type' der Ergebnisdatei behalten,*/
/*da nur diese von Interesse ist*/

DATA testcontent;
	SET testcontent;
	KEEP type;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=testcontent);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test4=&nnnn1;

/*Prfung ob Variablen vom Typ 'character' vorhanden sind*/

DATA test4;
	SET testcontent;
	IF type=2 THEN DELETE;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test4);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test5=&nnnn1;

%IF &test4 ne &test5 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error60.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE test4 testcontent test3;
RUN;
QUIT;

/*berprfung, ob alle bergebenen Variablen im bergebenen Datensatz*/
/*(Parameter 'data' vorhanden sind)*/

/*Zusammenfassung aller Variablen in einer neuen Makro- Variablen*/

%LET all_variables= &resp_var &xvar &cvar;

/*Prfung erfolgt mit Hilfe des Untermakros 'exist_var.mac.sas'*/

%EXIST_VAR(dsn=&data, var=&all_variables);

/*falls irgendwelche Variablen nicht im angegebenen Datensatz existieren,*/
/*Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error6.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob die Zielgre (Parameter 'resp_var')*/
/*2 Kategorien (Ausprgungen) besitzt*/

/*missing values ausschliessen*/

DATA freq2;
	SET &data;
	IF &resp_var ne .;
RUN;

/*Anzahl der Ausprgungen mit Hilfe der Prozedur PROC FREQ ermitteln*/

/*Ausgabe unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnis als ODS- Datei zur weiteren Verarbeitung ausgeben*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=freq2;
	TABLES &resp_var;
RUN;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;

/*ODS- Ausgabe wieder aktivieren*/

ODS LISTING;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen (= # Ausprgungen der Zielgre)*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq freq2;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

/*falls Zielgre mehr als zwei Ausprgungen besitz--> Fehlermeldung*/

%IF &event_check > 2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error5.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Parameter 'miss' 0 oder 1 ist*/

%IF &miss ne 1  %THEN %DO;
	%IF &miss ne 0  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'event' 0 oder 1 ist*/

%IF &event ne 1  %THEN %DO;
	%IF &event ne 0  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*Anzahl der angegebenen kategoriellen Variablen ermitteln:*/

%GLOBAL c_count c_var;
%LET c_count=1;
%LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%DO %WHILE(&c_var ne);
     %LET c_count=%EVAL(&c_count+1);
     %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%END;
%LET c_count=%EVAL(&c_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The cvar contains &c_count variables.;

/*Anzahl der Referenzen ermitteln*/

%GLOBAL ref_count var_ref;
%LET ref_count=1;
%LET var_ref=%QSCAN(&ref,&ref_count,*);
%DO %WHILE(&var_ref ne);
     %LET ref_count=%EVAL(&ref_count+1);
     %LET var_ref=%QSCAN(&ref,&ref_count,*);
%END;
%LET ref_count=%EVAL(&ref_count-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The refs contains &ref_count variables.;

%IF &ref_count >0 %THEN
	%DO;
		%IF &c_count ne &ref_count %THEN                                                                                  
			%DO;

/*falls Anzahl der kategoriellen Variablen grer Null ist und
/*nicht mit der Anzahl der angegebenen*/
/*Referenzkategorien bereinstimmt, Anzeige eines Fensters*/
/*mit Fehlermeldung/ Hinweis*/

			%INCLUDE y ('error3.sas');

/*Sprungmarke im Fehlerfall*/

			%GOTO ende;

   		%END;
%END;

/*berprfung, ob etwaige bergebene Referenzkategorien vorhanden sind bzw.*/
/*korrekt bergeben wurden*/

%IF &cvar ne %THEN %DO;
	%IF &ref_count ne 0 %THEN %DO;

	%REF(data=&data, resp_var=&resp_var, event=&event, cvar=&cvar, ref=&ref);

	%IF &correct='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error11.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;

%END;
%END;	

/*berprfen ob eine Variable nach Entfernung aller Beobachtungen mit missing*/
/*values nur noch eine Ausprgung besitzt*/

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values*/

DATA work.category_test;
	SET  &data;

	%DO i=1 %TO &count_x;
		%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
		IF &xvar_liste=. THEN DELETE;
	%END;

	%DO i=1 %TO &count;
		%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
		IF &cvar_liste=. THEN DELETE;
	%END;

		IF &resp_var=. THEN DELETE;

RUN;

/*Prfen ob 'resp_var' /'xvar'/ 'cvar' nur 1 Ausprgung besitzen*/

%LET category_test=0;
%LET category_test_fail=;

/*event*/

/*Anzahl der Ausprgungen mittels PROC FREQ bestimmen*/

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &resp_var;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

%IF &event_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &resp_var;

%END;

/*xvar*/

%IF &xvar ne %THEN %DO;

%DO uhgu=1 %TO &count_x;

%LET xvar_liste=%SCAN(&xvar,&uhgu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &xvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET xvar_check=&nnnn1;

%IF &xvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &xvar_liste;

%END;

%END;
%END;

/*cvar*/

%IF &cvar ne %THEN %DO;

%DO uhfu=1 %TO &count;

%LET cvar_liste=%SCAN(&cvar,&uhfu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &cvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET cvar_check=&nnnn1;

%IF &cvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &cvar_liste;

%END;

%END;
%END;

PROC DATASETS NOLIST;
	DELETE category_test;
RUN;
QUIT;

%IF &category_test=1 %THEN %DO;

	DATA category_test;
		var="&category_test_fail";
	RUN;

	PROC PRINT DATA=category_test NOOBS LABEL;
	TITLE1 "Folgende Variablen besitzen nach Entfernung aller";
	TITLE2 "missing values lediglich eine Ausprgung:";
		VAR var;
			LABEL var='Variable';
	RUN;

	PROC DATASETS NOLIST;
		DELETE category_test;
	RUN;
	QUIT;

	%INCLUDE y ('error24.sas');
	%GOTO ende;

%END;

/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/

/*-------------------------------------------------------------------------------*/

/*Standardoutput ausgeben*/

%LET datensatz=&data;

%IF &miss=1 %THEN %DO;

/*Fallzahl mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%numberobs(datensatz=&data);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/

DATA work.fallzahl;
	SET  &data;
		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

RUN;

%LET fallzahldata=fallzahl;

%numberobs(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

/*Makro-kopf ausgeben*/

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz       ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

/*temporr angelegte Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE standardoutput 

%IF &miss=0 %THEN %DO;

fallzahl

%END;

;
RUN;
QUIT;

/*ENDE Standardoutput*/

/*----------------------------------------------------------------------*/

/*Anzahl der angegebenen kategoriellen Variablen ermitteln:*/

%GLOBAL c_count c_var;
%LET c_count=1;
%LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%DO %WHILE(&c_var ne);
     %LET c_count=%EVAL(&c_count+1);
     %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%END;
%LET c_count=%EVAL(&c_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The cvar contains &c_count variables.;

/*Anzahl der angegebenen stetigen Variablen ermitteln:*/

%GLOBAL x_count x_var;
%LET x_count=1;
%LET x_var=%QSCAN(&xvar,&x_count,%STR( ));
%DO %WHILE(&x_var ne);
     %LET x_count=%EVAL(&x_count+1);
     %LET x_var=%QSCAN(&xvar,&x_count,%STR( ));
%END;
%LET x_count=%EVAL(&x_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The xvar contains &x_count variables.;


/*Anzahl der Referenzen ermitteln*/

%GLOBAL count_ref var_ref;
%LET count_ref=1;
%LET var_ref=%QSCAN(&ref,&count_ref,*);
%DO %WHILE(&var_ref ne);
     %LET count_ref=%EVAL(&count_ref+1);
     %LET var_ref=%QSCAN(&ref,&count_ref,*);
%END;
%LET count_ref=%EVAL(&count_ref-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The refs contains &count_ref variables.;

/*Elimination aller missing values des Datensatzes falls miss=0 angegeben wurde*/

%IF &miss=0 %THEN %DO;

DATA work.nomiss;
	SET  &data;
		%DO i=1 %TO &x_count;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &c_count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

%LET data=nomiss; 

RUN;

%END;

/*Datei "variables" initialisieren*/

DATA variables;
	SET _NULL_;
RUN;

/*Datei "resp_profile" initialisieren*/

DATA resp_profile;
	SET _NULL_;
RUN;

/*Falls kategorielle Variablen angegeben wurden Ausfhrung*/
/*von PROC LOGISTIC mit CLASS- STATEMENT*/

%IF &cvar ne %THEN %DO;

/*PROC LOGISTIC fr jede einzelne diskrete (kategorielle) Variable*/

%DO ip=1 %TO &c_count;
	%LET varxxx&ip=%SCAN(&cvar,&ip,%STR( ));

	/*ODS- Output fr TYPEIII und ResponseProfile*/

	ODS LISTING CLOSE;

	ODS OUTPUT Type3=TypeIII&ip;

	%IF &miss=1 %THEN
	%DO;

	ODS OUTPUT ResponseProfile=ResponseProfile&ip;

	%END;

	%IF &miss=0 %THEN
	%DO;

	ODS OUTPUT ResponseProfile=ResponseProfile;

	%END;

    /*(Quasi-) Separation prfen*/

	ODS OUTPUT ConvergenceStatus=cs;

	PROC LOGISTIC DATA=&data NAMELEN=32 OUTEST=fl_fail_est;

	%IF &count_ref=0 %THEN
	%DO;
		CLASS &&varxxx&ip (PARAM=REF REF=FIRST);
	%END;

	%ELSE
	%DO;

	/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

	%LET ref_liste=%SCAN(&ref,&ip,*);
		CLASS &&varxxx&ip (PARAM=REF REF=&ref_liste);
	%END;

	MODEL &resp_var

					%IF &event=1 %THEN
						%DO;
							(event='1')
						%END;

					%IF &event=0 %THEN
						%DO;
							(event='0')
						%END;

	=&&varxxx&ip;
	RUN;


/*ODS Output schliessen*/

	ODS OUTPUT close;


    ODS LISTING;

/*(Quasi)- Separation von PROC LOGISTIC  immer berprfen*/

%NUMBEROBS(datensatz=cs);

%LET cs_check=&nnnn1;

DATA est_check;
	SET cs;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

%NUMBEROBS (datensatz=est_check);

%LET est_count=&nnnn1;

PROC DATASETS NOLIST;
	DELETE est_check cs ;
RUN;
QUIT;

%IF &est_count ne &cs_check %THEN %DO;

%LET status_separation=1;
%LET var_separation=&var_separation &&varxxx&ip;

%END;
%ELSE %DO;

%LET status_separation=0;

%END;

/*Falls (quasi-) separation aufgetreten ist:*/
/*Korrektur:*/

%IF &status_separation=1 %THEN %DO;

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count_c varc;
%LET count_c=1;
%LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%DO %WHILE(&varc ne);
     %LET count_c=%EVAL(&count_c+1);
     %LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%END;
%LET count_c=%EVAL(&count_c-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The cvar contains &count_c variables.;

/*Dummy- Kodierung (nicht automatisch via CLASS- Statement bei PROC LOGISTIC*/
/*---> Heinze- Makro FL kennt keine kategorischen (klassierten) Variablen*/

%LET all_variables=;

DATA ergebnis_ende (KEEP=&resp_var &&varxxx&ip);
	SET &data;
RUN;

/*Referenzen zu jeweiliger Variable zuordnen*/

%LET var_ref=%SCAN(&ref,&ip,*);

/*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

PROC FREQ DATA=&data NOPRINT;
	TABLES &&varxxx&ip/  SPARSE OUT=&&varxxx&ip;
RUN;

/*Ausgabedatensatz von PROC FREQ anpassen*/

DATA &&varxxx&ip (DROP=percent count);
	SET &&varxxx&ip;
	IF &&varxxx&ip=. THEN DELETE;
	IF count=0 THEN DELETE;
RUN;

/*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/
	
%NUMBEROBS(datensatz=&&varxxx&ip);

	%LET count_var=&nnnn1;

		%LET underscore=_;

/*Falls keine Referenzkategorie angegeben wurde, die erste nehmen*/
	
			%IF &count_ref=0 %THEN %DO;
				%LET var_ref=_FIRST_;
			%END;

/*Referenzkategorie "FIRST" fr bergabe an Dummy- Makro anpassen ("_FIRST_")*/

			%IF &var_ref=FIRST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

/*Referenzkategorie "LAST" fr bergabe an Dummy- Makro anpassen ("_LAST_")*/

	        %IF &var_ref=LAST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*jeweilige Basiskategorie lschen*/

				DATA var2;
					SET &&varxxx&ip end=_eof_;
					%IF 		&var_ref = _FIRST_ | &var_ref = LOW 
					%THEN 		%STR( IF _n_ = 1 THEN DELETE;);
					%ELSE %IF 	&var_ref = _LAST_ | &var_ref = HIGH
					%THEN 		%STR( IF _eof_ THEN DELETE;);
					%ELSE 		%STR(IF &var = &var_ref THEN DELETE;);
				RUN;

/*Anzahl Kategorien der jeweiligen Variable ermitteln*/

				%NUMBEROBS(datensatz=var2);

/*mgliche Ausprgungen pro Variable als Makro- Variable zwischenspeichern*/

			%LET count_var2=&nnnn1;

			DATA var2;
				SET var2;
				CALL SYMPUT('auspraeg'||left(_n_), trim(left(put(&&varxxx&ip,BEST8.))));
			RUN;

/*Aufruf des Dummy- Kodierungs- Makros*/

ODS LISTING CLOSE;

%DUMMY( 
 	 	data	= work.ergebnis_ende,  /* name of input dataset                  */
   	    out		= work.ergebnis_ende,  /* name of output dataset                 */
  		var		= &&varxxx&ip,         /* variable(s) to be dummied              */
  		base	= &var_ref,     	   /* base category                          */
   		prefix 	= ,    				   /* prefix for dummy variable names        */
   		format 	= ,      			   /* format used to categorize variable     */
  		name  	= VAL,  			   /* VAL: variable names are D_value        */
  		fullrank= 1     			   /* Eliminate dummy for baseline category? */
     );

ODS LISTING;


/*Falls fr eine Variable Dummies generiert wurden, Originalvariable lschen*/

DATA ergebnis_ende (DROP=&&varxxx&ip);
	SET ergebnis_ende;

/*zustzliche Umbenennung (Anpassung an automatische Ausgabe*/
/* der Dummy- Variablen von PROC LOGISTIC)*/

		%DO o=1 %TO &count_var2;

		RENAME &&varxxx&ip.._&&auspraeg&o = &&varxxx&ip&&auspraeg&o;

		%END;
RUN;


/*Zwischendatensatz lschen*/

PROC DATASETS NOLIST;
	DELETE &&varxxx&ip;
RUN;
QUIT;


/*Mittels PROC CONTENTS die Variablen, die fr das FL- Makro selektieren*/
/*ohne Event (resp_var)*/

DATA ergebnis_ende2 (DROP=&resp_var);
	SET ergebnis_ende;
RUN;

PROC CONTENTS DATA=ergebnis_ende2 NOPRINT OUT=ergebnis2 (KEEP=name);
RUN;

%NUMBEROBS(datensatz=ergebnis2);

%LET count_ergebnis2=&nnnn1;

/*Variablennamen als Makro-Variablen speichern*/

DATA ergebnis2;
	SET ergebnis2;
	CALL SYMPUT('y'||left(_N_), name);
RUN;

/*Variablennamen fr FL- Makro zusammenfassen*/

%DO s=1 %TO &count_ergebnis2;

	%LET all_variables=&all_variables &&y&s;
	%PUT &all_variables;

%END;

/*Aufruf des FL- Makros*/

%FL		(	data			= ergebnis_ende, 
			y				= &resp_var, 
			varlist			= &all_variables, 
			outest			= outest_fl, 
			print			= 0
        );

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis_ende _work _outtab _offset_ _maxby;
	DELETE _fby _estwot_ _cats_ var2 ergebnis_ende2 ergebnis2;
RUN;
QUIT;

/*Datei mit den Regressionskoeffizienten aus dem FL- Makro fr*/
/*INEST- Statement bei PROC LOGISTIC vorbereiten*/

DATA inest_fl (DROP=_rby_ _penlik_ _it_ _resp_ _noresp_ _by_ _total_ _code_);
	SET outest_fl;
	IF _code_ ne -1 THEN DELETE;
	RENAME INTERCEP=Intercept;
RUN;


/*Prfen, ob FL- Makro korrekt abgelaufen ist*/

%NUMBEROBS(datensatz=inest_fl);

%LET fl_fail_check=&nnnn1;

%IF &fl_fail_check=0 %THEN %DO;

%LET fl_false=1;

%END;

%ELSE %DO;

%LET fl_false=0;

%END;

/*neue Proc Logistic mit INEST- Statement*/

/*ODS- Output fr TYPEIII und ResponseProfile*/

	ODS LISTING CLOSE;

	ODS OUTPUT Type3=TypeIII&ip;

	%IF &miss=1 %THEN
	%DO;

	ODS OUTPUT ResponseProfile=ResponseProfile&ip;

	%END;

	%IF &miss=0 %THEN
	%DO;

	ODS OUTPUT ResponseProfile=ResponseProfile;

	%END;

	PROC LOGISTIC DATA=&data 


	%IF &fl_false=0 %THEN %DO;

	INEST=inest_fl 

	%END;
	%ELSE %DO;

	INEST=fl_fail_est

	%END;

	NAMELEN=32;

	%IF &count_ref=0 %THEN
	%DO;
		CLASS &&varxxx&ip (PARAM=REF REF=FIRST);
	%END;

	%ELSE
	%DO;

	/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

	%LET ref_liste=%SCAN(&ref,&ip,*);
		CLASS &&varxxx&i (PARAM=REF REF=&ref_liste);
	%END;

	MODEL &resp_var

					%IF &event=1 %THEN
						%DO;
							(event='1')
						%END;

					%IF &event=0 %THEN
						%DO;
							(event='0')
						%END;

	=&&varxxx&ip / MAXITER=0;
	RUN;

	/*ODS Output schliessen*/

	ODS OUTPUT close;
	ODS LISTING;

	PROC DATASETS NOLIST;
		DELETE inest_fl outest_fl;
	RUN;
	QUIT;

%END;

/*Umbenennung der neuen Variablen Effect in Variable*/

DATA TypeIII&ip (RENAME=(Effect=Variable));
	SET TypeIII&ip;
RUN;

%IF &miss=1 %THEN %DO;


DATA responseprofile&ip (KEEP=outcome count);
	SET responseprofile&ip;
RUN;

/*Transponieren der "responseprofile"- Datei*/

PROC TRANSPOSE DATA=responseprofile&ip OUT=response_transpose&ip;
	ID outcome;
RUN;


/*Datensatz "variables" bei jedem Schleifendurchlauf um eine Variable mit*/
/*zugehrigem p-Wert ergnzen*/

DATA variables (KEEP=ProbChiSq variable); 
 	SET variables typeIII&ip;
RUN;

/*Ermittlung der Fallzahl auf der jeweilige Auswertung beruht*/

DATA resp_profile (KEEP=_0 _1);
	SET resp_profile response_transpose&ip;
RUN;

%END;

%IF &miss=0 %THEN %DO;

/*Datensatz "variables" bei jedem Schleifendurchlauf um eine Variable mit*/
/*zugehrigem p-Wert ergnzen*/

DATA variables (KEEP=ProbChiSq variable) ; 
 	SET variables typeIII&ip;
RUN;


%END;

/*Alle Zwischendatenstze lschen*/

PROC DATASETS NOLIST;
	DELETE typeiii&ip 

	%IF &miss=1 %THEN %DO;

	responseprofile&ip response_transpose&ip

	%END;

	;
RUN;
QUIT;

%END;

/*PROC LOGISTIC fr jede einzelne stetige Variable*/

%DO jp=1 %TO &x_count;
	%LET varxxx&jp=%SCAN(&xvar,&jp,%STR( ));

	/*ODS- Output fr ParameterEstimates*/

	ODS LISTING CLOSE;
	ODS OUTPUT ParameterEstimates=ParameterEstimates&jp;
	
	%IF &miss=1 %THEN
	%DO;

	ODS OUTPUT ResponseProfile=ResponseProfile&jp;

	%END;

	%IF &miss=0 %THEN
	%DO;

	ODS OUTPUT ResponseProfile=ResponseProfile;

	%END;

   /*(Quasi-)Separation berprfen*/

	ODS OUTPUT ConvergenceStatus=cs;

	PROC LOGISTIC DATA=&data NAMELEN=32 OUTEST=fl_fail_est;
	MODEL &resp_var

					%IF &event=1 %THEN
						%DO;
							(event='1')
						%END;

					%IF &event=0 %THEN
						%DO;
							(event='0')
						%END;

	=&&varxxx&jp;
	RUN;

	/*ODS Output schliessen*/

	ODS OUTPUT close;
	ODS LISTING;

/*(Quasi)- Separation von PROC LOGISTIC  immer berprfen*/

%NUMBEROBS(datensatz=cs);

%LET cs_check=&nnnn1;

DATA est_check;
	SET cs;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

%NUMBEROBS (datensatz=est_check);

%LET est_count=&nnnn1;

PROC DATASETS NOLIST;
	DELETE est_check cs ;
RUN;
QUIT;

%IF &est_count ne &cs_check %THEN %DO;

%LET status_separation=1;
%LET var_separation=&var_separation &&varxxx&jp;

%END;
%ELSE %DO;

%LET status_separation=0;

%END;

/*Falls (quasi-) separation aufgetreten ist:*/
/*Korrektur:*/

%IF &status_separation=1 %THEN %DO;

/*Kein Dummy-Codierung fr FL- Makro ntig ---> stetige Variablen*/

DATA ergebnis_ende (KEEP=&resp_var &&varxxx&jp);
	SET &data;
RUN;

/*Aufruf des FL- Makros*/

%FL		(	data			= ergebnis_ende, 
			y				= &resp_var, 
			varlist			= &&varxxx&jp, 
			outest			= outest_fl, 
			print			= 0
        );

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis_ende _work _outtab _offset_ _maxby;
	DELETE  _fby _estwot_ _cats_ var2 ergebnis_ende2 ergebnis2;
RUN;
QUIT;

/*Datei mit den Regressionskoeffizienten aus dem FL- Makro fr*/
/*INEST- Statement bei PROC LOGISTIC vorbereiten*/

DATA inest_fl (DROP=_rby_ _penlik_ _it_ _resp_ _noresp_ _by_ _total_ _code_);
	SET outest_fl;
	IF _code_ ne -1 THEN DELETE;
	RENAME INTERCEP=Intercept;
RUN;

/*Prfen, ob FL- Makro korrekt abgelaufen ist*/

%NUMBEROBS(datensatz=inest_fl);

%LET fl_fail_check=&nnnn1;

%IF &fl_fail_check=0 %THEN %DO;

%LET fl_false=1;

%END;

%ELSE %DO;

%LET fl_false=0;

%END;

/*neue Proc Logistic mit INEST- Statement*/

/*ODS- Output fr ParameterEstimates*/

	ODS LISTING CLOSE;
	ODS OUTPUT ParameterEstimates=ParameterEstimates&jp;
	
	%IF &miss=1 %THEN
	%DO;

	ODS OUTPUT ResponseProfile=ResponseProfile&jp;

	%END;

	%IF &miss=0 %THEN
	%DO;

	ODS OUTPUT ResponseProfile=ResponseProfile;

	%END;

	PROC LOGISTIC DATA=&data 


	%IF &fl_false=0 %THEN %DO;

	INEST=inest_fl 

	%END;
	%ELSE %DO;

	INEST=fl_fail_est

	%END;

	NAMELEN=32;

	MODEL &resp_var

					%IF &event=1 %THEN
						%DO;
							(event='1')
						%END;

					%IF &event=0 %THEN
						%DO;
							(event='0')
						%END;

	=&&varxxx&jp /MAXITER=0;
	RUN;

	/*ODS Output schliessen*/

	ODS OUTPUT close;
	ODS LISTING;

	PROC DATASETS NOLIST;
	DELETE inest_fl outest_fl;
	RUN;
	QUIT;


%END;

%IF &miss=1 %THEN %DO;

DATA responseprofile&jp (KEEP=outcome count);
	SET responseprofile&jp;
RUN;

PROC TRANSPOSE DATA=responseprofile&jp OUT=response_transpose&jp;
	ID outcome;
RUN;

/*Datensatz "variables" bei jedem Schleifendurchlauf um eine Variable mit*/
/*zugehrigem p-Wert ergnzen*/

DATA variables (KEEP=variable ProbChiSq); 
	variable='                                                               ';
 	SET variables ParameterEstimates&jp;
RUN;

/*Ermittlung der Fallzahl auf der jeweilige Auswertung beruht*/

DATA resp_profile (KEEP=_0 _1);
	SET resp_profile response_transpose&jp;
RUN;

%END;

%IF &miss=0 %THEN %DO;

/*Datensatz "variables" bei jedem Schleifendurchlauf um eine Variable mit*/
/*zugehrigem p-Wert ergnzen*/

DATA variables (KEEP=variable ProbChiSq) ; 
	variable='                                                               ';
	 SET variables ParameterEstimates&jp;
RUN;
	
%END;

/*Alle Zwischendatenstze lschen*/

PROC DATASETS NOLIST;
	DELETE ParameterEstimates&jp 

	%IF &miss=1 %THEN %DO;

	responseprofile&jp response_transpose&jp

	%END;

	;
RUN;
QUIT;

%END;
	
%END;

%ELSE %DO;

/*PROC LOGISTIC fr jede einzelne stetige Variable falls*/
/*nur stetige Variablen angegeben wurden (PROC LOGISTIC ohne CLASS- Statement!)*/

%DO jp=1 %TO &x_count;
	%LET varxxx&jp=%SCAN(&xvar,&jp,%STR( ));

	/*ODS- Output fr ParameterEstimates*/

	ODS LISTING CLOSE;
	ODS OUTPUT ParameterEstimates=ParameterEstimates&jp;
	
	%IF &miss=1 %THEN
	%DO;

	ODS OUTPUT ResponseProfile=ResponseProfile&jp;

	%END;

	%IF &miss=0 %THEN
	%DO;

	ODS OUTPUT ResponseProfile=ResponseProfile;

	%END;

   /*(Quasi-)Separation berprfen*/

	ODS OUTPUT ConvergenceStatus=cs;

	PROC LOGISTIC DATA=&data NAMELEN=32 OUTEST=fl_fail_est;
	MODEL &resp_var

					%IF &event=1 %THEN
						%DO;
							(event='1')
						%END;

					%IF &event=0 %THEN
						%DO;
							(event='0')
						%END;

	=&&varxxx&jp;
	RUN;

	/*ODS Output schliessen*/

	ODS OUTPUT close;
	ODS LISTING;

/*(Quasi)- Separation von PROC LOGISTIC  immer berprfen*/

%NUMBEROBS(datensatz=cs);

%LET cs_check=&nnnn1;

DATA est_check;
	SET cs;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

%NUMBEROBS (datensatz=est_check);

%LET est_count=&nnnn1;

PROC DATASETS NOLIST;
	DELETE est_check cs ;
RUN;
QUIT;

%IF &est_count ne &cs_check %THEN %DO;

%LET status_separation=1;
%LET var_separation=&var_separation &&varxxx&ip;

%END;
%ELSE %DO;

%LET status_separation=0;

%END;

/*Falls (quasi-) separation aufgetreten ist:*/
/*Korrektur:*/

%IF &status_separation=1 %THEN %DO;

/*Kein Dummy-Codierung fr FL- Makro ntig ---> stetige Variablen*/

DATA ergebnis_ende (KEEP=&resp_var &&varxxx&jp);
	SET &data;
RUN;

/*Aufruf des FL- Makros*/

%FL		(	data			= ergebnis_ende, 
			y				= &resp_var, 
			varlist			= &&varxxx&jp, 
			outest			= outest_fl, 
			print			= 0
        );

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis_ende _work _outtab _offset_ _maxby;
	DELETE _fby _estwot_ _cats_ var2 ergebnis_ende2 ergebnis2;
RUN;
QUIT;

/*Datei mit den Regressionskoeffizienten aus dem*/
/*FL- Makro fr INEST- Statement bei PROC LOGISTIC vorbereiten*/

DATA inest_fl (DROP=_rby_ _penlik_ _it_ _resp_ _noresp_ _by_ _total_ _code_);
	SET outest_fl;
	IF _code_ ne -1 THEN DELETE;
	RENAME INTERCEP=Intercept;
RUN;

/*Prfen, ob FL- Makro korrekt abgelaufen ist*/

%NUMBEROBS(datensatz=inest_fl);

%LET fl_fail_check=&nnnn1;

%IF &fl_fail_check=0 %THEN %DO;

%LET fl_false=1;

%END;

%ELSE %DO;

%LET fl_false=0;

%END;

/*neue Proc Logistic mit INEST- Statement*/


	/*ODS- Output fr ParameterEstimates*/

	ODS LISTING CLOSE;
	ODS OUTPUT ParameterEstimates=ParameterEstimates&jp;
	
	%IF &miss=1 %THEN
	%DO;

	ODS OUTPUT ResponseProfile=ResponseProfile&jp;

	%END;

	%IF &miss=0 %THEN
	%DO;

	ODS OUTPUT ResponseProfile=ResponseProfile;

	%END;

	PROC LOGISTIC DATA=&data 


	%IF &fl_false=0 %THEN %DO;

	INEST=inest_fl 

	%END;
	%ELSE %DO;

	INEST=fl_fail_est

	%END;

	NAMELEN=32;

	MODEL &resp_var

					%IF &event=1 %THEN
						%DO;
							(event='1')
						%END;

					%IF &event=0 %THEN
						%DO;
							(event='0')
						%END;

	=&&varxxx&jp /MAXITER=0;
	RUN;

	/*ODS Output schliessen*/

	ODS OUTPUT close;
	ODS LISTING;

	PROC DATASETS NOLIST;
	DELETE inest_fl outest_fl;
	RUN;
	QUIT;

%END;

%IF &miss=1 %THEN %DO;

DATA responseprofile&jp (KEEP=outcome count);
	SET responseprofile&jp;
RUN;

PROC TRANSPOSE DATA=responseprofile&jp OUT=response_transpose&jp;
	ID outcome;
RUN;

/*Datensatz "variables" bei jedem Schleifendurchlauf um eine Variable mit*/
/*zugehrigem p-Wert ergnzen*/

DATA variables (KEEP=variable ProbChiSq) ; 
	variable='                                                               ';
 	SET variables ParameterEstimates&jp;
RUN;

/*Ermittlung der Fallzahl auf der jeweilige Auswertung beruht*/

DATA resp_profile (KEEP=_0 _1);
	SET resp_profile response_transpose&jp;
RUN;

%END;

%IF &miss=0 %THEN %DO;

/*Datensatz "variables" bei jedem Schleifendurchlauf um eine Variable mit*/
/*zugehrigem p-Wert ergnzen*/

DATA variables (KEEP=variable ProbChiSq) ; 
	variable='                                                               ';
 	SET variables ParameterEstimates&jp;
RUN;

/*Alle Zwischendatenstze lschen*/

PROC DATASETS NOLIST;
	DELETE ParameterEstimates&jp response_transpose&jp responseprofile&jp;
RUN;
QUIT;

%END;



%END;
	
%END;

%IF &var_separation ne %THEN %DO;

/*Warnung ausgeben, dass Firth- Prozedur benutzt wurde*/

DATA note;
	note="";
RUN;

PROC PRINT DATA=note NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung (PROC LOGISTIC):               ";
	TITLE4 ;
	TITLE5 "There is possibly a quasi-complete separation of data"; 
	TITLE6 "points. The maximum likelihood estimate may not exist.";
	TITLE7 "Validity of the model fit is questionable.";
	TITLE8 ;
	TITLE9 'Es wurde eine automatische Korrektur mittels der ';
	TITLE10 "Firth- Prozedur (Makro 'FL_HEINZE.MAC.SAS') durchgefhrt!";

	
	LABEL note = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
RUN;

/*Anzahl der Elemente im String var_separation ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&var_separation,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&var_separation,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The var_separartion contains &count_x variables.;

DATA var_separation;
	variable='                                                                            ';
		%DO ik=1 %TO &count_x %BY 1; 
			variable="%scan(&var_separation,&ik," ")";
			output var_separation;
		%END;
RUN;

PROC PRINT DATA=var_separation NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  variable;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Bei folgenden Variablen trat die 'quasi-complete";
	TITLE4 "separation' auf:";
	TITLE5 ; 
	TITLE6 ;
	TITLE7 ;
	TITLE8 ;
	TITLE9 ;
	TITLE10 ;

	LABEL variable = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
RUN;

PROC DATASETS NOLIST;
	DELETE note var_separation;
RUN;
QUIT;

%IF &fl_false=1 %THEN %DO;

/*Hinweis ausgeben, dass Firth- Prozedur (Korrektur) fehlgeschlagen ist*/

DATA note2;
	note2="";
RUN;

PROC PRINT DATA=note2 NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note2;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung:               ";
	TITLE4 "Die Korrektur der Regressionskoeffizienten mittels der";
	TITLE5 "Firth- Prozedur (Makro 'FL_HEINZE.MAC.SAS') ist"; 
	TITLE6 "fehlgeschlagen! Evtl. wurden zu viele Koeffizienten in";
	TITLE7 "Bezug auf die Anzahl der Beobachtungen geschtzt. In der";
	TITLE8 "weiteren Analyse werden die Koeffizienten des letzten";
	TITLE9 "ML-Iterationsschrittes aus PROC LOGISTIC verwendet.";
	TITLE10 "Die Validitt dieses Modells ist fraglich!";
	
	LABEL note2 = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";

RUN;

/*temporre Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE note2;
RUN;
QUIT;

%END;

%END;

%LET var_separation=;

/*Ausgabe der Ergebnisse der univariaten logistischen Regression:*/

/*Finale Datei mit allen Variablen und dazugehrigem p-Wert ausgeben*/
	
%IF &miss=0 %THEN %DO;

DATA variables;
	SET variables;
	IF variable='Intercept' THEN DELETE;
RUN;

TITLE "Univariate Logistische Regression. Zielgre &resp_var . Ohne missing values";

PROC PRINT DATA=variables NOOBS;
RUN;

%END;

%IF &miss=1 %THEN %DO;

TITLE "Univariate Logistische Regression. Zielgre &resp_var . Mit missing values";

/*Variablen und zugehrige Fallzahl der jeweiligen Auswertung zusammenfhren*/

DATA variables;
	SET variables;
	IF variable='Intercept' THEN DELETE;
RUN;

DATA gesamt;
	MERGE variables resp_profile;
RUN;

PROC PRINT DATA=gesamt NOOBS;
RUN;

%END;

%IF &miss=0 %THEN %DO;

/*Ausgabe der Fallzahl auf der Auswertung beruht*/

TITLE "n (Fallzahl) auf der Auswertung beruht. Ohne missing values";
PROC PRINT DATA=responseprofile NOOBS;
RUN;

%END;

PROC DATASETS NOLIST;
DELETE  resp_profile variables fl_fail_est

%IF &miss=0 %THEN %DO;

nomiss responseprofile

%END;
%ELSE %DO;

gesamt 

%END;

;

RUN;
QUIT;

/*Fractional Polynomials als Schlu*/

%FRACPOLY (	data		= &datensatz,
			resp_var	= &resp_var,
			event		= &event,
		   	xvar		= &xvar,
			macro_path	= &macro_path	
		  );

/*Sprungmarke fr Fehlerfall*/

%ende:

%MEND PM_UNI_LOGREG;
