/************************************************************************************/
/*																				  	*/
/*	NAME:		printit.mac.sas														*/
/*  TITLE: 		Visualisierung von ROC- Daten										*/
/*  KEYS: 		roc- analysis, roc- curve, confidence limit, (max.) youden-index,	*/
/*			 	area under the curve (auc), sensitivity, specificity, cutpoint, 	*/
/*			    predicted probablities 								 				*/
/*	VERSION:	SAS 8.2																*/
/* 	PRODUCT:	BASE, STAT, GRAPH													*/
/*  PROCS:		GPLOT, GCHART														*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\printit.mac.sas						*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*	Programmierer:					Christoph Ziegler								*/
/*	Datum der Ersterstellung:		10/2002-8/2003									*/
/*  Aktuelle Makro- Version:		12/2003											*/
/*																					*/
/*	Kurzbeschreibung:				Allgemeines Makro zur Visualisierung von ROC-	*/
/*							    	Daten											*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_ROC.MAC.SAS									*/
/*																					*/
/*  Genauere Beschreibung siehe:	PM_ROC.MAC.SAS									*/
/*																					*/
/*  Verwendete externe Untermakros:	winkel.mac.sas									*/
/*																					*/
/*  Bentigte Dateien:				Datensatz 		  (Parameter 'data=')			*/
/*									ROC- Daten 		  (Parameter 'data_roc=')		*/
/*     								ROC- Daten		  (Parameter 'cut_youden_data=')*/
/*     								Predicted Values  (Parameter 'data_pred=')		*/
/*																					*/
/*																					*/
/*  Erzeugte Dateien:				'path'/gplot      (falls Parameter cibinom=1)	*/
/*									'path'/gplot1									*/
/*									'path'/gplot2	  (falls Parameter rocdat =1)	*/
/*  (Dateinamen nicht 				'path'/gplot3									*/
/*	 frei whlbar)					'path'/gchart									*/
/*																					*/
/*																					*/
/*  Programmnderungen 																*/
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):     Sandra Mller, 12.04.2016, Beschriftungen in   */
/*									 gplot1 und gplot3 gendert						*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%MACRO PRINTIT (
				data			=,
				resp_var		=,
				data_roc		=,
				data_pred		=,
				cibinom			=,
				rocdat			=,
				maxyouden		=,
				cutpoint		=,
				sensit			=,
				spez			=,
				outset1			=,
				outset2			=,
				alpha			=,
				cl				=,
				cut_youden_data	=,
				ext				=,
				winkel			=,
				point			=,
				path			=,
				macro_path  	=
               );

/*Zielverzeichnis fr Grafikdateien*/

FILENAME y &macro_path;
%INCLUDE y ('winkel.mac.sas');

FILENAME grafout &path;

/*Globale Optionen:*/

GOPTIONS DEVICE=&ext GSFNAME=grafout GSFMODE=replace FONTRES=PRESENTATION KEYMAP=NONE 
         FTEXT="Arial" HTITLE=1 HTEXT=1 XMAX=6in YMAX=6in XPIXELS=1800 YPIXELS=1800;

/*Falls Konfidenzintervalle mit CIBINOM berechnet wurden, Zeichnung einer ROC- Kurve mit */
/*Konfidenzintervall*/

%IF &cibinom=1 %THEN
	%DO;

/*Sortierung nach _1mspec_*/

PROC SORT DATA=&outset1 OUT=outset1;
	BY _1mspec_;
RUN;

/*Sortierung nach _1mspec_*/

PROC SORT DATA=&outset2 OUT=outset2;
	BY _1mspec_;
RUN;


DATA x2;
	SET outset2;
	exucl=1-exucl;
	exlcl=1-exlcl;
RUN;

DATA new1;
	KEEP exu1 exl1 spez obs;
	SET outset1;
	exu1=exucl;
	exl1=exlcl;
	spez=_1mspec_;
	obs=_n_;
RUN;

DATA new2;
	KEEP exu2 exl2 sens obs;
	SET x2;
	exu2=exucl;
	exl2=exlcl;
	sens=_sensit_;
	obs=_n_;
RUN;

DATA together;
	MERGE new1 new2;
	BY obs;
RUN;

%IF &winkel=1 %THEN
	%DO;

%WINKEL;

/*OBS-Nummer hinzufgen*/

DATA winkel2 (KEEP=x y obs); 
	SET winkel;
	obs=_N_;
RUN;

/*Merge ber obs- Nummer*/

DATA zusammen;
	MERGE together winkel2;
	BY obs;
RUN;

%END;

/*Ermittlung der Datenquelle: Mit oder ohne Winkelhalbierende*/

%GLOBAL datasource;
%IF &winkel=1 %THEN
	%DO;
		%LET datasource=zusammen;
%END;

%IF &winkel=0 %THEN
	%DO;
		%LET datasource=together;
%END;

/*Prozentzahl fr Grafik- berschrift*/

DATA xy;
	x=(1-&alpha)*100;
	CALL SYMPUT ('x',compbl(x));
RUN;

/*ROC- Kurve mit punktweisem Konfidenzintervall zeichnen*/

PROC GPLOT DATA=&datasource;

  symbol1 C=BLACK I=STEPCJ L=1 W=2 V=NONE;
  symbol2 C=RED   I=STEPCJ L=1 W=2 V=NONE;
  symbol3 C=BLACK I=JOIN   L=1 W=2 V=NONE;

PLOT    sens*spez=1     /* ESTIMATOR   */
        exu1*exu2=2     /* UPPER BOUND */
	    exl1*exl2=2	    /* LOWER BOUND */

%IF &winkel=1 %THEN
	%DO;
	   	 x*y=3 		    /*Winkelhalbierende*/
%END;

		

  / HAXIS=axis1
    VAXIS=axis2
	OVERLAY
    FRAME;


  TITLE1 H=1 "ROC- Kurve mit punktweisem &x % Konfidenzintervall. Datei: &data .Event=&resp_var." ;

  axis1 LABEL  = ('1-Spezifitt')
        ORDER  = (0 to 1 by 0.1)
		length = 13.1 cm;
  axis2 LABEL  = (a=90 'Sensitivitt')
        ORDER  = (0 to 1 by 0.1)
		length = 13.1 cm;
RUN;
QUIT;

%END;


%IF &cibinom=0 %THEN
	%DO;

	%IF &winkel=1 %THEN
	%DO;

	/*Daten fr Winkelhalbierende*/

	%WINKEL;

	/*OBS-Nummer hinzufgen*/

	DATA winkel2 (KEEP=x y obs); 
		SET winkel;
		obs=_N_;
	RUN;

	%END;

%END;

/*Normale ROC- Kurve ohne Konfidenzintervall zeichnen*/


%IF &winkel=0 %THEN
	%DO;

	DATA normalplot;
		SET &data_roc;
			sensi=&sensit;
	        spezi=1-&spez;
	RUN;
	   	 
%END;

%IF &winkel=1 %THEN
	%DO;

	DATA normalplot;
    	SET &data_roc;
		obs=_n_;
		/*Fr Label in der Grafik*/
	    sensi=&sensit;
	    spezi=1-&spez;
	RUN;

	DATA normalplot;
		MERGE normalplot winkel2;
		BY obs;
	RUN;

%END;


PROC GPLOT DATA=normalplot;

  symbol4 C=BLACK I=STEPCJ L=1 W=2 V=NONE;


  %IF &point=1 %THEN %DO;

  /*Fr Label in der Grafik*/

   symbol5 C=RED            L=1 W=1 V=CIRCLE; /* Schrift entfernt, da das Label mehrmals in der Graphik war */
 %END;

  symbol6 C=BLACK I=JOIN   L=1 W=2 V=NONE;

PLOT    _sensit_*_1mspec_=4     /* ESTIMATOR   */


%IF &point=1 %THEN %DO;

	sensi*spezi=5		/*Fr Label in der Grafik*/

%END;

%IF &winkel=1 %THEN
	%DO;
	   	 x*y=6 		    /*Winkelhalbierende*/
%END;

  / HAXIS=axis1
    VAXIS=axis2
	OVERLAY
    FRAME;

  %IF &point=1 %THEN %DO;

  TITLE1 H=1 "ROC- Kurve. Datei: &data .Event=&resp_var .Sens=&sensit / Spez=&spez bei max. Youdenindex &maxyouden" ;

  %END;
  %ELSE %DO;

  TITLE1 H=1 "ROC- Kurve. Datei: &data .Event=&resp_var." ;

  %END;	

  axis1 LABEL  = ('1-Spezifitt')
        ORDER  = (0 to 1 by 0.1)
		length = 13.1 cm;
  axis2 LABEL  = (a=90 'Sensitivitt')
        ORDER  = (0 to 1 by 0.1)
		length = 13.1 cm;
RUN;
QUIT;

%IF &rocdat=1 %THEN
	%DO;

%IF &winkel=1 %THEN
	%DO;


DATA plotdat2; 
	SET plotdat;
	obs=_N_;
RUN;

DATA zusammen2;
	MERGE plotdat2 winkel2;
	BY obs;
RUN;

%END;

/*Ermittlung der Datenquelle: Mit oder ohne Winkelhalbierende*/

%GLOBAL datasource;
%IF &winkel=1 %THEN
	%DO;
		%LET datasource=zusammen2;
%END;

%IF &winkel=0 %THEN
	%DO;
		%LET datasource=plotdat;
%END;

/*Prozentzahl fr berschrift*/
DATA xz;
	z=&cl*100;
	CALL SYMPUT ('z',compbl(z));
RUN;

/*ROC- Kurve mit Konfidenzband zeichnen*/

PROC GPLOT DATA=&datasource;

  symbol7 C=BLACK I=STEPCJ L=1 W=2 V=NONE;
  symbol8 C=RED   I=STEPCJ L=1 W=2 V=NONE;
  symbol9 C=BLACK I=JOIN   L=1 W=2 V=NONE;

  PLOT Sens05*Spec05=7  		/* ESTIMATOR   */
       SensPi*Spec1_Pi=8   	    /* UPPER BOUND */
	   Sens1_Pi*SpecPi=8		/* LOWER BOUND */

%IF &winkel=1 %THEN
	%DO;
	   	 x*y=9 					/*Winkelhalbierende*/
%END;

  / HAXIS=axis1
    VAXIS=axis2
	OVERLAY
    FRAME;
  
  TITLE1 H=1 "ROC- Kurve mit &z % Konfidenzband (Hilgers). Datei: &data .Event=&resp_var." ;

  axis1 LABEL  = ('1-Spezifitt')
        ORDER  = (0 TO 1 BY 0.1)
		length = 13.1 cm;
  axis2 LABEL  = (A=90 'Sensitivitt')
        ORDER  = (0 TO 1 BY 0.1)
		length = 13.1 cm;
RUN;
QUIT;

%END;

DATA youden_roc;
	SET &cut_youden_data;
	spez=(1-_1mspec_);
	youden=_sensit_ + spez -1;
	dummy=1;
	/*Fr Label in der Grafik*/
	maxyouden=&maxyouden;
	cutpoint=&cutpoint;
RUN;

/*Grafik, die Cutpoint in Abhngigkeit von Youdenindex zeigt zeichnen*/

PROC GPLOT DATA=youden_roc;

  symbol10 C=BLACK I=STEPCJ L=1 W=2 V=NONE;

  %IF &point=1 %THEN %DO;


  /*Fr Label in der Grafik*/
  symbol11 C=RED   I=STEPCJ L=1 W=1 V=CIRCLE;

  %END;

PLOT youden*_PROB_=10


%IF &point=1 %THEN %DO;

/*Fr Label in der Grafik*/
    maxyouden*cutpoint=11

%END;

  / HAXIS=axis1
    VAXIS=axis2
	OVERLAY

    FRAME;
  
  TITLE1 H=1 "Identifizierung des Cutpoints bei maximalem Youdenindex. Datei: &data .Event=&resp_var." ;
  TITLE2 H=1 "Maximaler Youdenindex: &maxyouden, Cutpoint: &cutpoint" ; /* Werte in berschrift eingefgt */

  axis1 LABEL  = ('Cutpoint')
        ORDER  = (0 TO 1 BY 0.1)
		length = 13.1 cm;
  axis2 LABEL  = (A=90 'Youdenindex')
        ORDER  = (0 TO 1 BY 0.1)
		length = 13.1 cm;
RUN;
QUIT;

/*GCHART ausgeben (Hufigkeit/predicted values)*/

/*manuelle Klassierung, um Lnge der CP- Anzeige in der Grafik zu bestimmen*/

DATA neu (KEEP= &predprobsart p_neu);
	SET &data_pred;
	IF &predprobsart>0    AND &predprobsart<=0.02 THEN p_neu=1;
	IF &predprobsart>0.02 AND &predprobsart<=0.04 THEN p_neu=2;
	IF &predprobsart>0.04 AND &predprobsart<=0.06 THEN p_neu=3;
	IF &predprobsart>0.06 AND &predprobsart<=0.08 THEN p_neu=4;
	IF &predprobsart>0.08 AND &predprobsart<=0.10 THEN p_neu=5;
    IF &predprobsart>0.10 AND &predprobsart<=0.12 THEN p_neu=6;
	IF &predprobsart>0.12 AND &predprobsart<=0.14 THEN p_neu=7;
	IF &predprobsart>0.14 AND &predprobsart<=0.16 THEN p_neu=8;
	IF &predprobsart>0.16 AND &predprobsart<=0.18 THEN p_neu=9;
	IF &predprobsart>0.18 AND &predprobsart<=0.20 THEN p_neu=10;
	IF &predprobsart>0.20 AND &predprobsart<=0.22 THEN p_neu=11;
	IF &predprobsart>0.22 AND &predprobsart<=0.24 THEN p_neu=12;
	IF &predprobsart>0.24 AND &predprobsart<=0.26 THEN p_neu=13;
	IF &predprobsart>0.26 AND &predprobsart<=0.28 THEN p_neu=14;
	IF &predprobsart>0.28 AND &predprobsart<=0.30 THEN p_neu=15;
	IF &predprobsart>0.30 AND &predprobsart<=0.32 THEN p_neu=16;
	IF &predprobsart>0.32 AND &predprobsart<=0.34 THEN p_neu=17;
	IF &predprobsart>0.34 AND &predprobsart<=0.36 THEN p_neu=18;
	IF &predprobsart>0.36 AND &predprobsart<=0.38 THEN p_neu=19;
	IF &predprobsart>0.38 AND &predprobsart<=0.40 THEN p_neu=20;
	IF &predprobsart>0.40 AND &predprobsart<=0.42 THEN p_neu=21;
	IF &predprobsart>0.42 AND &predprobsart<=0.44 THEN p_neu=22;
	IF &predprobsart>0.44 AND &predprobsart<=0.46 THEN p_neu=23;
	IF &predprobsart>0.46 AND &predprobsart<=0.48 THEN p_neu=24;
	IF &predprobsart>0.48 AND &predprobsart<=0.50 THEN p_neu=25;
	IF &predprobsart>0.50 AND &predprobsart<=0.52 THEN p_neu=26;
	IF &predprobsart>0.52 AND &predprobsart<=0.54 THEN p_neu=27;
	IF &predprobsart>0.54 AND &predprobsart<=0.56 THEN p_neu=28;
	IF &predprobsart>0.56 AND &predprobsart<=0.58 THEN p_neu=29;
	IF &predprobsart>0.58 AND &predprobsart<=0.60 THEN p_neu=30;
	IF &predprobsart>0.60 AND &predprobsart<=0.62 THEN p_neu=31;
	IF &predprobsart>0.62 AND &predprobsart<=0.64 THEN p_neu=32;
	IF &predprobsart>0.64 AND &predprobsart<=0.66 THEN p_neu=33;
	IF &predprobsart>0.66 AND &predprobsart<=0.68 THEN p_neu=34;
	IF &predprobsart>0.68 AND &predprobsart<=0.70 THEN p_neu=35;
	IF &predprobsart>0.70 AND &predprobsart<=0.72 THEN p_neu=36;
	IF &predprobsart>0.72 AND &predprobsart<=0.74 THEN p_neu=37;
	IF &predprobsart>0.74 AND &predprobsart<=0.76 THEN p_neu=38;
	IF &predprobsart>0.76 AND &predprobsart<=0.78 THEN p_neu=39;
	IF &predprobsart>0.78 AND &predprobsart<=0.80 THEN p_neu=40;
	IF &predprobsart>0.80 AND &predprobsart<=0.82 THEN p_neu=41;
	IF &predprobsart>0.82 AND &predprobsart<=0.84 THEN p_neu=42;
	IF &predprobsart>0.84 AND &predprobsart<=0.86 THEN p_neu=43;
	IF &predprobsart>0.86 AND &predprobsart<=0.88 THEN p_neu=44;
	IF &predprobsart>0.88 AND &predprobsart<=0.90 THEN p_neu=45;
	IF &predprobsart>0.90 AND &predprobsart<=0.92 THEN p_neu=46;
	IF &predprobsart>0.92 AND &predprobsart<=0.94 THEN p_neu=47;
	IF &predprobsart>0.94 AND &predprobsart<=0.96 THEN p_neu=48;
	IF &predprobsart>0.96 AND &predprobsart<=0.98 THEN p_neu=49;
	IF &predprobsart>0.98 	 		  THEN p_neu=50;
RUN;

/*Klasse mit den meisten Hufigkeiten bestimmen*/

ODS LISTING CLOSE;

ODS OUTPUT OneWayFreqs=freqs;

PROC FREQ DATA=neu;
TABLES p_neu;
RUN;

ODS OUTPUT CLOSE;
ODS LISTING;

PROC SORT DATA=freqs;
	BY DESCENDING frequency;
RUN;

/*als Makro- Variable speichern*/

DATA freqs2 (KEEP=frequency);
	SET freqs;
	IF _N_=1;
	CALL SYMPUT('max_frequency',frequency);
RUN;

%LET height=%SYSEVALF(&cutpoint*100);

data b;
	length color function $8 text $14;
	retain xsys '1' ysys '2' when 'a';
	function='move'; x=&height; y=0; output;
	function='draw'; x=&height; y=&max_frequency; color='black'; size=1; output;
	function='label'; x=&height; y=&max_frequency; size=1; position='2'; color='black'; text=' '; output;
run;

axis1 label=(justify=center 'Estimated Probability (Step=0.02)');
axis2 label=(angle=90 justify=center'Hufigkeit') ;

PROC GCHART DATA=&data_pred;
TITLE "Verteilung der Predicted Values nach Reklassifikation (Cutpoint= &cutpoint). Datei: &data";
	VBAR &predprobsart / midpoints=(0.01 to 0.99 by 0.02)  anno=b maxis=axis1  raxis=axis2 SUBGROUP=&resp_var;
RUN;
QUIT;

/*Temporre Zwischendateien lschen*/

PROC DATASETS NOLIST;
DELETE   youden_roc normalplot b freqs freqs2 neu;

	%IF &winkel=1 %THEN
	 %DO;

    DELETE winkel winkel2;

    %END;

	%IF &cibinom=1 %THEN
	 %DO;

    DELETE outset1 zusammen outset2 x2 new1 new2 together xy;

    %END;

	%IF &rocdat=1 %THEN
	 %DO;

    DELETE plotdat2 xz zusammen2;

    %END;


RUN;
QUIT;


%MEND PRINTIT;
