/************************************************************************************/
/*																				  	*/
/*	NAME:		ref.mac.sas															*/
/*  TITLE: 		Prfung ob Referenzkategorien von kategoriellen Variablen existieren*/
/*				beziehungsweise korrekt bergeben worden sind						*/
/*  KEYS: 		categorical variables, reference categories, logistic regression,	*/
/*				proc logistic-class-statement										*/
/*	VERSION:	SAS 8.2																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		LOGISTIC															*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\ref.mac.sas							*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*	Programmierer:					Christoph Ziegler								*/
/*	Datum der Ersterstellung:		8/2003											*/
/*  Aktuelle Makro- Version:		12/2003											*/
/*																					*/
/*	Kurzbeschreibung:				Allgemeines Makro zur Prfung ob 				*/
/*							    	Referenzkategorien von kategoriellen Variablen  */
/*									existieren beziehungsweise korrekt bergeben 	*/
/*									worden sind										*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_INFLUENCE.MAC.SAS							*/
/*									PM_LOGREG.MAC.SAS								*/
/*																					*/
/*  Verwendete externe Untermakros:	-----											*/
/*																					*/
/*  Bentigte Dateien:				Datensatz 		  (Parameter 'data=')			*/
/*																					*/
/*  Erzeugte Dateien:				-----											*/
/*																					*/
/*  Programmnderungen 																*/
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):     												*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%MACRO REF(data  	= ,
		   resp_var = ,
		   event	= ,
		   cvar  	= ,
		   ref   	=
    	  );

%GLOBAL correct;

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Mittels kurzer Proc Logistic die entsprechende Fehlermeldung generieren,*/
/*falls Referenzkategorien nicht korrekt sind:*/

PROC LOGISTIC DATA=&data NOPRINT NAMELEN=32;
CLASS

	%DO i=1 %TO &count;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;

MODEL &resp_var(

					%IF &event=1 %THEN
						%DO;
							event='1'
						%END;

					%IF &event=0 %THEN
						%DO;
							event='0'
						%END;

					)

	=&cvar ;
RUN;

%IF &syserr=3000 %THEN %LET correct='NO';
%ELSE %LET correct='YES';

%MEND REF;