/************************************************************************************/
/*																				  	*/
/*	NAME:		rocdat_hilgers.mac.sas												*/
/*  TITLE: 		Nichtparametrische Berechnung eines Konfidenzbandes			 		*/
/*  KEYS: 		roc analysis, roc curves, confidence limit						    */
/*	VERSION:	SAS ???																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		RANK, EXPAND														*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\rocdat_hilgers.mac.sas				*/
/*  Autoren:						R. Hilgers										*/
/*	Programmierer:					R. Hilgers										*/
/*	Datum der Ersterstellung:		-----											*/
/*  Aktuelle Makro- Version:		-----											*/
/*																					*/
/*  Kurzbeschreibung:				Makro zur nichtparametrischen Berechnung eines 	*/
/*									Konfidenzbandes									*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_ROC.MAC.SAS									*/
/*																					*/
/*  Genauere Beschreibung siehe:	Makrobeschreibung des Autors					*/
/*									R.A. Hilgers, Distribution-Free Confidence 		*/
/*									Bounds for ROC Curves							*/
/*  								(Meth Inform Med 1991; 30: 96-101)				*/
/*																					*/
/*  Verwendete externe Untermakros:	-----											*/
/*																					*/
/*	Bentigte Dateien:				Datensatz (Parameter 'daten=')					*/
/*																					*/
/*  Erzeugte Dateien:				work.plotdat work.rocnobs work.rocfn work.rocf0 */
/*									work.roc1fn  work.roc1f0  work.roc1f work.roc0fn*/
/*									work.roc0f0  work.roc0f   work.roc   work.roc0	*/
/*																					*/
/*  Programmnderungen 																*/
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):     												*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%macro rocdat;

 %let Pi=(sqrt(&CL)+1)/2;     /*  Fuer Grenzen wie im oben genannten
                                 Artikel */
 %let PiL=(sqrt((&CL+1)/2));
 %let PiU=(1-sqrt((1-&CL)/2));

DATA roc;                     /* Einlesen der Rohdaten */
 set &daten;
keep x gruppe;
RUN;

PROC MEANS DATA=roc NOPRINT N; /* Bestimmen der Anzahl der
                                  Stichprobenwerte */
 BY gruppe;
 VAR x;
 OUTPUT OUT=rocnobs n=nobs;
RUN;

DATA roc;                      /* Aufnahme der Stichprobenanzahl in die
                                  Datei der Werte */
 MERGE roc rocnobs;
 BY gruppe;
 KEEP x gruppe nobs;




PROC RANK DATA=roc TIES=LOW OUT=roc;  /* Vergeben von Raengen innerhalb
                                         der Gruppen (HIGH-, LOW- und
                                         MEAN-ranks) */
 RANKS lrank;
 VAR x;
 BY gruppe;

PROC RANK DATA=roc TIES=MEAN OUT=roc;
 RANKS mrank;
 VAR x;
 BY gruppe;

PROC RANK DATA=roc TIES=HIGH OUT=roc;
 RANKS hrank;
 VAR x;
 BY gruppe;


DATA roc0;
 SET roc;
 IF gruppe=0;

DATA roc1;
 SET roc;
 IF gruppe=1;
RUN;

%IF &model=I %THEN %DO;

  DATA roc0f;
   SET roc0;
     Spec1_Pi=betainv(1-&PiU,lrank,nobs+1-lrank); /* bzw. 1-&Pi */
     Spec05  =betainv(.5,mrank,nobs+1-mrank);
     SpecPi  =betainv(&PiL,hrank,nobs+1-hrank);   /* bzw. &Pi */
    drop lrank mrank hrank;

  DATA roc1f;
   SET roc1;
     Sens1_Pi=betainv(1-&PiL,lrank,nobs+1-lrank); /* bzw. 1-&Pi */
     Sens05  =betainv(.5,mrank,nobs+1-mrank);
     SensPi  =betainv(&PiU,hrank,nobs+1-hrank);   /* bzw. &Pi */
    drop lrank mrank hrank;

  DATA roc0f0;        /* definiere aeussere Punkte zwecks Interpolation */
   SET roc0f;
     IF _N_=1;
     Spec1_Pi=0;
     Spec05=0;
     SpecPi=SpecPi;

  DATA roc1f0;
   SET roc1f;
    IF _N_=1;
    Sens1_Pi=0;
    Sens05=0;
    SensPi=SensPi;

  DATA roc0fn;
   SET roc0f;
    IF _N_=nobs;
    Spec1_Pi=Spec1_Pi;
    Spec05=1;
    SpecPi=1;

   DATA roc1fn;
    SET roc1f;
    IF _N_=nobs;
    Sens1_Pi=Sens1_Pi;
    Sens05=1;
    SensPi=1;

%END;

%IF &model=II %THEN %DO;

  DATA roc0f;
   SET roc0;
     Spec1_Pi=1-betainv(&PiU,hrank,nobs+1-hrank);
     Spec05  =1-betainv(.5,mrank,nobs+1-mrank);
     SpecPi  =1-betainv(1-&PiL,lrank,nobs+1-lrank);
    drop lrank mrank hrank;

  DATA roc1f;
   SET roc1;
     Sens1_Pi=1-betainv(&PiL,hrank,nobs+1-hrank);
     Sens05  =1-betainv(.5,mrank,nobs+1-mrank);
     SensPi  =1-betainv(1-&PiU,lrank,nobs+1-lrank);
    drop lrank mrank hrank;

  DATA roc0f0;
   SET roc0f;
     IF _N_=1;
     Spec1_Pi=spec1_Pi;
     Spec05=1;
     SpecPi=1;

  DATA roc1f0;
   SET roc1f;
    IF _N_=1;
    Sens1_Pi=sens1_Pi;
    Sens05=1;
    SensPi=1;

  DATA roc0fn;
   SET roc0f;
    IF _N_=nobs;
    Spec1_Pi=0;
    Spec05=0;
    SpecPi=SpecPi;

   DATA roc1fn;
    SET roc1f;
    IF _N_=nobs;
    Sens1_Pi=0;
    Sens05=0;
    SensPi=SensPi;

%END;

DATA rocf0;
 MERGE roc0f0 roc1f0;
RUN;

DATA rocfn;
 MERGE roc0fn roc1fn;
RUN;

DATA roc;
 SET roc0f roc1f;

PROC SORT DATA=roc;
 BY x;
RUN;

DATA roc;
 SET rocf0 roc rocfn;
RUN;


PROC EXPAND DATA=roc OUT=plotdat;   /* Interpoliere fehlende Werte */
 CONVERT Spec1_Pi SpecPi Spec05
         Sens1_Pi SensPi Sens05 /METHOD=JOIN;
RUN;

%mend rocdat;

/* ROCdat.mac

  erzeugt Daten, mit deren Hilfe eine ROC-Kurve gezeichnet werden kann
  gem. R.A. Hilgers, Distribution-Free Confidence Bounds for ROC Curves
  (Meth Inform Med 1991; 30: 96-101).

  Das Niveau fuer obere und untere Grenze wird allerdings nicht wie im
  Artikel beschrieben symmetrisch gewaehlt, sondern folgendermassen:
  Die untere Grenze soll so liegen, dass mit W-keit>=beta=(1+CL)/2 sowohl
  Sensitivitaet als auch Spezifitaet besser liegen und mit W-keit<alpha=1-beta
  mind. ein Kriterium schlechter ist.
  Die obere Grenze soll so liegen, dass mit W-keit<alpha sowohl
  Sensitivitt als auch Spezifitt noch besser liegen und mit W-keit>=beta
  mind. ein Kriterium schlechter ist.
  Daraus ergeben sich die Werte PiU fr die obere Grenze und PiL fr
  die untere Grenze.

  Es muessen zwei Modelle unterschieden werden:
  I : Annahme, dass F_0 >= F_1 gilt, d.h. dass die Werte der Kranken
      niedriger sind als die der Gesunden
  II: Annahme, dass F_0 <= F_1 gilt, d.h. dass die Werte der Kranken
      hoeher sind als die der Gesunden

*/
